@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ClientPattern.cc 4199 2006-02-16 06:53:05Z mathias $
+// $Id: ClientPattern.cc 4831 2007-04-01 21:42:01Z markt $
 
 #include "ClientPattern.hh"
 #include "RegExp.hh"
@@ -35,8 +35,6 @@
 #define _GNU_SOURCE
 #endif // _GNU_SOURCE
 
-
-#include <iostream>
 #include <fstream>
 #include <string>
 #include <memory>
@@ -47,10 +45,10 @@
 #endif
 
 // needed as well for index on some systems (e.g. solaris)
-#include <strings.h> 
+#include <strings.h>
 
+using std::string;
 
-using namespace std;
 
 ClientPattern::ClientPattern():
     m_matchlimit(0),
@@ -63,13 +61,13 @@ ClientPattern::ClientPattern(const char *str):
 {
     /* A rough grammar of a pattern is:
        PATTERN ::= MATCH+ LIMIT?
-       MATCH ::= '(' word ')' 
+       MATCH ::= '(' word ')'
                  | '(' propertyname '=' word ')'
        LIMIT ::= '{' number '}'
-                 
+
        i.e. one or more match definitions, followed by
             an optional limit on the number of apps to match to
-       
+
        Match definitions are enclosed in parentheses, and if no
        property name is given, then CLASSNAME is assumed.
        If no limit is specified, no limit is applied (i.e. limit = infinity)
@@ -81,7 +79,7 @@ ClientPattern::ClientPattern(const char *str):
     string match;
     int err = 1; // for starting first loop
     while (had_error == 0 && err > 0) {
-        err = FbTk::StringUtil::getStringBetween(match, 
+        err = FbTk::StringUtil::getStringBetween(match,
                                                  str + pos,
                                                  '(', ')', " \t\n", true);
         if (err > 0) {
@@ -115,7 +113,7 @@ ClientPattern::ClientPattern(const char *str):
                 }
             }
             pos += err;
-        } 
+        }
     }
     if (pos == 0 && had_error == 0) {
         // no match terms given, this is not allowed
@@ -125,7 +123,7 @@ ClientPattern::ClientPattern(const char *str):
     if (had_error == 0) {
         // otherwise, we check for a number
         string number;
-        err = FbTk::StringUtil::getStringBetween(number, 
+        err = FbTk::StringUtil::getStringBetween(number,
                                              str+pos,
                                              '{', '}');
         if (err > 0) {
@@ -134,13 +132,14 @@ ClientPattern::ClientPattern(const char *str):
             pos+=err;
         }
         // we don't care if there isn't one
-        
+
         // there shouldn't be anything else on the line
         match = str + pos;
-        err = match.find_first_not_of(" \t\n", pos);
-        if ((unsigned) err != match.npos) {
+        size_t uerr;// need a special type here
+        uerr = match.find_first_not_of(" \t\n", pos);
+        if (uerr != match.npos) {
             // found something, not good
-            had_error = err;
+            had_error++;
         }
     }
 
@@ -153,7 +152,7 @@ ClientPattern::ClientPattern(const char *str):
             m_terms.pop_back();
         }
     }
-} 
+}
 
 ClientPattern::~ClientPattern() {
     // delete all the terms
@@ -164,22 +163,17 @@ ClientPattern::~ClientPattern() {
 }
 
 // return a string representation of this pattern
-std::string ClientPattern::toString() const {
+string ClientPattern::toString() const {
     string pat;
     Terms::const_iterator it = m_terms.begin();
     Terms::const_iterator it_end = m_terms.end();
     for (; it != it_end; ++it) {
 
         pat.append(" (");
-        // replace special chars like ( ) and [ ] with \( \) and \[ \]
-        string orig = FbTk::StringUtil::replaceString((*it)->orig, "(", "\\(");
-        orig = FbTk::StringUtil::replaceString(orig, ")", "\\)");
-        orig = FbTk::StringUtil::replaceString(orig, "[", "\\[");
-        orig = FbTk::StringUtil::replaceString(orig, "]", "\\]");
 
         switch ((*it)->prop) {
         case NAME:
-            // do nothing -> this is the default
+            pat.append("name=");
             break;
         case CLASS:
             pat.append("class=");
@@ -191,7 +185,7 @@ std::string ClientPattern::toString() const {
             pat.append("role=");
         }
 
-        pat.append(orig);
+        pat.append((*it)->orig);
         pat.append(")");
     }
 
@@ -205,7 +199,7 @@ std::string ClientPattern::toString() const {
 
 // does this client match this pattern?
 bool ClientPattern::match(const WinClient &win) const {
-    if (m_matchlimit != 0 && m_nummatches >= m_matchlimit || 
+    if (m_matchlimit != 0 && m_nummatches >= m_matchlimit ||
         m_terms.empty())
         return false; // already matched out
 
@@ -224,7 +218,7 @@ bool ClientPattern::match(const WinClient &win) const {
 // add an expression to match against
 // The first argument is a regular expression, the second is the member
 // function that we wish to match against.
-bool ClientPattern::addTerm(const std::string &str, WinProperty prop) {
+bool ClientPattern::addTerm(const string &str, WinProperty prop) {
 
     Term *term = new Term(str, true);
     term->orig = str;
@@ -238,7 +232,7 @@ bool ClientPattern::addTerm(const std::string &str, WinProperty prop) {
     return true;
 }
 
-std::string ClientPattern::getProperty(WinProperty prop, const WinClient &client) const {
+string ClientPattern::getProperty(WinProperty prop, const WinClient &client) const {
     switch (prop) {
     case TITLE:
         return client.title();
@@ -256,3 +250,19 @@ std::string ClientPattern::getProperty(WinProperty prop, const WinClient &client
     }
     return client.getWMClassName();
 }
+
+bool ClientPattern::equals(const ClientPattern &pat) const {
+    // we require the terms to be identical (order too)
+    Terms::const_iterator it = m_terms.begin();
+    Terms::const_iterator it_end = m_terms.end();
+    Terms::const_iterator other_it = pat.m_terms.begin();
+    Terms::const_iterator other_it_end = pat.m_terms.end();
+    for (; it != it_end && other_it != other_it_end; ++it, ++other_it) {
+        if ((*it)->orig != (*other_it)->orig)
+            return false;
+    }
+    if (it != it_end || other_it != other_it_end)
+        return false;
+
+    return true;
+}
