@@ -1,5 +1,5 @@
 // CommandParser.cc for Fluxbox - an X11 Window manager
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: CommandParser.cc 3910 2005-03-10 16:51:24Z mathias $
+// $Id: CommandParser.cc 4972 2007-07-03 22:56:46Z markt $
 
 #include "CommandParser.hh"
 #include "FbTk/StringUtil.hh"
@@ -32,6 +32,9 @@ using std::vector;
 using FbTk::StringUtil::removeFirstWhitespace;
 using FbTk::StringUtil::toLower;
 
+
+CommandParser *CommandParser::s_singleton = 0;
+
 CommandFactory::CommandFactory() {
 
 }
@@ -39,18 +42,27 @@ CommandFactory::CommandFactory() {
 CommandFactory::~CommandFactory() {
     // remove all associations with this factory
     CommandParser::instance().removeAssociation(*this);
+
 }
 
-void CommandFactory::addCommand(const string &command_name) {
+void CommandFactory::addCommand(const std::string &command_name) {
     CommandParser::instance().associateCommand(command_name, *this);
 }
 
-CommandParser &CommandParser::instance() {
-    static CommandParser singleton;
-    return singleton;
+// ensure it is singleton
+CommandParser::CommandParser() {
+    if (s_singleton != 0)
+        throw std::string("CommandParser currently meant ot be singleton");
 }
 
-FbTk::Command *CommandParser::parseLine(const string &line) {
+CommandParser &CommandParser::instance() {
+    if (s_singleton == 0)
+        s_singleton = new CommandParser();
+
+    return *s_singleton;
+}
+
+FbTk::Command *CommandParser::parseLine(const std::string &line, bool trusted) {
 
     // parse arguments and command
     string command = line;
@@ -71,13 +83,14 @@ FbTk::Command *CommandParser::parseLine(const string &line) {
     // we didn't find any matching command in default commands,
     // so we search in the command creators modules for a 
     // matching command string
-    return toCommand(command, arguments);
+    return toCommand(command, arguments, trusted);
 
 }
 
-FbTk::Command *CommandParser::toCommand(const std::string &command_str, const std::string &arguments) {
+FbTk::Command *CommandParser::toCommand(const std::string &command_str,
+        const std::string &arguments, bool trusted) {
     if (m_commandfactorys[command_str] != 0)
-        return m_commandfactorys[command_str]->stringToCommand(command_str, arguments);
+        return m_commandfactorys[command_str]->stringToCommand(command_str, arguments, trusted);
 
     return 0;
 }
@@ -105,4 +118,7 @@ void CommandParser::removeAssociation(CommandFactory &factory) {
         m_commandfactorys.erase(commands.back());
         commands.pop_back();
     }
+
+    if (m_commandfactorys.empty())
+        delete s_singleton;
 }
