@@ -1,5 +1,5 @@
 // FbCommandFactory.cc for Fluxbox Window manager
-// Copyright (c) 2003 Henrik Kinnunen (fluxgen at users.sourceforge.net)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbCommandFactory.cc 3787 2004-12-21 16:09:36Z mathias $
+// $Id: FbCommandFactory.cc 4972 2007-07-03 22:56:46Z markt $
 
 #include "FbCommandFactory.hh"
 
@@ -34,36 +34,39 @@
 
 #include "FbTk/StringUtil.hh"
 #include "FbTk/MacroCommand.hh"
+#include "FbTk/stringstream.hh"
 
 #include <string>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
-#ifdef HAVE_SSTREAM
-#include <sstream>
-#define FB_istringstream istringstream
-#elif HAVE_STRSTREAM 
-#include <strstream>
-#define FB_istringstream istrstream
+#ifdef HAVE_CSTDIO
+  #include <cstdio>
 #else
-#error "You dont have sstream or strstream headers!"
-#endif // HAVE_STRSTREAM
+  #include <stdio.h>
+#endif
 
-using namespace std;
+using std::string;
+using std::vector;
+using std::cerr;
+using std::endl;
 
 // autoregister this module to command parser
 FbCommandFactory FbCommandFactory::s_autoreg;
 
+static int getint(const char *str, int defaultvalue) {
+    sscanf(str, "%d", &defaultvalue);
+    return defaultvalue;
+}
 
 FbCommandFactory::FbCommandFactory() {
     // setup commands that we can handle
     const char* commands[] = {
+        "addworkspace",
         "arrangewindows",
         "bindkey",
         "close",
+        "closeallwindows",
         "commanddialog",
+        "custommenu",
         "deiconify",
         "detachclient",
         "export",
@@ -75,7 +78,11 @@ FbCommandFactory::FbCommandFactory() {
         "focusdown",
         "focusleft",
         "focusright",
+        "fullscreen",
+        "hidemenus",
         "iconify",
+        "keymode",
+        "kill",
         "killwindow",
         "leftworkspace",
         "lower",
@@ -109,6 +116,7 @@ FbCommandFactory::FbCommandFactory() {
         "reconfig",
         "reconfigure",
         "reloadstyle",
+        "removelastworkspace",
         "resizeto",
         "resize",
         "resizehorizontal",
@@ -117,11 +125,13 @@ FbCommandFactory::FbCommandFactory() {
         "rightworkspace",
         "rootmenu",
         "saverc",
-        "setenv",
-        "sethead",
         "sendtoworkspace",
         "sendtonextworkspace",
         "sendtoprevworkspace",
+        "setalpha",
+        "setenv",
+        "sethead",
+        "setmodkey",
         "setstyle",
         "setworkspacename",
         "setworkspacenamedialog",
@@ -133,6 +143,10 @@ FbCommandFactory::FbCommandFactory() {
         "stick",
         "stickwindow",
         "tab",
+        "taketoworkspace",
+        "taketonextworkspace",
+        "taketoprevworkspace",
+        "togglecmd",
         "toggledecor",
         "windowmenu",
         "workspace",
@@ -151,24 +165,20 @@ FbCommandFactory::FbCommandFactory() {
         "workspace12",
         /* end note */
         "workspacemenu",
-        ""
+        0
     };
 
-    for (int i=0;; ++i) {
-        if (strcmp(commands[i], "") == 0)
-            break;
+    for (int i=0; commands[i]; ++i)
         addCommand(commands[i]);
-    }
-                     
 }
 
 FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
-                                                 const std::string &arguments) {
+        const std::string &arguments, bool trusted) {
     using namespace FbCommands;
     //
     // WM commands
     //
-    if (command == "restart")
+    if (command == "restart" && trusted)
         return new RestartFluxboxCmd(arguments);
     else if (command == "reconfigure" || command == "reconfig")
         return new ReconfigureFluxboxCmd();
@@ -176,32 +186,40 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         return new SetStyleCmd(arguments);
     else if (command == "reloadstyle")
         return new ReloadStyleCmd();
+    else if (command == "keymode")
+        return new KeyModeCmd(arguments);
     else if (command == "saverc")
         return new SaveResources();
-    else if (command == "execcommand" || command == "execute" || command == "exec")
+    else if (command == "execcommand" || command == "execute" || command == "exec") {
+        if (!trusted) return 0;
         return new ExecuteCmd(arguments); // execute command on key screen
-    else if (command == "exit")
+    } else if (command == "exit" || command == "quit")
         return new ExitFluxboxCmd();
-    else if (command == "setenv" || command == "export") {
+    else if ((command == "setenv" || command == "export") && trusted) {
 
-        std::string name = arguments;
+        string name = arguments;
         FbTk::StringUtil::removeFirstWhitespace(name);
         FbTk::StringUtil::removeTrailingWhitespace(name);
         size_t pos = name.find_first_of(command == "setenv" ? " \t" : "=");
-        if (pos == std::string::npos || pos == name.size())
+        if (pos == string::npos || pos == name.size())
             return 0;
-        
-        std::string value = name.substr(pos + 1);
+
+        string value = name.substr(pos + 1);
         name = name.substr(0, pos);
         return new ExportCmd(name, value);
     }
-    else if (command == "quit")
-        return new FbTk::SimpleCommand<Fluxbox>(*Fluxbox::instance(), &Fluxbox::shutdown);
+    else if (command == "setmodkey") {
+        string modkey(arguments);
+        FbTk::StringUtil::removeFirstWhitespace(modkey);
+        FbTk::StringUtil::removeTrailingWhitespace(modkey);
+
+        return new SetModKeyCmd(modkey);
+    }
     else if (command == "commanddialog") // run specified fluxbox command
         return new CommandDialogCmd();
-    else if (command == "bindkey")
+    else if (command == "bindkey" && trusted)
         return new BindKeyCmd(arguments);
-    else if (command == "setresourcevalue") {
+    else if (command == "setresourcevalue" && trusted) {
         // we need to parse arguments as:
         // <remove whitespace here><resname><one whitespace><value>
         string name = arguments;
@@ -217,25 +235,59 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         return new SetResourceValueCmd(name, value);
     } else if (command == "setresourcevaluedialog")
         return new SetResourceValueDialogCmd();
+    else if (command == "addworkspace")
+        return new AddWorkspaceCmd();
+    else if (command == "removelastworkspace")
+        return new RemoveLastWorkspaceCmd();
     //
     // Current focused window commands
     //
-    else if (command == "minimizewindow" || command == "minimize" || command == "iconify")
-        return new CurrentWindowCmd(&FluxboxWindow::iconify);
-    else if (command == "maximizewindow" || command == "maximize")
+    else if (command == "fullscreen")
+        return new FullscreenCmd();
+    else if (command == "minimizewindow" || command == "minimize" || command == "iconify") {
+        string cmd;
+        if (FbTk::StringUtil::getStringBetween(cmd, arguments.c_str() +
+                                               0, '(', ')', " \t\n", true)
+            && cmd == "layer")
+            return new MinimizeLayerCmd();
+        else
+            return new CurrentWindowCmd(&FluxboxWindow::iconify);
+    } else if (command == "maximizewindow" || command == "maximize")
         return new CurrentWindowCmd(&FluxboxWindow::maximizeFull);
     else if (command == "maximizevertical")
         return new CurrentWindowCmd(&FluxboxWindow::maximizeVertical);
     else if (command == "maximizehorizontal")
         return new CurrentWindowCmd(&FluxboxWindow::maximizeHorizontal);
-    else if (command == "resize") {
-        FB_istringstream is(arguments.c_str()); 
+    else if (command == "setalpha") {
+        typedef vector<string> StringTokens;
+        StringTokens tokens;
+        FbTk::StringUtil::stringtok<StringTokens>(tokens, arguments);
+
+        int focused, unfocused;
+        bool relative, un_rel;
+
+        if (tokens.empty()) { // set default alpha
+            focused = unfocused = 256;
+            relative = un_rel = false;
+        } else {
+            relative = un_rel = (tokens[0][0] == '+' || tokens[0][0] == '-');
+            focused = unfocused = atoi(tokens[0].c_str());
+        }
+
+        if (tokens.size() > 1) { // set different unfocused alpha
+            un_rel = (tokens[1][0] == '+' || tokens[1][0] == '-');
+            unfocused = atoi(tokens[1].c_str());
+        }
+
+        return new SetAlphaCmd(focused, relative, unfocused, un_rel);
+    } else if (command == "resize") {
+        FbTk_istringstream is(arguments.c_str());
         int dx = 0, dy = 0;
         is >> dx >> dy;
         return new ResizeCmd(dx, dy);
     }
     else if (command == "resizeto") {
-        FB_istringstream is(arguments.c_str());
+        FbTk_istringstream is(arguments.c_str());
         int dx = 0, dy = 0;
         is >> dx >> dy;
         return new ResizeToCmd(dx, dy);
@@ -245,13 +297,52 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
     else if (command == "resizevertical")
         return new ResizeCmd(0,atoi(arguments.c_str()));
     else if (command == "moveto") {
-       FB_istringstream is(arguments.c_str());
-       int dx = 0, dy = 0;
-       is >> dx >> dy;
-       return new MoveToCmd(dx,dy);    
+        typedef vector<string> StringTokens;
+        StringTokens tokens;
+        FbTk::StringUtil::stringtok<StringTokens>(tokens, arguments);
+
+        if (tokens.size() < 2) {
+            cerr<<"*** WARNING: missing arguments for MoveTo\n";
+            return NULL;
+        }
+
+        unsigned int refc = MoveToCmd::UPPER|MoveToCmd::LEFT;
+        int dx = 0;
+        int dy = 0;
+
+        if (tokens[0][0] == '*')
+            refc |= MoveToCmd::IGNORE_X;
+        else
+            dx = atoi(tokens[0].c_str());
+
+        if (tokens[1][0] == '*' && ! (refc & MoveToCmd::IGNORE_X))
+            refc |= MoveToCmd::IGNORE_Y;
+        else
+            dy = atoi(tokens[1].c_str());
+
+        if (tokens.size() >= 3) {
+            tokens[2] = FbTk::StringUtil::toLower(tokens[2]);
+            if (tokens[2] == "left" || tokens[2] == "upperleft" || tokens[2] == "lowerleft") {
+                refc |= MoveToCmd::LEFT;
+                refc &= ~MoveToCmd::RIGHT;
+            } else if (tokens[2] == "right" || tokens[2] == "upperright" || tokens[2] == "lowerright") {
+                refc |= MoveToCmd::RIGHT;
+                refc &= ~MoveToCmd::LEFT;
+            }
+
+            if (tokens[2] == "upper" || tokens[2] == "upperleft" || tokens[2] == "upperright") {
+                refc |= MoveToCmd::UPPER;
+                refc &= ~MoveToCmd::LOWER;
+            } else if (tokens[2] == "lower" || tokens[2] == "lowerleft" || tokens[2] == "lowerright") {
+                refc |= MoveToCmd::LOWER;
+                refc &= ~MoveToCmd::UPPER;
+            }
+        }
+
+        return new MoveToCmd(dx, dy, refc);
     }
     else if (command == "move") {
-        FB_istringstream is(arguments.c_str());
+        FbTk_istringstream is(arguments.c_str());
         int dx = 0, dy = 0;
         is >> dx >> dy;
         return new MoveCmd(dx, dy);
@@ -274,6 +365,8 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         return new CurrentWindowCmd(&FluxboxWindow::lowerLayer);
     else if (command == "close")
         return new CurrentWindowCmd(&FluxboxWindow::close);
+    else if (command == "closeallwindows")
+        return new CloseAllWindowsCmd();
     else if (command == "shade" || command == "shadewindow")
         return new CurrentWindowCmd(&FluxboxWindow::shade);
     else if (command == "stick" || command == "stickwindow")
@@ -283,20 +376,24 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
     else if (command == "sethead")
         return new SetHeadCmd(atoi(arguments.c_str()));
     else if (command == "sendtoworkspace")
-        return new SendToWorkspaceCmd(atoi(arguments.c_str()) - 1); // make 1-indexed to user
+        // workspaces appear 1-indexed to the user, hence the minus 1
+        return new SendToWorkspaceCmd(getint(arguments.c_str(), 1) - 1);
     else if (command == "sendtonextworkspace")
-        return new SendToNextWorkspaceCmd(atoi(arguments.c_str()));
+        return new SendToNextWorkspaceCmd(getint(arguments.c_str(), 1));
     else if (command == "sendtoprevworkspace")
-        return new SendToPrevWorkspaceCmd(atoi(arguments.c_str()));
+        return new SendToPrevWorkspaceCmd(getint(arguments.c_str(), 1));
+    else if (command == "taketoworkspace")
+        // workspaces appear 1-indexed to the user, hence the minus 1
+        return new TakeToWorkspaceCmd(getint(arguments.c_str(), 1) - 1);
+    else if (command == "taketonextworkspace")
+        return new TakeToNextWorkspaceCmd(getint(arguments.c_str(), 1));
+    else if (command == "taketoprevworkspace")
+        return new TakeToPrevWorkspaceCmd(getint(arguments.c_str(), 1));
     else if (command == "killwindow" || command == "kill")
         return new KillWindowCmd();
-    else if (command == "tab") {
-        // XXX
-        int num = 1;
-        if (!arguments.empty())
-            num = atoi(arguments.c_str());
-        return new GoToTabCmd(num);
-    } else if (command == "nexttab")
+    else if (command == "tab")
+        return new GoToTabCmd(getint(arguments.c_str(), 1));
+    else if (command == "nexttab")
         return new CurrentWindowCmd(&FluxboxWindow::nextClient);
     else if (command == "prevtab")
         return new CurrentWindowCmd(&FluxboxWindow::prevClient);
@@ -308,50 +405,50 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         return new CurrentWindowCmd(&FluxboxWindow::detachCurrentClient);
     else if (command == "windowmenu")
         return new CurrentWindowCmd(&FluxboxWindow::popupMenu);
-    // 
+    //
     // Workspace commands
     //
-    else if (command == "nextworkspace" && arguments.empty())
-        return new NextWorkspaceCmd();
-    else if (command == "prevworkspace" && arguments.empty())
-        return new PrevWorkspaceCmd();
+    else if (command == "nextworkspace")
+        return new NextWorkspaceCmd(getint(arguments.c_str(), 1));
+    else if (command == "prevworkspace")
+        return new PrevWorkspaceCmd(getint(arguments.c_str(), 1));
     else if (command == "rightworkspace")
-        return new RightWorkspaceCmd(atoi(arguments.c_str()));
+        return new RightWorkspaceCmd(getint(arguments.c_str(), 1));
     else if (command == "leftworkspace")
-        return new LeftWorkspaceCmd(atoi(arguments.c_str()));
-    else if (command == "workspace") {
-        int num = 1; // workspaces appear 1-indexed to the user
-        if (!arguments.empty())
-            num = atoi(arguments.c_str());
-        return new JumpToWorkspaceCmd(num-1);
-    } if (command.substr(0, 9) == "workspace" && command[9] >= '0' && command[9] <= '9') {
+        return new LeftWorkspaceCmd(getint(arguments.c_str(), 1));
+    else if (command == "workspace")
+        // workspaces appear 1-indexed to the user, hence the minus 1
+        return new JumpToWorkspaceCmd(getint(arguments.c_str(), 1) - 1);
+    else if (command.substr(0, 9) == "workspace" && command[9] >= '0' && command[9] <= '9') {
         cerr<<"*** WARNING: 'Workspace<n>' actions are deprecated! Use 'Workspace <n>' instead"<<endl;
-        int num = 1;
-        num = atoi(command.substr(9).c_str());
-        return new JumpToWorkspaceCmd(num-1);
-        
+        return new JumpToWorkspaceCmd(getint(command.substr(9).c_str(), 1) - 1);
+
     } else if (command == "nextwindow")
         return new NextWindowCmd(atoi(arguments.c_str()));
     else if (command == "prevwindow")
         return new PrevWindowCmd(atoi(arguments.c_str()));
     else if (command == "focusup")
-        return new DirFocusCmd(BScreen::FOCUSUP);
+        return new DirFocusCmd(FocusControl::FOCUSUP);
     else if (command == "focusdown")
-        return new DirFocusCmd(BScreen::FOCUSDOWN);
+        return new DirFocusCmd(FocusControl::FOCUSDOWN);
     else if (command == "focusleft")
-        return new DirFocusCmd(BScreen::FOCUSLEFT);
+        return new DirFocusCmd(FocusControl::FOCUSLEFT);
     else if (command == "focusright")
-        return new DirFocusCmd(BScreen::FOCUSRIGHT);
+        return new DirFocusCmd(FocusControl::FOCUSRIGHT);
     else if (command == "nextgroup")
-        return new NextWindowCmd(atoi(arguments.c_str()) ^ BScreen::CYCLEGROUPS);
+        return new NextWindowCmd(atoi(arguments.c_str()) ^ FocusControl::CYCLEGROUPS);
     else if (command == "prevgroup")
-        return new PrevWindowCmd(atoi(arguments.c_str()) ^ BScreen::CYCLEGROUPS);
+        return new PrevWindowCmd(atoi(arguments.c_str()) ^ FocusControl::CYCLEGROUPS);
     else if (command == "arrangewindows")
         return new ArrangeWindowsCmd();
     else if (command == "showdesktop")
         return new ShowDesktopCmd();
+    else if (command == "hidemenus")
+        return new HideMenuCmd();
     else if (command == "rootmenu")
         return new ShowRootMenuCmd();
+    else if (command == "custommenu")
+        return new ShowCustomMenuCmd(arguments.c_str());
     else if (command == "workspacemenu")
         return new ShowWorkspaceMenuCmd();
     else if (command == "setworkspacename") {
@@ -367,7 +464,7 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
     //
     else if (command == "deiconify") {
 
-        FB_istringstream iss(arguments);
+        FbTk_istringstream iss(arguments.c_str());
         string mode;
         string d;
         DeiconifyCmd::Destination dest;
@@ -376,7 +473,7 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
         if (iss.fail())
             mode="lastworkspace";
         mode= FbTk::StringUtil::toLower(mode);
-        
+
         iss >> d;
         if (iss.fail())
             d="current";
@@ -387,9 +484,9 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
           dest= DeiconifyCmd::ORIGINQUIET;
         else
           dest= DeiconifyCmd::CURRENT;
-          
+
         if ( mode == "all" )
-            return new DeiconifyCmd(DeiconifyCmd::ALL, dest); 
+            return new DeiconifyCmd(DeiconifyCmd::ALL, dest);
         else if ( mode == "allworkspace" )
             return new DeiconifyCmd(DeiconifyCmd::ALLWORKSPACE, dest);
         else if ( mode == "last" )
@@ -398,33 +495,72 @@ FbTk::Command *FbCommandFactory::stringToCommand(const std::string &command,
             return new DeiconifyCmd(DeiconifyCmd::LASTWORKSPACE, dest);
 
     } else if (command == "macrocmd") {
-      std::string cmd;
+      string cmd;
       int   err= 0;
       int   parse_pos= 0;
       FbTk::MacroCommand* macro= new FbTk::MacroCommand();
 
       while (true) {
         parse_pos+= err;
-        err= FbTk::StringUtil::getStringBetween(cmd, arguments.c_str() + parse_pos,
-                                              '{', '}', " \t\n", true);
+        err= FbTk::StringUtil::getStringBetween(cmd, arguments.c_str() +
+                                                parse_pos,
+                                                '{', '}', " \t\n", true);
         if ( err > 0 ) {
-          std::string c, a;
-          std::string::size_type first_pos = FbTk::StringUtil::removeFirstWhitespace(cmd);
-          std::string::size_type second_pos= cmd.find_first_of(" \t", first_pos);
-          if (second_pos != std::string::npos) {
+          string c, a;
+          string::size_type first_pos =
+              FbTk::StringUtil::removeFirstWhitespace(cmd);
+          string::size_type second_pos =
+              cmd.find_first_of(" \t", first_pos);
+          if (second_pos != string::npos) {
             a= cmd.substr(second_pos);
             FbTk::StringUtil::removeFirstWhitespace(a);
             cmd.erase(second_pos);
           }
           c= FbTk::StringUtil::toLower(cmd);
 
-          FbTk::Command* fbcmd= stringToCommand(c,a);
-          if ( fbcmd ) {
+          FbTk::Command* fbcmd= stringToCommand(c,a,trusted);
+          if (fbcmd) {
             FbTk::RefCount<FbTk::Command> rfbcmd(fbcmd);
             macro->add(rfbcmd);
           }
-        }
-        else
+        } else
+          break;
+      }
+
+      if ( macro->size() > 0 )
+        return macro;
+
+      delete macro;
+    } else if (command == "togglecmd") {
+      string cmd;
+      int   err= 0;
+      int   parse_pos= 0;
+      FbTk::ToggleCommand* macro= new FbTk::ToggleCommand();
+
+      while (true) {
+        parse_pos+= err;
+        err= FbTk::StringUtil::getStringBetween(cmd, arguments.c_str() +
+                                                parse_pos,
+                                                '{', '}', " \t\n", true);
+        if ( err > 0 ) {
+          string c, a;
+          string::size_type first_pos =
+              FbTk::StringUtil::removeFirstWhitespace(cmd);
+          string::size_type second_pos=
+              cmd.find_first_of(" \t", first_pos);
+          if (second_pos != string::npos) {
+            a= cmd.substr(second_pos);
+            FbTk::StringUtil::removeFirstWhitespace(a);
+            cmd.erase(second_pos);
+          }
+          c= FbTk::StringUtil::toLower(cmd);
+
+          FbTk::Command* fbcmd= stringToCommand(c,a,trusted);
+          if (fbcmd) {
+            FbTk::RefCount<FbTk::Command> rfbcmd(fbcmd);
+            macro->add(rfbcmd);
+          }
+        } else
           break;
       }
 
