@@ -1,5 +1,5 @@
 // FbCommands.cc for Fluxbox
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbCommands.cc 4012 2005-05-12 20:03:08Z mathias $
+// $Id: FbCommands.cc 4992 2007-07-24 12:12:45Z markt $
 
 #include "FbCommands.hh"
 #include "fluxbox.hh"
@@ -28,6 +28,7 @@
 #include "Workspace.hh"
 #include "Window.hh"
 #include "Keys.hh"
+#include "MenuCreator.hh"
 
 #include "FbTk/Theme.hh"
 #include "FbTk/Menu.hh"
@@ -44,6 +45,11 @@
 #else
   #include <stdlib.h>
 #endif
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 
 
 #ifdef HAVE_CONFIG_H
@@ -54,14 +60,19 @@
 #include <process.h> // for P_NOWAIT
 #endif // __EMX__
 
-using namespace std;
+using std::string;
+using std::pair;
+using std::set;
+using std::ofstream;
+using std::endl;
+using std::ios;
 
 namespace {
 
 void showMenu(const BScreen &screen, FbTk::Menu &menu) {
 
     // special case for root menu
-    if (&menu == &screen.getRootmenu()) {
+    if (&menu == &screen.rootMenu()) {
         Fluxbox* fb = Fluxbox::instance();
         if(fb->menuTimestampsChanged()) {
             // we dont show the menu here because fluxbox
@@ -71,7 +82,7 @@ void showMenu(const BScreen &screen, FbTk::Menu &menu) {
             return;
         }
     }
-    
+
     Window root_ret; // not used
     Window window_ret; // not used
 
@@ -86,9 +97,10 @@ void showMenu(const BScreen &screen, FbTk::Menu &menu) {
     int borderw = menu.fbwindow().borderWidth();
     int head = screen.getHead(rx, ry);
 
-    pair<int, int> m = 
+    menu.updateMenu();
+    pair<int, int> m =
         screen.clampToHead(head,
-                           rx - menu.width() / 2, 
+                           rx - menu.width() / 2,
                            ry - menu.titleWindow().height() / 2,
                            menu.width() + 2*borderw,
                            menu.height() + 2*borderw);
@@ -98,7 +110,7 @@ void showMenu(const BScreen &screen, FbTk::Menu &menu) {
                    screen.getHeadY(head),
                    screen.getHeadWidth(head),
                    screen.getHeadHeight(head));
-    
+
     menu.show();
     menu.grabInputFocus();
 }
@@ -107,42 +119,66 @@ void showMenu(const BScreen &screen, FbTk::Menu &menu) {
 
 namespace FbCommands {
 
-ExecuteCmd::ExecuteCmd(const std::string &cmd, int screen_num):m_cmd(cmd), m_screen_num(screen_num) {
+ExecuteCmd::ExecuteCmd(const string &cmd, int screen_num):m_cmd(cmd), m_screen_num(screen_num) {
 
 }
 
 void ExecuteCmd::execute() {
 #ifndef __EMX__
-    if (! fork()) {
-        std::string displaystring("DISPLAY=");
-        displaystring += DisplayString(FbTk::App::instance()->display());
-        char intbuff[64];
-        int screen_num = m_screen_num;
-        if (screen_num < 0) {
-            if (Fluxbox::instance()->mouseScreen() == 0)
-                screen_num = 0;
-            else
-                screen_num = Fluxbox::instance()->mouseScreen()->screenNumber();
-        }
-
-        sprintf(intbuff, "%d", screen_num);
-
-        // remove last number of display and add screen num
-        displaystring.erase(displaystring.size()-1);
-        displaystring += intbuff;
-        setsid();
-        putenv(const_cast<char *>(displaystring.c_str()));
-        execl("/bin/sh", "/bin/sh", "-c", m_cmd.c_str(), static_cast<void*>(NULL));
-        exit(0);
-    }
+    run();
 #else //   __EMX__
     spawnlp(P_NOWAIT, "cmd.exe", "cmd.exe", "/c", m_cmd.c_str(), static_cast<void*>(NULL));
 #endif // !__EMX__
 
 }
 
-ExportCmd::ExportCmd(const std::string& name, const std::string& value) :
-    m_name(name), m_value(value) { 
+int ExecuteCmd::run() {
+    pid_t pid = fork();
+    if (pid)
+        return pid;
+
+    string displaystring("DISPLAY=");
+    displaystring += DisplayString(FbTk::App::instance()->display());
+    char intbuff[64];
+    int screen_num = m_screen_num;
+    if (screen_num < 0) {
+        if (Fluxbox::instance()->mouseScreen() == 0)
+            screen_num = 0;
+        else
+            screen_num = Fluxbox::instance()->mouseScreen()->screenNumber();
+    }
+
+    sprintf(intbuff, "%d", screen_num);
+
+    // get shell path from the environment
+    // this process exits immediately, so we don't have to worry about memleaks
+    const char *shell = getenv("SHELL");
+    if (!shell)
+        shell = "/bin/sh";
+
+    // remove last number of display and add screen num
+    displaystring.erase(displaystring.size()-1);
+    displaystring += intbuff;
+
+    setsid();
+    putenv(const_cast<char *>(displaystring.c_str()));
+    execl(shell, shell, "-c", m_cmd.c_str(), static_cast<void*>(NULL));
+    exit(0);
+
+    return pid; // compiler happy -> we are happy ;)
+}
+
+SetModKeyCmd::SetModKeyCmd(const string& modkey) : m_modkey(modkey) { }
+
+void SetModKeyCmd::execute() {
+    Fluxbox::instance()->setModKey(m_modkey.c_str());
+    Fluxbox::instance()->save_rc();
+    // TODO: we need a better way to do this ...
+    Fluxbox::instance()->reconfigure();
+}
+
+ExportCmd::ExportCmd(const string& name, const string& value) :
+    m_name(name), m_value(value) {
 }
 
 void ExportCmd::execute() {
@@ -150,12 +186,12 @@ void ExportCmd::execute() {
     // the setenv()-routine is not everywhere available and
     // putenv() doesnt manage the strings in the environment
     // and hence we have to do that on our own to avoid memleaking
-    static std::set<char*> stored;
+    static set<char*> stored;
     char* newenv = new char[m_name.size() + m_value.size() + 2];
     if (newenv) {
-    
+
         char* oldenv = getenv(m_name.c_str());
-        
+
         // oldenv points to the value .. we have to go back a bit
         if (oldenv && stored.find(oldenv - (m_name.size() + 1)) != stored.end())
             oldenv -= (m_name.size() + 1);
@@ -186,7 +222,7 @@ void SaveResources::execute() {
     Fluxbox::instance()->save_rc();
 }
 
-RestartFluxboxCmd::RestartFluxboxCmd(const std::string &cmd):m_cmd(cmd){
+RestartFluxboxCmd::RestartFluxboxCmd(const string &cmd):m_cmd(cmd){
 }
 
 void RestartFluxboxCmd::execute() {
@@ -206,14 +242,54 @@ void ReloadStyleCmd::execute() {
     cmd.execute();
 }
 
-SetStyleCmd::SetStyleCmd(const std::string &filename):m_filename(filename) {
+SetStyleCmd::SetStyleCmd(const string &filename):m_filename(filename) {
 
 }
 
 void SetStyleCmd::execute() {
     Fluxbox::instance()->saveStyleFilename(m_filename.c_str());
     Fluxbox::instance()->save_rc();
-    FbTk::ThemeManager::instance().load(m_filename);
+    FbTk::ThemeManager::instance().load(m_filename,
+                                        Fluxbox::instance()->getStyleOverlayFilename());
+}
+
+KeyModeCmd::KeyModeCmd(const string &arguments):m_keymode(arguments),m_end_args("None Escape") {
+    string::size_type second_pos = m_keymode.find_first_of(" \t", 0);
+    if (second_pos != string::npos) {
+        // ok we have arguments, parsing them here
+        m_end_args = m_keymode.substr(second_pos);
+        m_keymode.erase(second_pos); // remove argument from command
+    }
+    if (m_keymode != "default")
+        Fluxbox::instance()->keys()->addBinding(m_keymode + ": " + m_end_args + " :keymode default");
+}
+
+void KeyModeCmd::execute() {
+    Fluxbox::instance()->keys()->keyMode(m_keymode);
+}
+
+void HideMenuCmd::execute() {
+    BScreen *screen = Fluxbox::instance()->mouseScreen();
+    screen->hideMenus();
+    if (screen->rootMenu().isVisible())
+        screen->rootMenu().hide();
+    if (screen->workspaceMenu().isVisible())
+        screen->workspaceMenu().hide();
+    if (FbTk::Menu::shownMenu())
+        FbTk::Menu::shownMenu()->hide();
+}
+
+ShowCustomMenuCmd::ShowCustomMenuCmd(const string &arguments) : custom_menu_file(arguments) {}
+
+void ShowCustomMenuCmd::execute() {
+    BScreen *screen = Fluxbox::instance()->mouseScreen();
+    if (screen == 0)
+        return;
+    m_menu = MenuCreator::createFromFile(custom_menu_file,
+            screen->screenNumber(), true);
+    if (!m_menu.get())
+        return;
+    ::showMenu(*screen, **m_menu);
 }
 
 void ShowRootMenuCmd::execute() {
@@ -221,7 +297,7 @@ void ShowRootMenuCmd::execute() {
     if (screen == 0)
         return;
 
-    ::showMenu(*screen, screen->getRootmenu());
+    ::showMenu(*screen, screen->rootMenu());
 }
 
 void ShowWorkspaceMenuCmd::execute() {
@@ -229,12 +305,12 @@ void ShowWorkspaceMenuCmd::execute() {
     if (screen == 0)
         return;
 
-    ::showMenu(*screen, screen->getWorkspacemenu());
+    ::showMenu(*screen, screen->workspaceMenu());
 }
 
 
 
-SetWorkspaceNameCmd::SetWorkspaceNameCmd(const std::string &name, int spaceid):
+SetWorkspaceNameCmd::SetWorkspaceNameCmd(const string &name, int spaceid):
     m_name(name), m_workspace(spaceid) { }
 
 void SetWorkspaceNameCmd::execute() {
@@ -278,9 +354,9 @@ void CommandDialogCmd::execute() {
     win->show();
 }
 
-    
-SetResourceValueCmd::SetResourceValueCmd(const std::string &resname, 
-                                         const std::string &value):
+
+SetResourceValueCmd::SetResourceValueCmd(const string &resname,
+                                         const string &value):
     m_resname(resname),
     m_value(value) {
 
@@ -303,20 +379,20 @@ void SetResourceValueDialogCmd::execute() {
     win->show();
 };
 
-BindKeyCmd::BindKeyCmd(const std::string &keybind):m_keybind(keybind) { }
+BindKeyCmd::BindKeyCmd(const string &keybind):m_keybind(keybind) { }
 
 void BindKeyCmd::execute() {
     if (Fluxbox::instance()->keys() != 0) {
         if (Fluxbox::instance()->keys()->addBinding(m_keybind)) {
             ofstream ofile(Fluxbox::instance()->keys()->filename().c_str(), ios::app);
             if (!ofile)
-                return;            
+                return;
             ofile<<m_keybind<<endl;
         }
     }
 }
 
-DeiconifyCmd::DeiconifyCmd(Mode mode, 
+DeiconifyCmd::DeiconifyCmd(Mode mode,
                            Destination dest) : m_mode(mode), m_dest(dest) { }
 
 void DeiconifyCmd::execute() {
@@ -324,21 +400,24 @@ void DeiconifyCmd::execute() {
     if (screen == 0)
         return;
 
-    BScreen::Icons::reverse_iterator it= screen->getIconList().rbegin();
-    BScreen::Icons::reverse_iterator itend= screen->getIconList().rend();
+    // we need to make a copy of the list of icons, or else our iterator can
+    // become invalid
+    BScreen::Icons icon_list = screen->iconList();
+    BScreen::Icons::iterator it = icon_list.begin();
+    BScreen::Icons::iterator itend= icon_list.end();
     unsigned int workspace_num= screen->currentWorkspaceID();
     unsigned int old_workspace_num;
 
     const bool change_ws= m_dest == ORIGIN;
 
     switch(m_mode) {
-        
+
     case ALL:
     case ALLWORKSPACE:
         for(; it != itend; it++) {
             old_workspace_num= (*it)->workspaceNumber();
-            if (m_mode == ALL || old_workspace_num == workspace_num || 
-                    (*it)->isStuck()) {
+            if (m_mode == ALL || old_workspace_num == workspace_num ||
+                (*it)->isStuck()) {
                 if (m_dest == ORIGIN || m_dest == ORIGINQUIET)
                     screen->sendToWorkspace(old_workspace_num, (*it), change_ws);
                 else
@@ -353,8 +432,8 @@ void DeiconifyCmd::execute() {
         for (; it != itend; it++) {
             old_workspace_num= (*it)->workspaceNumber();
             if(m_mode == LAST || old_workspace_num == workspace_num ||
-                    (*it)->isStuck()) {
-                if ((m_dest == ORIGIN || m_dest == ORIGINQUIET) && 
+               (*it)->isStuck()) {
+                if ((m_dest == ORIGIN || m_dest == ORIGINQUIET) &&
                     m_mode != LASTWORKSPACE)
                     screen->sendToWorkspace(old_workspace_num, (*it), change_ws);
                 else
@@ -365,5 +444,5 @@ void DeiconifyCmd::execute() {
         break;
     };
 }
-    
+
 }; // end namespace FbCommands
