@@ -1,5 +1,5 @@
 // FbPixmap.cc for FbTk - Fluxbox ToolKit
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,19 +19,24 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbPixmap.cc 4004 2005-05-10 16:29:00Z simonb $
+// $Id: FbPixmap.cc 4970 2007-07-03 21:08:07Z markt $
 
 #include "FbPixmap.hh"
 #include "App.hh"
 #include "GContext.hh"
 #include "Transparent.hh"
+#include "FbWindow.hh"
 
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <iostream>
-#include <string>
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 
-using namespace std;
+using std::cerr;
 
 namespace FbTk {
 
@@ -53,18 +58,18 @@ Atom FbPixmap::root_prop_atoms[] = {
 
 FbPixmap::FbPixmap():m_pm(0),
                      m_width(0), m_height(0),
-                     m_depth(0) {
+                     m_depth(0), m_dont_free(false) {
 }
 
 FbPixmap::FbPixmap(const FbPixmap &the_copy):FbDrawable(), m_pm(0),
                                              m_width(0), m_height(0),
-                                             m_depth(0){
+                                             m_depth(0), m_dont_free(false) {
     copy(the_copy);
 }
 
 FbPixmap::FbPixmap(Pixmap pm):m_pm(0),
                               m_width(0), m_height(0),
-                              m_depth(0) {
+                              m_depth(0), m_dont_free(false) {
     if (pm == 0)
         return;
     // assign X pixmap to this
@@ -73,18 +78,18 @@ FbPixmap::FbPixmap(Pixmap pm):m_pm(0),
 
 FbPixmap::FbPixmap(const FbDrawable &src,
                    unsigned int width, unsigned int height,
-                   int depth):m_pm(0),
+                   unsigned int depth):m_pm(0),
                               m_width(0), m_height(0),
-                              m_depth(0) {
+                              m_depth(0), m_dont_free(false) {
 
     create(src.drawable(), width, height, depth);
 }
 
 FbPixmap::FbPixmap(Drawable src,
                    unsigned int width, unsigned int height,
-                   int depth):m_pm(0),
+                   unsigned int depth):m_pm(0),
                               m_width(0), m_height(0),
-                              m_depth(0) {
+                              m_depth(0), m_dont_free(false) {
 
     create(src, width, height, depth);
 }
@@ -109,13 +114,14 @@ FbPixmap &FbPixmap::operator = (Pixmap pm) {
     Window root;
     int x, y;
     unsigned int border_width, bpp;
-    XGetGeometry(display(),
-                 pm,
-                 &root,
-                 &x, &y,
-                 &m_width, &m_height,
-                 &border_width,
-                 &bpp);
+    if (!XGetGeometry(display(),
+                      pm,
+                      &root,
+                      &x, &y,
+                      &m_width, &m_height,
+                      &border_width,
+                      &bpp))
+        return *this;
 
     m_depth = bpp;
 
@@ -125,24 +131,17 @@ FbPixmap &FbPixmap::operator = (Pixmap pm) {
 }
 
 void FbPixmap::copy(const FbPixmap &the_copy) {
-
-    bool create_new = false;
-
-    if (the_copy.width() != width() ||
-        the_copy.height() != height() ||
-        the_copy.depth() != depth() ||
-        drawable() == 0)
-        create_new = true;
-
-    if (create_new)
-        free();
+    /* This function previously retained the old pixmap and copied in
+       the new contents if they had the same dimensions.
+       This broke the image cache, so we don't do that now. If you want to
+       do it, then you'll need to invalidate all copies of this pixmap in
+       the cache */
+    free();
 
     if (the_copy.drawable() != 0) {
-        if (create_new) {
-            create(the_copy.drawable(),
-                   the_copy.width(), the_copy.height(),
-                   the_copy.depth());
-        }
+        create(the_copy.drawable(),
+               the_copy.width(), the_copy.height(),
+               the_copy.depth());
 
         if (drawable()) {
             GContext gc(drawable());
@@ -157,7 +156,7 @@ void FbPixmap::copy(const FbPixmap &the_copy) {
 }
 
 // screen doesn't count if depth is "zero"...
-void FbPixmap::copy(Pixmap pm, int depth, int screen_num) {
+void FbPixmap::copy(Pixmap pm, unsigned int depth, int screen_num) {
     free();
     if (pm == 0)
         return;
@@ -168,13 +167,14 @@ void FbPixmap::copy(Pixmap pm, int depth, int screen_num) {
     unsigned int border_width, bpp;
     unsigned int new_width, new_height;
 
-    XGetGeometry(display(),
-                 pm,
-                 &root,
-                 &x, &y,
-                 &new_width, &new_height,
-                 &border_width,
-                 &bpp);
+    if (!XGetGeometry(display(),
+                      pm,
+                      &root,
+                      &x, &y,
+                      &new_width, &new_height,
+                      &border_width,
+                      &bpp))
+        return;
 
     if (depth == 0)
         depth = bpp;
@@ -201,25 +201,65 @@ void FbPixmap::copy(Pixmap pm, int depth, int screen_num) {
     XFreeGC(display(), gc);
 }
 
-void FbPixmap::rotate() {
+void FbPixmap::rotate(FbTk::Orientation orient) {
+    if (orient == ROT0)
+        return;
+
+    unsigned int oldw = width(), oldh = height();
+    unsigned int neww = oldw, newh = oldh;
+    translateSize(orient, neww, newh);
 
     // make an image copy
     XImage *src_image = XGetImage(display(), drawable(),
                                   0, 0, // pos
-                                  width(), height(), // size
+                                  oldw, oldh, // size
                                   ~0, // plane mask
                                   ZPixmap); // format
     // reverse height/width for new pixmap
-    FbPixmap new_pm(drawable(), height(), width(), depth());
+    FbPixmap new_pm(drawable(), neww, newh, depth());
 
     GContext gc(drawable());
 
-    // copy new area
-    for (unsigned int y = 0; y < height(); ++y) {
-        for (unsigned int x = 0; x < width(); ++x) {
-            gc.setForeground(XGetPixel(src_image, x, y));
-            // revers coordinates
-            XDrawPoint(display(), new_pm.drawable(), gc.gc(), y, x);
+    if (orient == ROT180) {
+        unsigned int srcx, srcy, destx, desty;
+        for (srcy = 0, desty = oldh; srcy < oldh; ++srcy, --desty) {
+            for (srcx = 0, destx = oldw; srcx < oldw; ++srcx, --destx) {
+                gc.setForeground(XGetPixel(src_image, srcx, srcy));
+                XDrawPoint(display(), new_pm.drawable(), gc.gc(), destx, desty);
+            }
+        }
+    } else {
+        // need to flip x and y
+
+        // set start, end and direction based on rotation
+        // NOTE that startx etc are in the direction of the OLD pixmap
+        unsigned int startx = 0, starty = 0;
+        int dirx = 0, diry = 0;
+        switch (orient) {
+        case ROT90:
+            startx = neww-1;
+            starty = 0;
+            dirx = -1;
+            diry = 1;
+            break;
+        case ROT270:
+            startx = 0;
+            starty = newh-1;
+            dirx = 1;
+            diry = -1;
+            break;
+        default: // kill warning
+            break;
+        }
+
+
+        // copy new area
+        unsigned int srcx, srcy, destx, desty;
+        for (srcy = 0, destx = startx; srcy < oldh; ++srcy, destx+=dirx) {
+            for (srcx = 0, desty = starty; srcx < oldw; ++srcx, desty+=diry) {
+                gc.setForeground(XGetPixel(src_image, srcx, srcy));
+                XDrawPoint(display(), new_pm.drawable(), gc.gc(), destx, desty);
+            }
         }
     }
 
@@ -314,9 +354,10 @@ Pixmap FbPixmap::release() {
     return ret;
 }
 
-void FbPixmap::rootwinPropertyNotify(int screen_num, Atom atom) {
-    if (!FbTk::Transparent::haveRender()) 
-        return;
+// returns whether or not the background was changed
+bool FbPixmap::rootwinPropertyNotify(int screen_num, Atom atom) {
+    if (!FbTk::Transparent::haveRender())
+        return false;
 
     checkAtoms();
     for (int i=0; root_prop_ids[i] != 0; ++i) {
@@ -327,9 +368,8 @@ void FbPixmap::rootwinPropertyNotify(int screen_num, Atom atom) {
             unsigned long items_read, items_left;
             unsigned long *data;
 
-            unsigned int prop = 0;
             if (XGetWindowProperty(display(),
-                                   RootWindow(display(), i),
+                                   RootWindow(display(), screen_num),
                                    root_prop_atoms[i],
                                    0l, 1l,
                                    False, XA_PIXMAP,
@@ -341,71 +381,87 @@ void FbPixmap::rootwinPropertyNotify(int screen_num, Atom atom) {
                 }
                 XFree(data);
                 if (root_pm != None)
-                    setRootPixmap(screen_num, root_pm);
+                    return setRootPixmap(screen_num, root_pm);
             }
-            break;
+            return false;
         }
     }
+    return false;
 }
 
-void FbPixmap::setRootPixmap(int screen_num, Pixmap pm) {
+// returns whether or not the background was changed
+bool FbPixmap::setRootPixmap(int screen_num, Pixmap pm) {
     if (!m_root_pixmaps) {
         m_root_pixmaps = new Pixmap[ScreenCount(display())];
+        for (int i=0; i < ScreenCount(display()); ++i)
+            m_root_pixmaps[i] = None;
     }
 
-    m_root_pixmaps[screen_num] = pm;
+    if (m_root_pixmaps[screen_num] != pm) {
+        m_root_pixmaps[screen_num] = pm;
+        FbWindow::updatedAlphaBackground(screen_num);
+        return true;
+    }
+    return false;
 }
 
-Pixmap FbPixmap::getRootPixmap(int screen_num) {
-    if (!FbTk::Transparent::haveRender()) 
-        return None;
+Pixmap FbPixmap::getRootPixmap(int screen_num, bool force_update) {
+    /*
+      if (!FbTk::Transparent::haveRender())
+      return None;
+    */
 
-    if (!m_root_pixmaps) {
-        int numscreens = ScreenCount(display());
-        for (int i=0; i < numscreens; ++i) {
-            Atom real_type;
-            int real_format;
-            unsigned long items_read, items_left;
-            unsigned long *data;
+    // check and see if if we have the pixmaps in cache
+    if (m_root_pixmaps && !force_update)
+        return m_root_pixmaps[screen_num];
 
-            unsigned int prop = 0;
+    // else setup pixmap cache
+    int numscreens = ScreenCount(display());
+    for (int i=0; i < numscreens; ++i) {
+        Atom real_type;
+        int real_format;
+        unsigned long items_read, items_left;
+        unsigned long *data;
 
-            static bool print_error = true; // print error_message only once
-            static const char* error_message = { "\n\n !!! WARNING WARNING WARNING WARNING !!!!!\n"
-        "   if you experience problems with transparency:\n"
-        "   you are using a wallpapersetter that \n"
-        "   uses _XSETROOT_ID .. which we do not support.\n"
-        "   consult 'fbsetbg -i' or try any other wallpapersetter\n"
-        "   that uses _XROOTPMAP_ID !\n"
-        " !!! WARNING WARNING WARNING WARNING !!!!!!\n\n"
-            };
+        unsigned int prop = 0;
 
-            Pixmap root_pm = None;
-            for (prop = 0; root_prop_ids[prop]; prop++) {
-                checkAtoms();
-                if (XGetWindowProperty(display(),
-                                       RootWindow(display(), i),
-                                       root_prop_atoms[i],
-                                       0l, 1l,
-                                       False, XA_PIXMAP,
-                                       &real_type, &real_format,
-                                       &items_read, &items_left,
-                                       (unsigned char **) &data) == Success) {
-                    if (real_format == 32 && items_read == 1) {
-                        if (print_error && strcmp(root_prop_ids[prop], "_XSETROOT_ID") == 0) {
-                            cerr<<error_message;
-                            print_error = false;
-                        } else
-                            root_pm = (Pixmap) (*data);
-                    }
-                    XFree(data);
-                    if (root_pm != None)
-                        break;
+        static bool print_error = true; // print error_message only once
+        static const char* error_message = {
+            "\n\n !!! WARNING WARNING WARNING WARNING !!!!!\n"
+            "   if you experience problems with transparency:\n"
+            "   you are using a wallpapersetter that \n"
+            "   uses _XSETROOT_ID .. which we do not support.\n"
+            "   consult 'fbsetbg -i' or try any other wallpapersetter\n"
+            "   that uses _XROOTPMAP_ID !\n"
+            " !!! WARNING WARNING WARNING WARNING !!!!!!\n\n"
+        };
+
+        Pixmap root_pm = None;
+        for (prop = 0; root_prop_ids[prop]; prop++) {
+            checkAtoms();
+            if (XGetWindowProperty(display(),
+                                   RootWindow(display(), i),
+                                   root_prop_atoms[prop],
+                                   0l, 1l,
+                                   False, XA_PIXMAP,
+                                   &real_type, &real_format,
+                                   &items_read, &items_left,
+                                   (unsigned char **) &data) == Success) {
+                if (real_format == 32 && items_read == 1) {
+                    if (print_error && strcmp(root_prop_ids[prop], "_XSETROOT_ID") == 0) {
+                        cerr<<error_message;
+                        print_error = false;
+                    } else
+                        root_pm = (Pixmap) (*data);
                 }
+                XFree(data);
+                if (root_pm != None)
+                    break;
             }
-            setRootPixmap(i, root_pm);
         }
+        setRootPixmap(i, root_pm);
     }
+
     return m_root_pixmaps[screen_num];
 }
 
@@ -418,10 +474,13 @@ void FbPixmap::checkAtoms() {
 }
 
 void FbPixmap::free() {
-    if (m_pm != 0) {
+    if (!m_dont_free && m_pm != 0)
         XFreePixmap(display(), m_pm);
-        m_pm = 0;
-    }
+
+    /* note: m_dont_free shouldnt be required anywhere else,
+       because then free() isn't being called appropriately! */
+    m_dont_free = false;
+    m_pm = 0;
     m_width = 0;
     m_height = 0;
     m_depth = 0;
@@ -429,7 +488,7 @@ void FbPixmap::free() {
 
 void FbPixmap::create(Drawable src,
                       unsigned int width, unsigned int height,
-                      int depth) {
+                      unsigned int depth) {
     if (src == 0)
         return;
 
