@@ -19,10 +19,11 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbWindow.cc 4199 2006-02-16 06:53:05Z mathias $
+// $Id: FbWindow.cc 4998 2007-08-04 17:14:13Z simonb $
 
 #include "FbWindow.hh"
 #include "FbPixmap.hh"
+#include "FbString.hh"
 
 #include "EventManager.hh"
 #include "Color.hh"
@@ -118,8 +119,10 @@ FbWindow::FbWindow(Window client):FbDrawable(), m_parent(0),
 FbWindow::~FbWindow() {
 
     // Need to free xrender pics before destroying window
-    if (m_transparent.get() != 0)
+    if (m_transparent.get() != 0) {
+        removeAlphaWin(*this);
         m_transparent.reset(0);
+    }
 
     if (m_window != 0) {
         // so we don't get any dangling eventhandler for this window
@@ -127,6 +130,7 @@ FbWindow::~FbWindow() {
         if (m_destroy)
             XDestroyWindow(display(), m_window);
     }
+
 }
 
 void FbWindow::setBackgroundColor(const FbTk::Color &bg_color) {
@@ -149,6 +153,11 @@ void FbWindow::setBackgroundPixmap(Pixmap bg_pixmap) {
     updateBackground(false);
 }
 
+void FbWindow::invalidateBackground() {
+    m_lastbg_pm = None;
+    m_lastbg_color_set = false;
+}
+
 void FbWindow::updateBackground(bool only_if_alpha) {
     Pixmap newbg = m_lastbg_pm;
     unsigned char alpha = 255;
@@ -245,13 +254,19 @@ void FbWindow::setEventMask(long mask) {
 
 void FbWindow::clear() {
     XClearWindow(display(), m_window);
+    if (m_lastbg_pm == ParentRelative && m_renderer)
+        m_renderer->renderForeground(*this, *this);
+
 }
 
 void FbWindow::clearArea(int x, int y,
                          unsigned int width, unsigned int height,
                          bool exposures) {
     // TODO: probably could call renderForeground here (with x,y,w,h)
-    XClearArea(display(), window(), x, y, width, height, exposures);
+    if (m_lastbg_pm == ParentRelative && m_renderer)
+        FbWindow::clear();
+    else
+        XClearArea(display(), window(), x, y, width, height, exposures);
 }
 
 // If override_is_offset, then dest_override is a pixmap located at the_x, the_y
@@ -319,8 +334,10 @@ void FbWindow::updateTransparent(int the_x, int the_y, unsigned int the_width, u
 void FbWindow::setAlpha(unsigned char alpha) {
 #ifdef HAVE_XRENDER
     if (FbTk::Transparent::haveComposite()) {
-        if (m_transparent.get() != 0)
+        if (m_transparent.get() != 0) {
+            removeAlphaWin(*this);
             m_transparent.reset(0);
+        }
 
         // don't setOpaque, let controlling objects do that
         // since it's only needed on toplevel windows
@@ -330,10 +347,13 @@ void FbWindow::setAlpha(unsigned char alpha) {
 
         if (m_transparent.get() == 0 && alpha < 255) {
             m_transparent.reset(new Transparent(FbPixmap::getRootPixmap(screenNumber()), window(), alpha, screenNumber()));
+            addAlphaWin(*this);
         } else if (alpha < 255 && alpha != m_transparent->alpha())
             m_transparent->setAlpha(alpha);
-        else if (alpha == 255)
+        else if (alpha == 255) {
+            removeAlphaWin(*this);
             m_transparent.reset(0); // destroy transparent object
+        }
     }
 #endif // HAVE_XRENDER
 }
@@ -429,9 +449,11 @@ void FbWindow::setCursor(Cursor cur) {
     XDefineCursor(display(), window(), cur);
 }
 
+#ifdef NOT_USED
 void FbWindow::unsetCursor() {
     XUndefineCursor(display(), window());
 }
+#endif
 
 void FbWindow::reparent(const FbWindow &parent, int x, int y, bool continuing) {
     XReparentWindow(display(), window(), parent.window(), x, y);
@@ -439,31 +461,64 @@ void FbWindow::reparent(const FbWindow &parent, int x, int y, bool continuing) {
     if (continuing) // we will continue managing this window after reparent
         updateGeometry();
 }
-
+struct TextPropPtr {
+    TextPropPtr(XTextProperty& prop):m_prop(prop) {}
+    ~TextPropPtr() {
+        if (m_prop.value != 0) {
+            XFree(m_prop.value);
+            m_prop.value = 0;
+        }
+    }
+    XTextProperty& m_prop;
+};
 std::string FbWindow::textProperty(Atom property) const {
     XTextProperty text_prop;
-    char ** stringlist;
-    int count;
+    TextPropPtr helper(text_prop);
+    char ** stringlist = 0;
+    int count = 0;
     std::string ret;
 
-    if (XGetTextProperty(display(), window(), &text_prop, property) == 0)
+    static Atom m_utf8string = XInternAtom(display(), "UTF8_STRING", False);
+
+    if (XGetTextProperty(display(), window(), &text_prop, property) == 0) {
         return "";
-
-    if (text_prop.value == 0 || text_prop.nitems == 0)
-        return "";
-
-    if (text_prop.encoding != XA_STRING) {
-        // still returns a "StringList" despite the different name
-        if (XmbTextPropertyToTextList(display(), &text_prop, &stringlist, &count) == 0 || count == 0)
-            return "";
-    } else {
-        if (XTextPropertyToStringList(&text_prop, &stringlist, &count) == 0 || count == 0)
-            return "";
-
     }
 
-    ret = stringlist[0];
-    XFreeStringList(stringlist);
+    if (text_prop.value == 0 || text_prop.nitems == 0) {
+        return "";
+    }
+
+    if (text_prop.encoding == XA_STRING) {
+        if (XTextPropertyToStringList(&text_prop, &stringlist, &count) == 0 || count == 0) {
+            return "";
+        }
+        ret = FbStringUtil::XStrToFb(stringlist[0]);
+    } else if (text_prop.encoding == m_utf8string && text_prop.format == 8) {
+#ifdef X_HAVE_UTF8_STRING
+        Xutf8TextPropertyToTextList(display(), &text_prop, &stringlist, &count);
+        if (count == 0 || stringlist == 0) {
+            return "";
+        }
+#else
+        if (XTextPropertyToStringList(&text_prop, &stringlist, &count) == 0 || count == 0 || stringlist == 0) {
+            return "";
+        }
+#endif
+        ret = stringlist[0];
+    } else {
+        // still returns a "StringList" despite the different name
+        XmbTextPropertyToTextList(display(), &text_prop, &stringlist, &count);
+        if (count == 0 || stringlist == 0) {
+            return "";
+        }
+        ret = FbStringUtil::LocaleStrToFb(stringlist[0]);
+    }
+
+    // they all use stringlist
+    if (stringlist) {
+        XFreeStringList(stringlist);
+    }
+
     return ret;
 }
 
@@ -501,6 +556,14 @@ void FbWindow::deleteProperty(Atom property) {
     XDeleteProperty(display(), m_window, property);
 }
 
+void FbWindow::addToSaveSet() {
+    XAddToSaveSet(display(), m_window);
+}
+
+void FbWindow::removeFromSaveSet() {
+    XRemoveFromSaveSet(display(), m_window);
+}
+
 int FbWindow::screenNumber() const {
     return m_screen_num;
 }
@@ -516,7 +579,7 @@ long FbWindow::eventMask() const {
 void FbWindow::setOpaque(unsigned char alpha) {
 #ifdef HAVE_XRENDER
     static Atom m_alphaatom = XInternAtom(display(), "_NET_WM_WINDOW_OPACITY", False);
-    unsigned int opacity = alpha << 24;
+    unsigned long opacity = alpha * 0x1010101;
     changeProperty(m_alphaatom, XA_CARDINAL, 32, PropModeReplace, (unsigned char *) &opacity, 1l);
 #endif // HAVE_XRENDER
 }
@@ -527,10 +590,10 @@ void FbWindow::updateGeometry() {
 
     Window root;
     unsigned int border_width, depth;
-    XGetGeometry(display(), m_window, &root, &m_x, &m_y,
-                 (unsigned int *)&m_width, (unsigned int *)&m_height,
-                 &border_width, &depth);
-    m_depth = depth;
+    if (XGetGeometry(display(), m_window, &root, &m_x, &m_y,
+                     (unsigned int *)&m_width, (unsigned int *)&m_height,
+                     &border_width, &depth))
+        m_depth = depth;
 }
 
 void FbWindow::create(Window parent, int x, int y,
@@ -592,6 +655,29 @@ void FbWindow::sendConfigureNotify(int x, int y,
 
 }
 
+FbWindow::FbWinList FbWindow::m_alpha_wins;
+
+void FbWindow::addAlphaWin(FbWindow &win) {
+    m_alpha_wins.insert(&win);
+}
+
+void FbWindow::removeAlphaWin(FbWindow &win) {
+    FbWinList::iterator it = m_alpha_wins.find(&win);
+    if (it != m_alpha_wins.end())
+        m_alpha_wins.erase(it);
+}
+
+void FbWindow::updatedAlphaBackground(int screen) {
+    FbWinList::iterator it = m_alpha_wins.begin();
+    FbWinList::iterator it_end = m_alpha_wins.end();
+    for (; it != it_end; ++it) {
+        if ((*it)->screenNumber() == screen) {
+            (*it)->updateBackground(false);
+            (*it)->clear();
+        }
+    }
+}
+
 bool operator == (Window win, const FbWindow &fbwin) {
     return win == fbwin.window();
 }
