@@ -1,5 +1,5 @@
 // ImageControl.cc for FbTk - Fluxbox Toolkit
-// Copyright (c) 2001 - 2003 Henrik Kinnunen (fluxbox at users.sourceforge.net)
+// Copyright (c) 2001 - 2006 Henrik Kinnunen (fluxbox at users.sourceforge.net)
 //
 // Image.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ImageControl.cc 3700 2004-11-19 11:37:27Z mathias $
+// $Id: ImageControl.cc 4857 2007-04-17 07:52:40Z mathias $
 
 #include "ImageControl.hh"
 
@@ -61,13 +61,11 @@
   #include <stdio.h>
 #endif
 
-#ifdef HAVE_CTYPE_H
-#include <ctype.h>
-#endif // HAVE_CTYPE_H
-
 #include <iostream>
 
-using namespace std;
+using std::cerr;
+using std::endl;
+using std::list;
 
 namespace FbTk {
 
@@ -166,7 +164,7 @@ ImageControl::~ImageControl() {
 
 
 Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
-                                 const Texture &text) const {
+                                 const Texture &text, FbTk::Orientation orient) const {
 
     if (text.pixmap().drawable() != None) {
         // do comparsion with width/height and texture_pixmap
@@ -174,7 +172,9 @@ Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
         CacheList::iterator it_end = cache.end();
         for (; it != it_end; ++it) {
             if ((*it)->texture_pixmap == text.pixmap().drawable() &&
-                (*it)->width == width && (*it)->height == height &&
+                (*it)->orient == orient &&
+                (*it)->width == width &&
+                (*it)->height == height &&
                 (*it)->texture == text.type()) {
                 (*it)->count++;
                 return (*it)->pixmap;
@@ -196,6 +196,7 @@ Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
     CacheList::iterator it_end = cache.end();
     for (; it != it_end; ++it) {
         if (((*it)->width == width) &&
+            ((*it)->orient == orient) &&
             ((*it)->height == height) &&
             ((*it)->texture == text.type()) &&
             ((*it)->pixel1 == text.color().pixel())) {
@@ -217,19 +218,21 @@ Pixmap ImageControl::searchCache(unsigned int width, unsigned int height,
 
 
 Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
-                                 const FbTk::Texture &texture) {
+                                 const FbTk::Texture &texture,
+                                 FbTk::Orientation orient) {
 
     if (texture.type() & FbTk::Texture::PARENTRELATIVE)
         return ParentRelative;
 
     // search cache first
-    Pixmap pixmap = searchCache(width, height, texture);
+    Pixmap pixmap = searchCache(width, height, texture, orient);
     if (pixmap) {
         return pixmap; // return cache item
     }
 
     // render new image
-    TextureRender image(*this, width, height, m_colors, m_num_colors);
+
+    TextureRender image(*this, width, height, orient, m_colors, m_num_colors);
     pixmap = image.render(texture);
 
     if (pixmap) {
@@ -239,6 +242,7 @@ Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
 
         tmp->pixmap = pixmap;
         tmp->texture_pixmap = texture.pixmap().drawable();
+        tmp->orient = orient;
         tmp->width = width;
         tmp->height = height;
         tmp->count = 1;
@@ -252,7 +256,7 @@ Pixmap ImageControl::renderImage(unsigned int width, unsigned int height,
 
         cache.push_back(tmp);
 
-        if ((unsigned) cache.size() > cache_max)
+        if (cache.size() > cache_max)
             cleanCache();
 
         return pixmap;
@@ -305,12 +309,12 @@ void ImageControl::colorTables(const unsigned char **rmt, const unsigned char **
     if (bbit) *bbit = blue_bits;
 }
 
-
+#ifdef NOT_USED
 void ImageControl::getXColorTable(XColor **c, int *n) {
     if (c) *c = m_colors;
     if (n) *n = m_num_colors;
 }
-
+#endif
 
 void ImageControl::getGradientBuffers(unsigned int w,
                                       unsigned int h,
@@ -367,14 +371,14 @@ void ImageControl::installRootColormap() {
     XUngrabServer(disp);
 }
 
-
+#ifdef NOT_USED
 void ImageControl::setColorsPerChannel(int cpc) {
     if (cpc < 2) cpc = 2;
     if (cpc > 6) cpc = 6;
 
     m_colors_per_channel = cpc;
 }
-
+#endif
 
 unsigned long ImageControl::getSqrt(unsigned int x) const {
     if (! sqrt_table) {
@@ -392,7 +396,7 @@ unsigned long ImageControl::getSqrt(unsigned int x) const {
 
 void ImageControl::cleanCache() {
     Display *disp = FbTk::App::instance()->display();
-    std::list<CacheList::iterator> deadlist;
+    list<CacheList::iterator> deadlist;
     CacheList::iterator it = cache.begin();
     CacheList::iterator it_end = cache.end();
     for (; it != it_end; ++it) {
@@ -405,8 +409,8 @@ void ImageControl::cleanCache() {
         }
     }
 
-    std::list<CacheList::iterator>::iterator dead_it = deadlist.begin();
-    std::list<CacheList::iterator>::iterator dead_it_end = deadlist.end();
+    list<CacheList::iterator>::iterator dead_it = deadlist.begin();
+    list<CacheList::iterator>::iterator dead_it_end = deadlist.end();
     for (; dead_it != dead_it_end; ++dead_it) {
         cache.erase(*dead_it);
     }
@@ -654,7 +658,7 @@ void ImageControl::createColorTable() {
 
     default:
         _FB_USES_NLS;
-        cerr<<"FbTk::ImageControl: "<<_FBTKTEXT(Error, UnsupportedVisual, "Unsupported visual", "A visual is a technical term in X")<<endl;
+        cerr<<"FbTk::ImageControl: "<<_FBTK_CONSOLETEXT(Error, UnsupportedVisual, "Unsupported visual", "A visual is a technical term in X")<<endl;
         break;
     }
 }
