@@ -19,11 +19,12 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Resource.cc 4199 2006-02-16 06:53:05Z mathias $
+// $Id: Resource.cc 4833 2007-04-03 17:14:00Z markt $
 
 #include "XrmDatabaseHelper.hh"
 #include "Resource.hh"
 #include "I18n.hh"
+#include "StringUtil.hh"
 
 #include <iostream>
 #ifdef HAVE_CASSERT
@@ -32,7 +33,9 @@
   #include <assert.h>
 #endif
 
-using namespace std;
+using std::cerr;
+using std::endl;
+using std::string;
 
 namespace FbTk {
 
@@ -55,11 +58,11 @@ ResourceManager::~ResourceManager() {
 bool ResourceManager::m_init = false;
 
 /**
-  reloads all resources from resourcefile 
+  reloads all resources from resourcefile
   @return true on success else false
 */
 bool ResourceManager::load(const char *filename) {
-    m_filename = filename;
+    m_filename = StringUtil::expandFilename(filename).c_str();
 
     // force reload (lock will ensure it exists)
     if (m_database) {
@@ -72,23 +75,23 @@ bool ResourceManager::load(const char *filename) {
         unlock();
         return false;
     }
-        
+
     XrmValue value;
     char *value_type;
-	
+
     //get list and go throu all the resources and load them
     ResourceList::iterator i = m_resourcelist.begin();
-    ResourceList::iterator i_end = m_resourcelist.end();	
+    ResourceList::iterator i_end = m_resourcelist.end();
     for (; i != i_end; ++i) {
-	
+
         Resource_base *resource = *i;
         if (XrmGetResource(**m_database, resource->name().c_str(),
-                           resource->altName().c_str(), &value_type, &value))			
+                           resource->altName().c_str(), &value_type, &value))
             resource->setFromString(value.addr);
         else {
             _FB_USES_NLS;
-            cerr<<_FBTKTEXT(Error, FailedRead, "Failed to read", "Couldn't load a resource (following)")<<": "<<resource->name()<<endl;
-            cerr<<_FBTKTEXT(Error, UsingDefault, "Setting default value", "Falling back to default value for resource")<<endl;
+            cerr<<_FBTK_CONSOLETEXT(Error, FailedRead, "Failed to read", "Couldn't load a resource (following)")<<": "<<resource->name()<<endl;
+            cerr<<_FBTK_CONSOLETEXT(Error, UsingDefault, "Setting default value", "Falling back to default value for resource")<<endl;
             resource->setDefaultValue();
         }
     }
@@ -104,11 +107,20 @@ bool ResourceManager::load(const char *filename) {
 */
 bool ResourceManager::save(const char *filename, const char *mergefilename) {
     assert(filename);
-    
+
+    // these must be local variables; otherwise, the memory gets released by
+    // std::string, causing weird issues
+    string file_str = StringUtil::expandFilename(filename), mergefile_str;
+    filename = file_str.c_str();
+    if (mergefilename) {
+        mergefile_str = StringUtil::expandFilename(mergefilename);
+        mergefilename = mergefile_str.c_str();
+    }
+
     // empty database
     XrmDatabaseHelper database;
 
-    string rc_string;	
+    string rc_string;
     ResourceList::iterator i = m_resourcelist.begin();
     ResourceList::iterator i_end = m_resourcelist.end();
     //write all resources to database
@@ -125,7 +137,7 @@ bool ResourceManager::save(const char *filename, const char *mergefilename) {
     if (mergefilename) {
         // force reload of file
         m_filename = mergefilename;
-        if (m_database) 
+        if (m_database)
             delete m_database;
         m_database = 0;
 
@@ -149,31 +161,31 @@ bool ResourceManager::save(const char *filename, const char *mergefilename) {
     return true;
 }
 
-Resource_base *ResourceManager::findResource(const std::string &resname) {
+Resource_base *ResourceManager::findResource(const string &resname) {
    // find resource name
     ResourceList::iterator i = m_resourcelist.begin();
     ResourceList::iterator i_end = m_resourcelist.end();
     for (; i != i_end; ++i) {
         if ((*i)->name() == resname ||
-            (*i)->altName() == resname) 
+            (*i)->altName() == resname)
             return *i;
     }
     return 0;
 }
 
-const Resource_base *ResourceManager::findResource(const std::string &resname) const {
+const Resource_base *ResourceManager::findResource(const string &resname) const {
    // find resource name
     ResourceList::const_iterator i = m_resourcelist.begin();
     ResourceList::const_iterator i_end = m_resourcelist.end();
     for (; i != i_end; ++i) {
         if ((*i)->name() == resname ||
-            (*i)->altName() == resname) 
+            (*i)->altName() == resname)
             return *i;
     }
     return 0;
 }
 
-string ResourceManager::resourceValue(const std::string &resname) const {
+string ResourceManager::resourceValue(const string &resname) const {
     const Resource_base *res = findResource(resname);
     if (res != 0)
         return res->getString();
@@ -181,7 +193,7 @@ string ResourceManager::resourceValue(const std::string &resname) const {
     return "";
 }
 
-void ResourceManager::setResourceValue(const std::string &resname, const std::string &value) {
+void ResourceManager::setResourceValue(const string &resname, const string &value) {
     Resource_base *res = findResource(resname);
     if (res != 0)
         res->setFromString(value.c_str());
