@@ -1,5 +1,5 @@
 // TextureRender.cc for fluxbox
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // from Image.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: TextureRender.cc 3864 2005-01-24 18:02:34Z mathias $
+// $Id: TextureRender.cc 4945 2007-06-29 17:25:24Z markt $
 
 #include "TextureRender.hh"
 
@@ -39,7 +39,11 @@
 #else
   #include <stdio.h>
 #endif
-using namespace std;
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 
 // mipspro has no new(nothrow)
 #if defined sgi && ! defined GCC
@@ -48,10 +52,17 @@ using namespace std;
 #define FB_new_nothrow new(std::nothrow)
 #endif
 
+using std::cerr;
+using std::endl;
+using std::string;
+using std::max;
+using std::min;
+
 namespace FbTk {
 
-TextureRender::TextureRender(ImageControl &imgctrl, 
-                             unsigned int w, unsigned int h, 
+TextureRender::TextureRender(ImageControl &imgctrl,
+                             unsigned int w, unsigned int h,
+                             FbTk::Orientation orient,
                              XColor *_colors, size_t num_colors):
     control(imgctrl),
     colors(_colors),
@@ -59,33 +70,37 @@ TextureRender::TextureRender(ImageControl &imgctrl,
     cpc(imgctrl.colorsPerChannel()),
     cpccpc(cpc * cpc),
     red(0), green(0), blue(0),
+    orientation(orient),
     width(static_cast<signed>((w > 0 ? w : 1))), height(static_cast<signed>(h > 0 ? h : 1)),
     xtable(0), ytable(0) {
 
+    unsigned int texture_max_width = WidthOfScreen(ScreenOfDisplay(FbTk::App::instance()->display(), imgctrl.screenNumber())) * 2;
+    unsigned int texture_max_height = HeightOfScreen(ScreenOfDisplay(FbTk::App::instance()->display(), imgctrl.screenNumber())) * 2;
+
     _FB_USES_NLS;
     // clamp to "normal" size
-    if (width > 3200) {
-        cerr<<"TextureRender: "<<_FBTKTEXT(Error, BigWidth, "Warning! Width > 3200 setting Width = 3200", "Image width seems too big, clamping")<<endl;
-        width = 3200;
+    if (width > texture_max_width) {
+        cerr<<"TextureRender: "<<_FBTK_CONSOLETEXT(Error, BigWidth, "Warning! Width > 3200 setting Width = 3200", "Image width seems too big, clamping")<<endl;
+        width = texture_max_width;
     }
 
-    if (height > 3200) {
-        cerr<<"TextureRender: "<<_FBTKTEXT(Error, BigHeight, "Warning! Height > 3200 setting Height = 3200", "Image height seems too big, clamping")<<endl;
-        height = 3200;
+    if (height > texture_max_height) {
+        cerr<<"TextureRender: "<<_FBTK_CONSOLETEXT(Error, BigHeight, "Warning! Height > 3200 setting Height = 3200", "Image height seems too big, clamping")<<endl;
+        height = texture_max_height;
     }
 
 
     imgctrl.colorTables(&red_table, &green_table, &blue_table,
                         &red_offset, &green_offset, &blue_offset,
                         &red_bits, &green_bits, &blue_bits);
-	
+
 }
 
 
 TextureRender::~TextureRender() {
     if (red != 0) delete [] red;
     if (green != 0) delete [] green;
-    if (blue != 0) delete [] blue;	
+    if (blue != 0) delete [] blue;
 }
 
 
@@ -114,8 +129,8 @@ void TextureRender::allocateColorTables() {
     if (red == 0) {
         char sbuf[128];
         sprintf(sbuf, "%ld", (long int) size);
-        throw std::string("TextureRender::TextureRender(): " + 
-              std::string(_FBTKTEXT(Error, OutOfMemoryRed, "Out of memory while allocating red buffer.", "")) + string(sbuf));
+        throw string("TextureRender::TextureRender(): " +
+              string(_FBTK_CONSOLETEXT(Error, OutOfMemoryRed, "Out of memory while allocating red buffer.", "")) + string(sbuf));
     }
 
 
@@ -123,16 +138,16 @@ void TextureRender::allocateColorTables() {
     if (green == 0) {
         char sbuf[128];
         sprintf(sbuf, "%ld", (long int) size);
-        throw std::string("TextureRender::TextureRender(): " + 
-              std::string(_FBTKTEXT(Error, OutOfMemoryGreen, "Out of memory while allocating green buffer.", ""))+ string(sbuf));
+        throw string("TextureRender::TextureRender(): " +
+              string(_FBTK_CONSOLETEXT(Error, OutOfMemoryGreen, "Out of memory while allocating green buffer.", ""))+ string(sbuf));
     }
 
     blue = FB_new_nothrow unsigned char[size];
     if (blue == 0) {
         char sbuf[128];
         sprintf(sbuf, "%ld", (long int) size);
-        throw std::string("TextureRender::TextureRender(): " +
-              std::string(_FBTKTEXT(Error, OutOfMemoryBlue, "Out of memory while allocating blue buffer.", ""))+ string(sbuf));
+        throw string("TextureRender::TextureRender(): " +
+              string(_FBTK_CONSOLETEXT(Error, OutOfMemoryBlue, "Out of memory while allocating blue buffer.", ""))+ string(sbuf));
     }
 
 
@@ -141,18 +156,18 @@ void TextureRender::allocateColorTables() {
 Pixmap TextureRender::renderSolid(const FbTk::Texture &texture) {
 
     FbPixmap pixmap(RootWindow(FbTk::App::instance()->display(),
-                               control.screenNumber()), 
+                               control.screenNumber()),
                     width, height,
                     control.depth());
 
     if (pixmap.drawable() == None) {
         _FB_USES_NLS;
-        cerr<<"FbTk::TextureRender::render_solid(): "<<_FBTKTEXT(Error, CreatePixmap, "Error creating pixmap", "Couldn't create a pixmap - image - for some reason")<<endl;
+        cerr<<"FbTk::TextureRender::render_solid(): "<<_FBTK_CONSOLETEXT(Error, CreatePixmap, "Error creating pixmap", "Couldn't create a pixmap - image - for some reason")<<endl;
         return None;
     }
 
 
-    FbTk::GContext gc(pixmap), 
+    FbTk::GContext gc(pixmap),
         hgc(pixmap), lgc(pixmap);
 
     gc.setForeground(texture.color());
@@ -161,9 +176,9 @@ Pixmap TextureRender::renderSolid(const FbTk::Texture &texture) {
     hgc.setForeground(texture.hiColor());
 
     pixmap.fillRectangle(gc.gc(), 0, 0, width, height);
-	
+
     using namespace FbTk;
-	
+
     if (texture.type() & Texture::INTERLACED) {
         lgc.setForeground(texture.colorTo());
         register unsigned int i = 0;
@@ -176,7 +191,7 @@ Pixmap TextureRender::renderSolid(const FbTk::Texture &texture) {
 
     if (texture.type() & Texture::BEVEL1) {
         if (texture.type() & Texture::RAISED) {
-            pixmap.drawLine(lgc.gc(), 
+            pixmap.drawLine(lgc.gc(),
                             0, height - 1, width - 1, height - 1);
             pixmap.drawLine(lgc.gc(),
                             width - 1, height - 1, width - 1, 0);
@@ -228,6 +243,9 @@ Pixmap TextureRender::renderGradient(const FbTk::Texture &texture) {
 
     bool inverted = false;
 
+    // invert our width and height if necessary
+    translateSize(orientation, width, height);
+
     using namespace FbTk;
 
     interlaced = texture.type() & Texture::INTERLACED;
@@ -236,33 +254,33 @@ Pixmap TextureRender::renderGradient(const FbTk::Texture &texture) {
         from = &(texture.colorTo());
         to = &(texture.color());
 
-        if (! (texture.type() & Texture::INVERT)) 
+        if (! (texture.type() & Texture::INVERT))
             inverted = true;
     } else {
         from = &(texture.color());
         to = &(texture.colorTo());
 
-        if (texture.type() & Texture::INVERT) 
+        if (texture.type() & Texture::INVERT)
             inverted = true;
     }
 
     control.getGradientBuffers(width, height, &xtable, &ytable);
 
-    if (texture.type() & Texture::DIAGONAL) 
+    if (texture.type() & Texture::DIAGONAL)
         dgradient();
-    else if (texture.type() & Texture::ELLIPTIC) 
+    else if (texture.type() & Texture::ELLIPTIC)
         egradient();
-    else if (texture.type() & Texture::HORIZONTAL) 
+    else if (texture.type() & Texture::HORIZONTAL)
         hgradient();
-    else if (texture.type() & Texture::PYRAMID) 
+    else if (texture.type() & Texture::PYRAMID)
         pgradient();
-    else if (texture.type() & Texture::RECTANGLE) 
+    else if (texture.type() & Texture::RECTANGLE)
         rgradient();
-    else if (texture.type() & Texture::VERTICAL) 
+    else if (texture.type() & Texture::VERTICAL)
         vgradient();
-    else if (texture.type() & Texture::CROSSDIAGONAL) 
+    else if (texture.type() & Texture::CROSSDIAGONAL)
         cdgradient();
-    else if (texture.type() & Texture::PIPECROSS) 
+    else if (texture.type() & Texture::PIPECROSS)
         pcgradient();
 
     if (texture.type() & Texture::BEVEL1)
@@ -278,23 +296,30 @@ Pixmap TextureRender::renderGradient(const FbTk::Texture &texture) {
 }
 
 Pixmap TextureRender::renderPixmap(const FbTk::Texture &src_texture) {
-    if (width != src_texture.pixmap().width() ||
-        height != src_texture.pixmap().height()) {
+    unsigned int tmpw = width, tmph = height;
+    // we are given width and height in rotated form, we
+    // unrotate it here to render it
+    translateSize(orientation, tmpw, tmph);
+    if (tmpw != src_texture.pixmap().width() ||
+        tmph != src_texture.pixmap().height()) {
 
-        // copy src_texture's pixmap and 
+        // copy src_texture's pixmap and
         // scale/tile to fit our size
         FbPixmap new_pm(src_texture.pixmap());
- 
-        if ((src_texture.type() & Texture::TILED)) {
-            new_pm.tile(width,height);
-        } else {
-            new_pm.scale(width, height);
-        }
 
+        if ((src_texture.type() & Texture::TILED)) {
+            new_pm.tile(tmpw,tmph);
+        } else {
+            new_pm.scale(tmpw, tmph);
+        }
+        new_pm.rotate(orientation);
         return new_pm.release();
     }
     // return copy of pixmap
-    return FbPixmap(src_texture.pixmap()).release();
+    FbPixmap pm_copy = FbPixmap(src_texture.pixmap());
+    pm_copy.rotate(orientation);
+
+    return pm_copy.release();
 }
 
 XImage *TextureRender::renderXImage() {
@@ -306,7 +331,7 @@ XImage *TextureRender::renderXImage() {
 
     if (! image) {
         _FB_USES_NLS;
-        cerr<<"FbTk::TextureRender::renderXImage(): "<<_FBTKTEXT(Error, CreateXImage, "Error creating XImage", "Couldn't create an XImage")<<endl;
+        cerr << "FbTk::TextureRender::renderXImage(): " << _FBTK_CONSOLETEXT(Error, CreateXImage, "Can't create XImage", "Couldn't create an XImage") << "." << endl;
         return 0;
     }
 
@@ -321,14 +346,14 @@ XImage *TextureRender::renderXImage() {
     o = image->bits_per_pixel + ((image->byte_order == MSBFirst) ? 1 : 0);
 
     if (control.doDither()) {
-        unsigned char dither4[4][4] = { 
+        unsigned char dither4[4][4] = {
             {0, 4, 1, 5},
             {6, 2, 7, 3},
             {1, 5, 0, 4},
             {7, 3, 6, 2} };
 
 #ifdef ORDEREDPSEUDO
-        unsigned char dither8[8][8] = { 
+        unsigned char dither8[8][8] = {
             { 0, 32,  8, 40, 2,	34, 10, 42 },
             { 48, 16, 56, 24, 50, 18, 58, 26 },
             { 12, 44,  4, 36, 14, 46, 6, 38 },
@@ -578,8 +603,8 @@ XImage *TextureRender::renderXImage() {
 
     default:
         _FB_USES_NLS;
-        cerr<<"TextureRender::renderXImage(): "<<
-            _FBTKTEXT(Error, UnsupportedVisual, "unsupported visual", "A visual is a technical term in X")<<endl;
+        cerr << "TextureRender::renderXImage(): " <<
+            _FBTK_CONSOLETEXT(Error, UnsupportedVisual, "Unsupported visual", "A visual is a technical term in X") << endl;
         delete [] d;
         XDestroyImage(image);
         return (XImage *) 0;
@@ -681,8 +706,8 @@ XImage *TextureRender::renderXImage() {
 
     default:
         _FB_USES_NLS;
-        cerr<<"TextureRender::renderXImage(): "<<
-            _FBTKTEXT(Error, UnsupportedVisual, "unsupported visual", "A visual is a technical term in X")<<endl;
+        cerr << "TextureRender::renderXImage(): " <<
+            _FBTK_CONSOLETEXT(Error, UnsupportedVisual, "Unsupported visual", "A visual is a technical term in X") << endl;
         delete [] d;
         XDestroyImage(image);
         return (XImage *) 0;
@@ -696,12 +721,12 @@ return image;
 
 Pixmap TextureRender::renderPixmap() {
     Display *disp = FbTk::App::instance()->display();
-    FbPixmap pixmap(RootWindow(disp, control.screenNumber()), 
+    FbPixmap pixmap(RootWindow(disp, control.screenNumber()),
                     width, height, control.depth());
 
     if (pixmap.drawable() == None) {
         _FB_USES_NLS;
-        cerr<<"FbTk::TextureRender::renderPixmap(): "<<_FBTKTEXT(Error, CreatePixmap, "Error creating pixmap", "Couldn't create a pixmap - image - for some reason")<<endl;
+        cerr<<"FbTk::TextureRender::renderPixmap(): "<<_FBTK_CONSOLETEXT(Error, CreatePixmap, "Error creating pixmap", "Couldn't create a pixmap - image - for some reason")<<endl;
         return None;
     }
 
@@ -725,6 +750,8 @@ Pixmap TextureRender::renderPixmap() {
 
     XDestroyImage(image);
 
+    pixmap.rotate(orientation);
+
     return pixmap.release();
 }
 
@@ -1424,9 +1451,9 @@ void TextureRender::rgradient() {
         // normal rgradient
         for (yt = ytable, y = 0; y < height; y++, yt += 3) {
             for (xt = xtable, x = 0; x < width; x++) {
-                *(pr++) = (unsigned char) (tr - (rsign * std::max(*(xt++), *(yt))));
-                *(pg++) = (unsigned char) (tg - (gsign * std::max(*(xt++), *(yt + 1))));
-                *(pb++) = (unsigned char) (tb - (bsign * std::max(*(xt++), *(yt + 2))));
+                *(pr++) = (unsigned char) (tr - (rsign * max(*(xt++), *(yt))));
+                *(pg++) = (unsigned char) (tg - (gsign * max(*(xt++), *(yt + 1))));
+                *(pb++) = (unsigned char) (tb - (bsign * max(*(xt++), *(yt + 2))));
             }
         }
 
@@ -1437,32 +1464,32 @@ void TextureRender::rgradient() {
         for (yt = ytable, y = 0; y < height; y++, yt += 3) {
             for (xt = xtable, x = 0; x < width; x++) {
                 if (y & 1) {
-                    channel = (unsigned char) (tr - (rsign * std::max(*(xt++), *(yt))));
+                    channel = (unsigned char) (tr - (rsign * max(*(xt++), *(yt))));
                     channel2 = (channel >> 1) + (channel >> 2);
                     if (channel2 > channel) channel2 = 0;
                     *(pr++) = channel2;
 
-                    channel = (unsigned char) (tg - (gsign * std::max(*(xt++), *(yt + 1))));
+                    channel = (unsigned char) (tg - (gsign * max(*(xt++), *(yt + 1))));
                     channel2 = (channel >> 1) + (channel >> 2);
                     if (channel2 > channel) channel2 = 0;
                     *(pg++) = channel2;
 
-                    channel = (unsigned char) (tb - (bsign * std::max(*(xt++), *(yt + 2))));
+                    channel = (unsigned char) (tb - (bsign * max(*(xt++), *(yt + 2))));
                     channel2 = (channel >> 1) + (channel >> 2);
                     if (channel2 > channel) channel2 = 0;
                     *(pb++) = channel2;
                 } else {
-                    channel = (unsigned char) (tr - (rsign * std::max(*(xt++), *(yt))));
+                    channel = (unsigned char) (tr - (rsign * max(*(xt++), *(yt))));
                     channel2 = channel + (channel >> 3);
                     if (channel2 < channel) channel2 = ~0;
                     *(pr++) = channel2;
 
-                    channel = (unsigned char) (tg - (gsign * std::max(*(xt++), *(yt + 1))));
+                    channel = (unsigned char) (tg - (gsign * max(*(xt++), *(yt + 1))));
                     channel2 = channel + (channel >> 3);
                     if (channel2 < channel) channel2 = ~0;
                     *(pg++) = channel2;
 
-                    channel = (unsigned char) (tb - (bsign * std::max(*(xt++), *(yt + 2))));
+                    channel = (unsigned char) (tb - (bsign * max(*(xt++), *(yt + 2))));
                     channel2 = channel + (channel >> 3);
                     if (channel2 < channel) channel2 = ~0;
                     *(pb++) = channel2;
@@ -1658,9 +1685,9 @@ void TextureRender::pcgradient() {
         // normal pcgradient
         for (yt = ytable, y = 0; y < height; y++, yt += 3) {
             for (xt = xtable, x = 0; x < width; x++) {
-                *(pr++) = (unsigned char) (tr - (rsign * std::min(*(xt++), *(yt))));
-                *(pg++) = (unsigned char) (tg - (gsign * std::min(*(xt++), *(yt + 1))));
-                *(pb++) = (unsigned char) (tb - (bsign * std::min(*(xt++), *(yt + 2))));
+                *(pr++) = (unsigned char) (tr - (rsign * min(*(xt++), *(yt))));
+                *(pg++) = (unsigned char) (tg - (gsign * min(*(xt++), *(yt + 1))));
+                *(pb++) = (unsigned char) (tb - (bsign * min(*(xt++), *(yt + 2))));
             }
         }
 
@@ -1671,32 +1698,32 @@ void TextureRender::pcgradient() {
         for (yt = ytable, y = 0; y < height; y++, yt += 3) {
             for (xt = xtable, x = 0; x < width; x++) {
                 if (y & 1) {
-                    channel = (unsigned char) (tr - (rsign * std::min(*(xt++), *(yt))));
+                    channel = (unsigned char) (tr - (rsign * min(*(xt++), *(yt))));
                     channel2 = (channel >> 1) + (channel >> 2);
                     if (channel2 > channel) channel2 = 0;
                     *(pr++) = channel2;
 
-                    channel = (unsigned char) (tg - (bsign * std::min(*(xt++), *(yt + 1))));
+                    channel = (unsigned char) (tg - (bsign * min(*(xt++), *(yt + 1))));
                     channel2 = (channel >> 1) + (channel >> 2);
                     if (channel2 > channel) channel2 = 0;
                     *(pg++) = channel2;
 
-                    channel = (unsigned char) (tb - (gsign * std::min(*(xt++), *(yt + 2))));
+                    channel = (unsigned char) (tb - (gsign * min(*(xt++), *(yt + 2))));
                     channel2 = (channel >> 1) + (channel >> 2);
                     if (channel2 > channel) channel2 = 0;
                     *(pb++) = channel2;
                 } else {
-                    channel = (unsigned char) (tr - (rsign * std::min(*(xt++), *(yt))));
+                    channel = (unsigned char) (tr - (rsign * min(*(xt++), *(yt))));
                     channel2 = channel + (channel >> 3);
                     if (channel2 < channel) channel2 = ~0;
                     *(pr++) = channel2;
 
-                    channel = (unsigned char) (tg - (gsign * std::min(*(xt++), *(yt + 1))));
+                    channel = (unsigned char) (tg - (gsign * min(*(xt++), *(yt + 1))));
                     channel2 = channel + (channel >> 3);
                     if (channel2 < channel) channel2 = ~0;
                     *(pg++) = channel2;
 
-                    channel = (unsigned char) (tb - (bsign * std::min(*(xt++), *(yt + 2))));
+                    channel = (unsigned char) (tb - (bsign * min(*(xt++), *(yt + 2))));
                     channel2 = channel + (channel >> 3);
                     if (channel2 < channel) channel2 = ~0;
                     *(pb++) = channel2;
