@@ -30,14 +30,23 @@
 #include "fluxbox.hh"
 #include "FbWinFrameTheme.hh"
 
+#include "FbTk/EventManager.hh"
+
 #include <string>
-#include <cassert>
 #include <iostream>
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+
 using std::cerr;
 using std::endl;
 using std::string;
 
 WinClient *FocusControl::s_focused_window = 0;
+FluxboxWindow *FocusControl::s_focused_fbwindow = 0;
+bool FocusControl::s_reverting = false;
 
 FocusControl::FocusControl(BScreen &screen):
     m_screen(screen),
@@ -49,13 +58,11 @@ FocusControl::FocusControl(BScreen &screen):
                       CLICKTABFOCUS, 
                       screen.name()+".tabFocusModel", 
                       screen.altName()+".TabFocusModel"),
-    m_focus_last(screen.resourceManager(), true, 
-                 screen.name()+".focusLastWindow", 
-                 screen.altName()+".FocusLastWindow"),
     m_focus_new(screen.resourceManager(), true, 
                 screen.name()+".focusNewWindows", 
                 screen.altName()+".FocusNewWindows"),
-    m_cycling_focus(false),
+    m_cycling_list(0),
+    m_was_iconic(false),
     m_cycling_last(0) {
 
     m_cycling_window = m_focused_list.end();
@@ -72,134 +79,150 @@ bool doSkipWindow(const WinClient &winclient, int opts) {
     (opts & FocusControl::CYCLEGROUPS) != 0 && win->winClient().window() != winclient.window() ||
     // skip if shaded
     (opts & FocusControl::CYCLESKIPSHADED) != 0 && win->isShaded() || 
+    // skip if iconic
+    (opts & FocusControl::CYCLESKIPICONIC) != 0 && win->isIconic() ||
     // skip if hidden
     win->isFocusHidden()
     ); 
 }
 
-void FocusControl::prevFocus(int opts) {
-    int num_windows = m_screen.currentWorkspace()->numberOfWindows();
-	
-    if (num_windows < 1)
-        return;
+void FocusControl::cycleFocus(FocusedWindows &window_list, int opts, bool cycle_reverse) {
 
-    if (!(opts & CYCLELINEAR)) {
-        if (!m_cycling_focus) {
-            m_cycling_focus = true;
-            m_cycling_window = m_focused_list.end();
-            m_cycling_last = 0;
-        } else {
-            // already cycling, so restack to put windows back in their proper order
-            m_screen.layerManager().restack();
-        }
-        // if it is stacked, we want the highest window in the focused list
-        // that is on the same workspace
-        FocusedWindows::iterator it = m_cycling_window;
-        FocusedWindows::iterator it_end = m_focused_list.end();
+    if (!m_cycling_list) {
+        if (&m_screen == FbTk::EventManager::instance()->grabbingKeyboard())
+            // only set this when we're waiting for modifiers
+            m_cycling_list = &window_list;
+        m_was_iconic = 0;
+        m_cycling_last = 0;
+    } else if (m_cycling_list != &window_list)
+        m_cycling_list = &window_list;
 
-        while (true) {
-            --it;
-            if (it == it_end) {
-                it = m_focused_list.end();
-                --it;
-            }
-            // give up [do nothing] if we reach the current focused again
-            if ((*it) == (*m_cycling_window)) {
-                break;
-            }
+    FocusedWindows::iterator it_begin = window_list.begin();
+    FocusedWindows::iterator it_end = window_list.end();
 
-            FluxboxWindow *fbwin = (*it)->fbwindow();
-            if (fbwin && !fbwin->isIconic() &&
-                (fbwin->isStuck() 
-                 || fbwin->workspaceNumber() == m_screen.currentWorkspaceID())) {
-                // either on this workspace, or stuck
+    // too many things can go wrong with remembering this
+    m_cycling_window = find(it_begin, it_end, s_focused_window);
 
-                // keep track of the originally selected window in a set
-                WinClient &last_client = fbwin->winClient();
+    FocusedWindows::iterator it = m_cycling_window;
+    FluxboxWindow *fbwin = 0;
+    WinClient *last_client = 0;
+    WinClient *was_iconic = 0;
 
+    // find the next window in the list that works
+    while (true) {
+        if (cycle_reverse && it == it_begin)
+            it = it_end;
+        else if (!cycle_reverse && it == it_end)
+            it = it_begin;
+        else
+            cycle_reverse ? --it : ++it;
+        // give up [do nothing] if we reach the current focused again
+        if (it == m_cycling_window)
+            return;
+        if (it == it_end)
+            continue;
 
-                if (! (doSkipWindow(**it, opts) || !fbwin->setCurrentClient(**it)) ) {
-                    // moved onto a new fbwin
-                    if (!m_cycling_last || m_cycling_last->fbwindow() != fbwin) {
-                        if (m_cycling_last) {
-                            // set back to orig current Client in that fbwin
-                            m_cycling_last->fbwindow()->setCurrentClient(*m_cycling_last, false);
-                        }
-                        m_cycling_last = &last_client;
-                    }
-                    fbwin->tempRaise();
-                    break;
-                }
-            }
-        }
-        m_cycling_window = it;
-    } else { // not stacked cycling
-            
-        Workspace &wksp = *m_screen.currentWorkspace();
-        Workspace::Windows &wins = wksp.windowList();
-        Workspace::Windows::iterator it = wins.begin();
-            
-        FluxboxWindow *focused_group = 0;
-        // start from the focused window
-        bool have_focused = false;
-        WinClient *focused = focusedWindow();
-        if (focused != 0) {
-            if (focused->screen().screenNumber() == m_screen.screenNumber()) {
-                have_focused = true;
-                focused_group = focused->fbwindow();
-            }
-        }
+        fbwin = (*it)->fbwindow();
+        // make sure the window is on the same workspace,
+        // unless its sticky, which is ok
+        if (!fbwin || 
+            ( fbwin->workspaceNumber() != m_screen.currentWorkspaceID() &&
+              ! fbwin->isStuck()))
+            continue;
 
-        if (!have_focused) {
-            focused_group = (*it);
-        } else {
-            //get focused window iterator
-            for (; it != wins.end() && (*it) != focused_group; ++it) 
-                continue;
-        }
+        // keep track of the originally selected window in a group
+        last_client = &fbwin->winClient();
+        was_iconic = (fbwin->isIconic() ? last_client : 0);
 
-        do {
-            if (it == wins.begin())
-                it = wins.end();
-            --it;
-            // see if the window should be skipped
-            if (! (doSkipWindow((*it)->winClient(), opts) || !(*it)->setInputFocus()) )
-                break;
-        } while ((*it) != focused_group);
-            
-        if ((*it) != focused_group && it != wins.end())
-            (*it)->raise();
+        // now we actually try to focus the window
+        if (!doSkipWindow(**it, opts) && (*it)->focus())
+            break;
     }
 
-}
+    // if we're still in the same fbwin, there's nothing else to do
+    if (m_cycling_last && m_cycling_last->fbwindow() == fbwin)
+        return;
+
+    // if we were already cycling, then restore the old state
+    if (m_cycling_last) {
+        m_screen.layerManager().restack();
+
+        // set back to originally selected window in that group
+        m_cycling_last->fbwindow()->setCurrentClient(*m_cycling_last, false);
+
+        if (m_was_iconic == m_cycling_last) {
+            s_reverting = true; // little hack
+            m_cycling_last->fbwindow()->iconify();
+            s_reverting = false;
+        }
+    }
+
+    if (isCycling())
+        fbwin->tempRaise();
+    else
+        fbwin->raise();
+
+    m_cycling_window = it;
+    m_cycling_last = last_client;
+    m_was_iconic = was_iconic;
 
-void FocusControl::addFocusFront(WinClient &client) {
-    m_focused_list.push_front(&client);
 }
 
 void FocusControl::addFocusBack(WinClient &client) {
     m_focused_list.push_back(&client);
+    m_creation_order_list.push_back(&client);
 }
 
-void FocusControl::stopCyclingFocus() {
-    // nothing to do
-    if (!m_cycling_focus)
+void FocusControl::addFocusFront(WinClient &client) {
+    m_focused_list.push_front(&client);
+    m_creation_order_list.push_back(&client);
+}
+
+// move all clients in given window to back of focused list
+void FocusControl::setFocusBack(FluxboxWindow *fbwin) {
+    // do nothing if there are no windows open
+    // don't change focus order while cycling
+    if (m_focused_list.empty() || s_reverting)
         return;
 
-    m_cycling_focus = false;
+    FocusedWindows::iterator it = m_focused_list.begin();
+    // use back to avoid an infinite loop
+    FocusedWindows::iterator it_back = --m_focused_list.end();
+
+    while (it != it_back) {
+        if ((*it)->fbwindow() == fbwin) {
+            m_focused_list.push_back(*it);
+            it = m_focused_list.erase(it);
+        } else
+            ++it;
+    }
+    // move the last one, if necessary, in order to preserve focus order
+    if ((*it)->fbwindow() == fbwin) {
+        m_focused_list.push_back(*it);
+        m_focused_list.erase(it);
+    }
+}
+    
+void FocusControl::stopCyclingFocus() {
+    // nothing to do
+    if (m_cycling_list == 0)
+        return;
+
+    FocusedWindows::iterator it_end = m_cycling_list->end();
     m_cycling_last = 0;
+    m_cycling_list = 0;
+
     // put currently focused window to top
     // the iterator may be invalid if the window died
     // in which case we'll do a proper revert focus
-    if (m_cycling_window != m_focused_list.end()) {
+    if (m_cycling_window != it_end && (*m_cycling_window)->fbwindow() &&
+        (*m_cycling_window)->fbwindow()->isVisible()) {
         WinClient *client = *m_cycling_window;
-        m_focused_list.erase(m_cycling_window);
+        m_focused_list.remove(client);
         m_focused_list.push_front(client);
         client->fbwindow()->raise();
-    } else {
+    } else
         revertFocus(m_screen);
-    }
-
 }
 
 /**
@@ -216,11 +239,9 @@ WinClient *FocusControl::lastFocusedWindow(int workspace) {
     FocusedWindows::iterator it_end = m_focused_list.end();
     for (; it != it_end; ++it) {
         if ((*it)->fbwindow() &&
-            (((int)(*it)->fbwindow()->workspaceNumber()) == workspace 
-             && !(*it)->fbwindow()->isIconic()
-             && (!(*it)->fbwindow()->isStuck() || (*it)->fbwindow()->isFocused())))
-            // only give focus to a stuck window if it is currently focused
-            // otherwise they tend to override normal workspace focus
+            ((((int)(*it)->fbwindow()->workspaceNumber()) == workspace ||
+             (*it)->fbwindow()->isStuck()) && (*it)->acceptsFocus() &&
+             !(*it)->fbwindow()->isIconic()))
             return *it;
     }
     return 0;
@@ -244,133 +265,15 @@ WinClient *FocusControl::lastFocusedWindow(FluxboxWindow &group, WinClient *igno
     return 0;
 }
 
-void FocusControl::nextFocus(int opts) {
-    const int num_windows = m_screen.currentWorkspace()->numberOfWindows();
-
-    if (num_windows < 1)
-        return;
-
-    if (!(opts & CYCLELINEAR)) {
-        if (!m_cycling_focus) {
-            m_cycling_focus = true;
-            m_cycling_window = m_focused_list.begin();
-            m_cycling_last = 0;
-        } else {
-            // already cycling, so restack to put windows back in their proper order
-            m_screen.layerManager().restack();
-        }
-        // if it is stacked, we want the highest window in the focused list
-        // that is on the same workspace
-        FocusedWindows::iterator it = m_cycling_window;
-        const FocusedWindows::iterator it_end = m_focused_list.end();
-        int safety_counter = 0;
-        while (true) {
-            ++it;
-            if (it == it_end) {
-                it = m_focused_list.begin();
-                safety_counter++;
-                if (safety_counter > 3)
-                    break;
-            }
-            // give up [do nothing] if we reach the current focused again
-            if ((*it) == (*m_cycling_window)) {
-                break;
-            }
-
-            FluxboxWindow *fbwin = (*it)->fbwindow();
-            if (fbwin && !fbwin->isIconic() &&
-                (fbwin->isStuck() 
-                 || fbwin->workspaceNumber() == m_screen.currentWorkspaceID())) {
-                // either on this workspace, or stuck
-
-                // keep track of the originally selected window in a set
-                WinClient &last_client = fbwin->winClient();
-
-                if (! (doSkipWindow(**it, opts) || !fbwin->setCurrentClient(**it)) ) {
-                    // moved onto a new fbwin
-                    if (!m_cycling_last || m_cycling_last->fbwindow() != fbwin) {
-                        if (m_cycling_last) {
-                            // set back to orig current Client in that fbwin
-                            m_cycling_last->fbwindow()->setCurrentClient(*m_cycling_last, false);
-                        }
-                        m_cycling_last = &last_client;
-                    }
-                    fbwin->tempRaise();
-                    break;
-                }
-            }
-        }
-        m_cycling_window = it;
-    } else { // not stacked cycling
-        // I really don't like this, but evidently some people use it(!)
-        Workspace &wksp = *m_screen.currentWorkspace();
-        Workspace::Windows &wins = wksp.windowList();
-        Workspace::Windows::iterator it = wins.begin();
-
-        FluxboxWindow *focused_group = 0;
-        // start from the focused window
-        bool have_focused = false;
-        WinClient *focused = focusedWindow();
-        if (focused != 0) {
-            if (focused->screen().screenNumber() == m_screen.screenNumber()) {
-                have_focused = true;
-                focused_group = focused->fbwindow();
-            }
-        }
-
-        if (!have_focused) {
-            focused_group = (*it);
-        } else {
-            // get focused window iterator
-            for (; it != wins.end() && (*it) != focused_group; ++it) 
-                continue;
-        }
-
-        int safety_counter = 0;
-        do {
-            ++it;
-            if (it == wins.end()) {
-                it = wins.begin();
-                safety_counter++;
-                if (safety_counter > 3)
-                    break;
-            }
-            // see if the window should be skipped
-            if (! (doSkipWindow((*it)->winClient(), opts) || !(*it)->setInputFocus()) )
-                break;
-        } while ((*it) != focused_group);
-
-        if ((*it) != focused_group && it != wins.end())
-            (*it)->raise();
-    }
-
-}
-
-void FocusControl::raiseFocus() {
-    bool have_focused = false;
-
-    // set have_focused if the currently focused window 
-    // is on this screen
-    if (focusedWindow()) {
-        if (focusedWindow()->screen().screenNumber() == m_screen.screenNumber()) {
-            have_focused = true;
-        }
-    }
-
-    // if we have a focused window on this screen and
-    // number of windows is greater than one raise the focused window
-    if (m_screen.currentWorkspace()->numberOfWindows() > 1 && have_focused)
-        focusedWindow()->raise();
-
-}
-
 void FocusControl::setScreenFocusedWindow(WinClient &win_client) {
 
      // raise newly focused window to the top of the focused list
-    if (!m_cycling_focus) { // don't change the order if we're cycling
+     // don't change the order if we're cycling or shutting down
+     // don't change on startup, as it may add windows that aren't listed yet
+    if (!isCycling() && !m_screen.isShuttingdown() && !s_reverting &&
+            !Fluxbox::instance()->isStartup()) {
         m_focused_list.remove(&win_client);
         m_focused_list.push_front(&win_client);
-        m_cycling_window = m_focused_list.begin();
     }
 }
 
@@ -391,10 +294,10 @@ void FocusControl::dirFocus(FluxboxWindow &win, FocusDir dir) {
     FluxboxWindow *foundwin = 0;
     int weight = 999999, exposure = 0; // extreme values
     int borderW = m_screen.winFrameTheme().border().width(),
-        top = win.y(), 
-        bottom = win.y() + win.height() + 2*borderW,
-        left = win.x(),
-        right = win.x() + win.width() + 2*borderW;
+        top = win.y() + borderW, 
+        bottom = win.y() + win.height() + borderW,
+        left = win.x() + borderW,
+        right = win.x() + win.width() + borderW;
 
     Workspace::Windows &wins = m_screen.currentWorkspace()->windowList();
     Workspace::Windows::iterator it = wins.begin();
@@ -408,12 +311,12 @@ void FocusControl::dirFocus(FluxboxWindow &win, FocusDir dir) {
         // we check things against an edge, and within the bounds (draw a picture)
         int edge=0, upper=0, lower=0, oedge=0, oupper=0, olower=0;
 
-        int otop = (*it)->y(), 
+        int otop = (*it)->y() + borderW, 
             // 2 * border = border on each side 
-            obottom = (*it)->y() + (*it)->height() + 2*borderW,
-            oleft = (*it)->x(),
+            obottom = (*it)->y() + (*it)->height() + borderW,
+            oleft = (*it)->x() + borderW,
             // 2 * border = border on each side
-            oright = (*it)->x() + (*it)->width() + 2*borderW;
+            oright = (*it)->x() + (*it)->width() + borderW;
 
         // check if they intersect
         switch (dir) {
@@ -482,19 +385,29 @@ void FocusControl::dirFocus(FluxboxWindow &win, FocusDir dir) {
 }
 
 void FocusControl::removeClient(WinClient &client) {
+    if (client.screen().isShuttingdown())
+        return;
+
     WinClient *cyc = 0;
-    if (m_cycling_window != m_focused_list.end())
+    if (m_cycling_list && m_cycling_window != m_cycling_list->end())
         cyc = *m_cycling_window;
 
     m_focused_list.remove(&client);
+    m_creation_order_list.remove(&client);
 
     if (cyc == &client) {
-        m_cycling_window = m_focused_list.end();
+        m_cycling_window = m_cycling_list->end();
+        stopCyclingFocus();
     }
+}
 
-    if (m_cycling_last == &client)
-        m_cycling_last = 0;
-
+void FocusControl::shutdown() {
+    // restore windows backwards so they get put back correctly on restart
+    FocusedWindows::reverse_iterator it = m_focused_list.rbegin();
+    for (; it != m_focused_list.rend(); ++it) {
+        if (*it && (*it)->fbwindow())
+            (*it)->fbwindow()->restore(*it, true);
+    }
 }
 
 /**
@@ -503,32 +416,39 @@ void FocusControl::removeClient(WinClient &client) {
  * last_focused is set to something if we want to make use of the
  * previously focused window (it must NOT be set focused now, it
  *   is probably dying).
- *
- * ignore_event means that it ignores the given event until
- * it gets a focusIn
  */
 void FocusControl::revertFocus(BScreen &screen) {
+    if (s_reverting)
+        return;
 
-    // Relevant resources:
-    // resource.focus_last = whether we focus last focused when changing workspace
-    // BScreen::FocusModel = sloppy, click, whatever
     WinClient *next_focus = 
         screen.focusControl().lastFocusedWindow(screen.currentWorkspaceID());
 
+    if (next_focus && next_focus->fbwindow() &&
+        next_focus->fbwindow()->isStuck())
+        FocusControl::s_reverting = true;
+
     // if setting focus fails, or isn't possible, fallback correctly
-    if (!(next_focus && next_focus->fbwindow() &&
-          next_focus->fbwindow()->setCurrentClient(*next_focus, true))) {
+    if (!(next_focus && next_focus->focus())) {
         setFocusedWindow(0); // so we don't get dangling m_focused_window pointer
-        switch (screen.focusControl().focusModel()) {
-        case FocusControl::MOUSEFOCUS:
-            XSetInputFocus(screen.rootWindow().display(),
-                           PointerRoot, None, CurrentTime);
-            break;
-        case FocusControl::CLICKFOCUS:
-            screen.rootWindow().setInputFocus(RevertToPointerRoot, CurrentTime);
-            break;
+        // if there's a menu open, focus it
+        if (FbTk::Menu::shownMenu())
+            FbTk::Menu::shownMenu()->grabInputFocus();
+        else {
+            switch (screen.focusControl().focusModel()) {
+            case FocusControl::MOUSEFOCUS:
+                XSetInputFocus(screen.rootWindow().display(),
+                               PointerRoot, None, CurrentTime);
+                break;
+            case FocusControl::CLICKFOCUS:
+                screen.rootWindow().setInputFocus(RevertToPointerRoot,
+                                                  CurrentTime);
+                break;
+            }
         }
     }
+
+    FocusControl::s_reverting = false;
 }
 
 /*
@@ -550,21 +470,6 @@ void FocusControl::unfocusWindow(WinClient &client,
     // go up the transient tree looking for a focusable window
 
     FluxboxWindow *fbwin = client.fbwindow();
-    if (fbwin == 0)
-        unfocus_frame = false;
-
-    WinClient *trans_parent = client.transientFor();
-    while (trans_parent) {
-        if (trans_parent->fbwindow() && // can't focus if no fbwin
-            (!unfocus_frame || trans_parent->fbwindow() != fbwin) && // can't be this window
-            trans_parent->fbwindow()->isVisible() &&
-            trans_parent->fbwindow()->setCurrentClient(*trans_parent, 
-                                                       s_focused_window == &client)) {
-            return;
-        }
-        trans_parent = trans_parent->transientFor();
-    }
-
     if (fbwin == 0)
         return; // nothing more we can do
 
@@ -594,42 +499,28 @@ void FocusControl::setFocusedWindow(WinClient *client) {
 #ifdef DEBUG
     cerr<<"------------------"<<endl;
     cerr<<"Setting Focused window = "<<client<<endl;
-    if (client != 0 && client->fbwindow() != 0)
-        cerr<<"title: "<<client->fbwindow()->title()<<endl;
+    if (client != 0)
+        cerr<<"title: "<<client->title()<<endl;
     cerr<<"Current Focused window = "<<s_focused_window<<endl;
     cerr<<"------------------"<<endl;
 #endif // DEBUG
-    
-    WinClient *old_client = 0;
 
     // Update the old focused client to non focus
-    // check if s_focused_window is valid
-    if (s_focused_window != 0 &&
-        Fluxbox::instance()->validateClient(s_focused_window)) {
-
-        old_client = s_focused_window;
-        if (old_client->fbwindow()) {
-            FluxboxWindow *old_win = old_client->fbwindow();
-            
-            if (!client || client->fbwindow() != old_win)
-                old_win->setFocusFlag(false);
-        }
-
-    } else {
-        s_focused_window = 0;
-    }
-
+    if (s_focused_fbwindow)
+        s_focused_fbwindow->setFocusFlag(false);
 
     if (client && client->fbwindow() && !client->fbwindow()->isIconic()) {
         // screen should be ok
-        FluxboxWindow *win = client->fbwindow();        
+        s_focused_fbwindow = client->fbwindow();        
         s_focused_window = client;     // update focused window
-        win->setCurrentClient(*client, 
+        s_focused_fbwindow->setCurrentClient(*client, 
                               false); // don't set inputfocus
-        win->setFocusFlag(true); // set focus flag
+        s_focused_fbwindow->setFocusFlag(true); // set focus flag
 
-    } else
+    } else {
         s_focused_window = 0;
+        s_focused_fbwindow = 0;
+    }
 
     // update AtomHandlers and/or other stuff...
     Fluxbox::instance()->updateFocusedWindow(screen, old_screen);
