@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: IconbarTool.cc 4229 2006-03-07 17:17:52Z fluxgen $
+// $Id: IconbarTool.cc 4945 2007-06-29 17:25:24Z markt $
 
 #include "IconbarTool.hh"
 
@@ -49,60 +49,42 @@
 #include <typeinfo>
 #include <string>
 #include <iterator>
-using namespace std;
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+
+using std::string;
+using std::list;
+
+#ifdef DEBUG
+using std::cerr;
+using std::endl;
+#endif // DEBUG
 
 namespace FbTk {
 
 template<>
 void FbTk::Resource<IconbarTool::Mode>::setFromString(const char *strval) {
-    if (strcasecmp(strval, "None") == 0) 
+    if (strcasecmp(strval, "None") == 0)
         m_value = IconbarTool::NONE;
-    else if (strcasecmp(strval, "Icons") == 0) 
+    else if (strcasecmp(strval, "Icons") == 0)
         m_value = IconbarTool::ICONS;
     else if (strcasecmp(strval, "NoIcons") == 0)
         m_value = IconbarTool::NOICONS;
-    else if (strcasecmp(strval, "WorkspaceIcons") == 0) 
+    else if (strcasecmp(strval, "WorkspaceIcons") == 0)
         m_value = IconbarTool::WORKSPACEICONS;
     else if (strcasecmp(strval, "WorkspaceNoIcons") == 0)
         m_value = IconbarTool::WORKSPACENOICONS;
-    else if (strcasecmp(strval, "Workspace") == 0) 
+    else if (strcasecmp(strval, "Workspace") == 0)
         m_value = IconbarTool::WORKSPACE;
-    else if (strcasecmp(strval, "AllWindows") == 0) 
+    else if (strcasecmp(strval, "AllWindows") == 0)
         m_value = IconbarTool::ALLWINDOWS;
     else
         setDefaultValue();
 }
 
-template<>
-void FbTk::Resource<IconbarTool::DeiconifyMode>::setDefaultValue() {
-    m_value = IconbarTool::FOLLOW;
-}
-
-template<>
-void FbTk::Resource<IconbarTool::DeiconifyMode>::setFromString(const char* strval) {
-    if (strncasecmp(strval, "Current", strlen("Current")) == 0)
-        m_value = IconbarTool::CURRENT;
-    else if (strncasecmp(strval, "Follow", strlen("Follow")) == 0)
-        m_value = IconbarTool::FOLLOW;
-    else if (strncasecmp(strval, "SemiFollow", strlen("SemiFollow")) == 0)
-        m_value = IconbarTool::SEMIFOLLOW;
-    else
-        setDefaultValue();
-}
-
-template<>
-std::string FbTk::Resource<IconbarTool::DeiconifyMode>::getString() const {
-    switch (m_value) {
-    case IconbarTool::SEMIFOLLOW:
-        return std::string("SemiFollow");
-        break;
-    case IconbarTool::CURRENT:
-        return std::string("Current");
-        break;
-    };
-    return std::string("Follow");
-}
-
 template<>
 void FbTk::Resource<IconbarTool::WheelMode>::setDefaultValue() {
     m_value = IconbarTool::SCREEN;
@@ -123,16 +105,18 @@ void FbTk::Resource<IconbarTool::WheelMode>::setFromString(const char* strval) {
 
 
 template<>
-std::string FbTk::Resource<IconbarTool::WheelMode>::getString() const {
+string FbTk::Resource<IconbarTool::WheelMode>::getString() const {
     switch(m_value) {
     case IconbarTool::ON:
-        return std::string("On");
+        return string("On");
         break;
     case IconbarTool::SCREEN:
-        return std::string("Screen");
+        return string("Screen");
         break;
+    case IconbarTool::OFF:
+    default:
+        return string("Off");
     };
-    return std::string("Off");
 }
 
 template<>
@@ -200,8 +184,8 @@ namespace {
 
 class ToolbarModeMenuItem : public FbTk::MenuItem {
 public:
-    ToolbarModeMenuItem(const char *label, IconbarTool &handler, 
-                        IconbarTool::Mode mode, 
+    ToolbarModeMenuItem(const FbTk::FbString &label, IconbarTool &handler,
+                        IconbarTool::Mode mode,
                         FbTk::RefCount<FbTk::Command> &cmd):
         FbTk::MenuItem(label, cmd), m_handler(handler), m_mode(mode) {
     }
@@ -218,8 +202,8 @@ private:
 
 class ToolbarAlignMenuItem: public FbTk::MenuItem {
 public:
-    ToolbarAlignMenuItem(const char *label, IconbarTool &handler, 
-                        Container::Alignment mode, 
+    ToolbarAlignMenuItem(const FbTk::FbString &label, IconbarTool &handler,
+                        Container::Alignment mode,
                         FbTk::RefCount<FbTk::Command> &cmd):
         FbTk::MenuItem(label, cmd), m_handler(handler), m_mode(mode) {
     }
@@ -238,65 +222,65 @@ void setupModeMenu(FbTk::Menu &menu, IconbarTool &handler) {
     using namespace FbTk;
     _FB_USES_NLS;
 
-    menu.setLabel(_FBTEXT(Toolbar, IconbarMode, "Iconbar Mode", "Menu title - chooses which set of icons are shown in the iconbar"));
+    menu.setLabel(_FB_XTEXT(Toolbar, IconbarMode, "Iconbar Mode", "Menu title - chooses which set of icons are shown in the iconbar"));
 
     RefCount<Command> saverc_cmd(new FbCommands::SaveResources());
-    
 
-    menu.insert(new ToolbarModeMenuItem(_FBTEXT(Toolbar, IconbarModeNone, 
+
+    menu.insert(new ToolbarModeMenuItem(_FB_XTEXT(Toolbar, IconbarModeNone,
                                                 "None", "No icons are shown in the iconbar"),
-                    handler, 
+                    handler,
                     IconbarTool::NONE, saverc_cmd));
 
     menu.insert(new ToolbarModeMenuItem(
-                    _FBTEXT(Toolbar, IconbarModeIcons, 
+                    _FB_XTEXT(Toolbar, IconbarModeIcons,
                             "Icons", "Iconified windows from all workspaces are shown"),
-                    handler, 
+                    handler,
                     IconbarTool::ICONS, saverc_cmd));
 
     menu.insert(new ToolbarModeMenuItem(
-                    _FBTEXT(Toolbar, IconbarModeNoIcons,
+                    _FB_XTEXT(Toolbar, IconbarModeNoIcons,
                         "NoIcons", "No iconified windows from all workspaces are shown"),
                     handler,
                     IconbarTool::NOICONS, saverc_cmd));
 
     menu.insert(new ToolbarModeMenuItem(
-                    _FBTEXT(Toolbar, IconbarModeWorkspaceIcons, 
+                    _FB_XTEXT(Toolbar, IconbarModeWorkspaceIcons,
                             "WorkspaceIcons", "Iconified windows from this workspace are shown"),
                     handler,
                     IconbarTool::WORKSPACEICONS, saverc_cmd));
 
     menu.insert(new ToolbarModeMenuItem(
-                    _FBTEXT(Toolbar, IconbarModeWorkspaceNoIcons, 
+                    _FB_XTEXT(Toolbar, IconbarModeWorkspaceNoIcons,
                             "WorkspaceNoIcons", "No iconified windows from this workspace are shown"),
                     handler,
                     IconbarTool::WORKSPACENOICONS, saverc_cmd));
-    
+
     menu.insert(new ToolbarModeMenuItem(
-                    _FBTEXT(Toolbar, IconbarModeWorkspace, 
+                    _FB_XTEXT(Toolbar, IconbarModeWorkspace,
                             "Workspace", "Normal and iconified windows from this workspace are shown"),
-                    handler, 
+                    handler,
                     IconbarTool::WORKSPACE, saverc_cmd));
 
     menu.insert(new ToolbarModeMenuItem(
-                    _FBTEXT(Toolbar, IconbarModeAllWindows, "All Windows", "All windows are shown"),
-                    handler, 
+                    _FB_XTEXT(Toolbar, IconbarModeAllWindows, "All Windows", "All windows are shown"),
+                    handler,
                     IconbarTool::ALLWINDOWS, saverc_cmd));
 
     menu.insert(new FbTk::MenuSeparator());
 
     menu.insert(new ToolbarAlignMenuItem(
-                    _FBTEXT(Align, Left, "Left", "Align to the left"),
+                    _FB_XTEXT(Align, Left, "Left", "Align to the left"),
                     handler,
                     Container::LEFT, saverc_cmd));
 
     menu.insert(new ToolbarAlignMenuItem(
-                    _FBTEXT(Align, Relative, "Relative", "Align relative to the width"),
+                    _FB_XTEXT(Align, Relative, "Relative", "Align relative to the width"),
                     handler,
                     Container::RELATIVE, saverc_cmd));
 
     menu.insert(new ToolbarAlignMenuItem(
-                    _FBTEXT(Align, Right, "Right", "Align to the right"),
+                    _FB_XTEXT(Align, Right, "Right", "Align to the right"),
                     handler,
                     Container::RIGHT, saverc_cmd));
 
@@ -304,7 +288,7 @@ void setupModeMenu(FbTk::Menu &menu, IconbarTool &handler) {
 
     menu.updateMenu();
 }
-                
+
 inline bool checkAddWindow(IconbarTool::Mode mode, const FluxboxWindow &win) {
     bool ret_val = false;
     // just add the icons that are on the this workspace
@@ -342,16 +326,15 @@ inline bool checkAddWindow(IconbarTool::Mode mode, const FluxboxWindow &win) {
     return ret_val;
 }
 
-void removeDuplicate(const IconbarTool::IconList &iconlist, std::list<FluxboxWindow *> &windowlist) {
-    IconbarTool::IconList::const_iterator win_it = iconlist.begin();
-    IconbarTool::IconList::const_iterator win_it_end = iconlist.end();
-    std::list<FluxboxWindow *>::iterator remove_it = windowlist.end();
-    for (; win_it != win_it_end; ++win_it)
-        remove_it = remove(windowlist.begin(), remove_it, &(*win_it)->win());
+void removeDuplicate(const IconbarTool::IconList &iconlist, list<FluxboxWindow *> &windowlist) {
+    IconbarTool::IconList::const_iterator icon_it = iconlist.begin();
+    IconbarTool::IconList::const_iterator icon_it_end = iconlist.end();
+    list<FluxboxWindow *>::iterator remove_it = windowlist.end();
+    for (; icon_it != icon_it_end; ++icon_it)
+        remove_it = remove(windowlist.begin(), remove_it, &(*icon_it)->win());
 
     // remove already existing windows
     windowlist.erase(remove_it, windowlist.end());
-    windowlist.unique();
 }
 
 }; // end anonymous namespace
@@ -362,16 +345,13 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme, BScr
     m_screen(screen),
     m_icon_container(parent),
     m_theme(theme),
-    m_focused_pm(0),
-    m_unfocused_pm(0),
-    m_focused_err_pm(0),
-    m_unfocused_err_pm(0),
-    m_empty_pm(0),
+    m_focused_pm( screen.imageControl() ),
+    m_unfocused_pm( screen.imageControl() ),
+    m_focused_err_pm( screen.imageControl() ),
+    m_unfocused_err_pm( screen.imageControl() ),
+    m_empty_pm( screen.imageControl() ),
     m_rc_mode(screen.resourceManager(), WORKSPACE,
               screen.name() + ".iconbar.mode", screen.altName() + ".Iconbar.Mode"),
-    m_deiconify_mode(screen.resourceManager(), FOLLOW,
-                     screen.name() + ".iconbar.deiconifyMode",
-                     screen.name() + ".iconbar.DeiconifyMode"),
     m_wheel_mode(screen.resourceManager(), OFF,
                  screen.name() + ".iconbar.wheelMode",
                  screen.name() + ".iconbar.WheelMode"),
@@ -391,20 +371,21 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme, BScr
     _FB_USES_NLS;
     using namespace FbTk;
     // setup use pixmap item to reconfig iconbar and save resource on click
-    MacroCommand *save_and_reconfig = new MacroCommand();   
+    MacroCommand *save_and_reconfig = new MacroCommand();
     RefCount<Command> reconfig(new SimpleCommand<IconbarTool>(*this, &IconbarTool::renderTheme));
     RefCount<Command> save(CommandParser::instance().parseLine("saverc"));
     save_and_reconfig->add(reconfig);
     save_and_reconfig->add(save);
     RefCount<Command> s_and_reconfig(save_and_reconfig);
-    m_menu.insert(new BoolMenuItem(_FBTEXT(Toolbar, ShowIcons, "Show Pictures", "chooses if little icons are shown next to title in the iconbar") , 
+    m_menu.insert(new BoolMenuItem(_FB_XTEXT(Toolbar, ShowIcons,
+                    "Show Pictures", "chooses if little icons are shown next to title in the iconbar"),
 	                *m_rc_use_pixmap, s_and_reconfig));
     m_menu.updateMenu();
     // must be internal menu, otherwise toolbar main menu tries to delete it.
     m_menu.setInternalMenu();
 
     // add iconbar menu to toolbar menu
-    menu.insert(m_menu.label().c_str(), &m_menu);
+    menu.insert(m_menu.label(), &m_menu);
 
     // setup signals
     theme.reconfigSig().attach(this);
@@ -425,19 +406,6 @@ IconbarTool::IconbarTool(const FbTk::FbWindow &parent, IconbarTheme &theme, BScr
 
 IconbarTool::~IconbarTool() {
     deleteIcons();
-
-    // remove cached images
-    if (m_focused_pm)
-        m_screen.imageControl().removeImage(m_focused_pm);
-    if (m_unfocused_pm)
-        m_screen.imageControl().removeImage(m_unfocused_pm);
-    if (m_focused_err_pm)
-        m_screen.imageControl().removeImage(m_focused_err_pm);
-    if (m_unfocused_err_pm)
-        m_screen.imageControl().removeImage(m_unfocused_err_pm);
-    if (m_empty_pm)
-        m_screen.imageControl().removeImage(m_empty_pm);
-
 }
 
 void IconbarTool::move(int x, int y) {
@@ -467,6 +435,7 @@ void IconbarTool::hide() {
 void IconbarTool::setAlignment(Container::Alignment align) {
     *m_rc_alignment = align;
     update(0);
+    m_menu.reconfigure();
 }
 
 void IconbarTool::setMode(Mode mode) {
@@ -481,22 +450,8 @@ void IconbarTool::setMode(Mode mode) {
     deleteIcons();
 
     // update mode
-    switch (mode) {
-    case NONE: // nothing
-        break;
-    case ICONS: // all icons from all workspaces
-    case WORKSPACEICONS: // all icons on current workspace
-        updateIcons();
-        break;
-    case WORKSPACENOICONS:
-    case WORKSPACE: // all windows and all icons on current workspace
-        updateWorkspace();
-        break;
-    case NOICONS:
-    case ALLWINDOWS: // all windows and all icons from all workspaces
-        updateAllWindows();
-        break;
-    };
+    if (mode != NONE)
+        updateList();
 
     // unlock graphics update
     m_icon_container.setUpdateLock(false);
@@ -504,22 +459,10 @@ void IconbarTool::setMode(Mode mode) {
     m_icon_container.showSubwindows();
 
     renderTheme();
+
+    m_menu.reconfigure();
 }
 
-void IconbarTool::setDeiconifyMode(DeiconifyMode mode) {
-    if (mode == *m_deiconify_mode)
-        return;
-
-    *m_deiconify_mode = mode;
-}
-
-void IconbarTool::setWheelMode(WheelMode mode) {
-    if (mode == *m_wheel_mode)
-        return;
-    *m_wheel_mode = mode;
-}
-
-
 unsigned int IconbarTool::width() const {
     return m_icon_container.width();
 }
@@ -532,12 +475,12 @@ unsigned int IconbarTool::borderWidth() const {
     return m_icon_container.borderWidth();
 }
 
-void IconbarTool::update(FbTk::Subject *subj) {    
+void IconbarTool::update(FbTk::Subject *subj) {
     // ignore updates if we're shutting down
     if (m_screen.isShuttingdown()) {
         m_screen.clientListSig().detach(this);
         m_screen.iconListSig().detach(this);
-        m_screen.currentWorkspaceSig().detach(this);        
+        m_screen.currentWorkspaceSig().detach(this);
         if (!m_icon_list.empty())
             deleteIcons();
         return;
@@ -570,8 +513,11 @@ void IconbarTool::update(FbTk::Subject *subj) {
             return;
         } else if (subj == &(winsubj->win().workspaceSig())) {
             // we can ignore this signal if we're in ALLWINDOWS mode
-            if (mode() == ALLWINDOWS)
+            // unless the window was focused and has nothing to revert to
+            if (mode() == ALLWINDOWS || mode() == ICONS || mode() == NOICONS) {
+                m_focus_timer.start();
                 return;
+            }
 
             // workspace changed for this window, and if it's not on current workspace we remove it
             if (m_screen.currentWorkspaceID() != winsubj->win().workspaceNumber()) {
@@ -585,17 +531,19 @@ void IconbarTool::update(FbTk::Subject *subj) {
             return; // we don't need to update the entire list
         } else if (subj == &(winsubj->win().stateSig())) {
             if (!checkAddWindow(mode(), winsubj->win())) {
-                removeWindow(winsubj->win());            
+                removeWindow(winsubj->win());
                 renderTheme();
             }
 
             return;
 
-        } else if (subj == &(winsubj->win().titleSig())) {
+        } else if (subj == &(winsubj->win().attentionSig())) {
+            // render with titlebar focus, on attention
             IconButton *button = findButton(winsubj->win());
-            if (button == 0)
-                return;
-            button->setText(winsubj->win().title());
+            if (button) {
+                renderButton(*button, true,
+                             winsubj->win().getAttentionState());
+            }
             return;
         } else {
             // signal not handled
@@ -603,15 +551,16 @@ void IconbarTool::update(FbTk::Subject *subj) {
         }
     }
 
-    bool remove_all = false; // if we should readd all windows    
+    bool remove_all = false; // if we should readd all windows
 
-    if (subj != 0 && typeid(*subj) == typeid(BScreen::ScreenSubject) && mode() != ALLWINDOWS) {
+    if (subj != 0 && typeid(*subj) == typeid(BScreen::ScreenSubject) &&
+        mode() != ALLWINDOWS && mode() != ICONS && mode() != NOICONS) {
         BScreen::ScreenSubject *screen_subj = static_cast<BScreen::ScreenSubject *>(subj);
         // current workspace sig
-        if (&m_screen.currentWorkspaceSig() == screen_subj &&
-            mode() != ALLWINDOWS && mode() != ICONS) {
+        if (&m_screen.currentWorkspaceSig() == screen_subj ) {
             remove_all = true; // remove and readd all windows
         }
+
     }
 
     // lock graphic update
@@ -621,32 +570,21 @@ void IconbarTool::update(FbTk::Subject *subj) {
         deleteIcons();
 
     // ok, we got some signal that we need to update our iconbar container
-    switch (mode()) {
-    case NONE:
-        return;
-        break;
-    case ICONS:        
-    case WORKSPACEICONS:
-        updateIcons();
-        break;
-    case WORKSPACENOICONS:
-    case WORKSPACE:
-        updateWorkspace();
-        break;
-    case NOICONS:
-    case ALLWINDOWS:
-        updateAllWindows();
-        break;
-    }
+    updateList();
 
     // unlock container and update graphics
     m_icon_container.setUpdateLock(false);
     m_icon_container.update();
     m_icon_container.showSubwindows();
-    
+
     // another renderTheme we hopefully shouldn't need? These renders
     // should be done individually above
-    // renderTheme();
+
+    // nope, we still need it (or at least I'm not bothering to fix it yet)
+    // a new IconButton doesn't get resized properly until the
+    // m_icon_container.update() above; then, it never runs drawText() again,
+    // so text can end up behind program icons
+    renderTheme();
 }
 
 IconButton *IconbarTool::findButton(FluxboxWindow &win) {
@@ -660,25 +598,25 @@ IconButton *IconbarTool::findButton(FluxboxWindow &win) {
 
     return 0;
 }
-
+/*
 void IconbarTool::renderWindow(FluxboxWindow &win) {
     IconButton *button = findButton(win);
     if (button == 0)
         return;
     renderButton(*button);
 }
-
+*/
 void IconbarTool::updateSizing() {
     m_icon_container.setBorderWidth(m_theme.border().width());
 
     IconList::iterator icon_it = m_icon_list.begin();
-    const IconList::iterator icon_it_end = m_icon_list.end(); 
+    const IconList::iterator icon_it_end = m_icon_list.end();
     for (; icon_it != icon_it_end; ++icon_it) {
         if ((*icon_it)->win().isFocused())
             (*icon_it)->setBorderWidth(m_theme.focusedBorder().width());
         else // unfocused
             (*icon_it)->setBorderWidth(m_theme.unfocusedBorder().width());
-    }    
+    }
 
 }
 
@@ -692,61 +630,57 @@ void IconbarTool::renderTheme() {
 
     // update button sizes before we get max width per client!
     updateSizing();
-    
-    Pixmap tmp = m_focused_pm;
-    Pixmap err_tmp = m_focused_err_pm;
-    unsigned int icon_width = m_icon_container.maxWidthPerClient();
-    if (!m_theme.focusedTexture().usePixmap()) {
-        m_focused_pm = 0;
-        m_focused_err_pm = 0;
-    } else {
-        m_focused_pm = m_screen.imageControl().renderImage(icon_width,
-                                                           m_icon_container.height(),
-                                                           m_theme.focusedTexture());
-        m_focused_err_pm = m_screen.imageControl().renderImage(icon_width+1,
-                                                               m_icon_container.height(),
-                                                               m_theme.focusedTexture());
-    }
-        
-    if (tmp)
-        m_screen.imageControl().removeImage(tmp);
-    if (err_tmp)
-        m_screen.imageControl().removeImage(err_tmp);
 
-    tmp = m_unfocused_pm;
-    err_tmp = m_unfocused_err_pm;
-    
-    if (!m_theme.unfocusedTexture().usePixmap()) {
-        m_unfocused_pm = 0;
-        m_unfocused_err_pm = 0;
+    unsigned int icon_width = 0, icon_height = 0;
+    unsigned int icon_width_off=0, icon_height_off=0;
+
+    if (orientation() == FbTk::ROT0 || orientation() == FbTk::ROT180) {
+        icon_width = m_icon_container.maxWidthPerClient();
+        icon_height = m_icon_container.height();
+        icon_width_off = 1;
     } else {
-        m_unfocused_pm = m_screen.imageControl().renderImage(icon_width,
-                                                             m_icon_container.height(),
-                                                             m_theme.unfocusedTexture());
-        m_unfocused_err_pm = m_screen.imageControl().renderImage(icon_width+1,
-                                                                 m_icon_container.height(),
-                                                                 m_theme.unfocusedTexture());
+        icon_width = m_icon_container.width();
+        icon_height = m_icon_container.maxWidthPerClient();
+        icon_height_off = 1;
+    }
+
+    if (!m_theme.focusedTexture().usePixmap()) {
+        m_focused_pm.reset( 0 );
+        m_focused_err_pm.reset( 0 );
+    } else {
+        m_focused_pm.reset( m_screen.imageControl().
+                            renderImage(icon_width, icon_height,
+                                        m_theme.focusedTexture(), orientation()) );
+        m_focused_err_pm.reset( m_screen.imageControl().
+                                renderImage(icon_width + icon_width_off,
+                                            icon_height + icon_height_off,
+                                            m_theme.focusedTexture(), orientation()) );
+    }
+
+    if (!m_theme.unfocusedTexture().usePixmap()) {
+        m_unfocused_pm.reset( 0 );
+        m_unfocused_err_pm.reset( 0 );
+    } else {
+        m_unfocused_pm.reset( m_screen.imageControl().
+                              renderImage(icon_width, icon_height,
+                                          m_theme.unfocusedTexture(), orientation()) );
+        m_unfocused_err_pm.reset( m_screen.imageControl().
+                                  renderImage(icon_width+icon_width_off,
+                                              icon_height+icon_height_off,
+                                              m_theme.unfocusedTexture(), orientation()) );
     }
-    if (tmp)
-        m_screen.imageControl().removeImage(tmp);
-    if (err_tmp)
-        m_screen.imageControl().removeImage(err_tmp);
 
     // if we dont have any icons then we should render empty texture
-    tmp = m_empty_pm;
     if (!m_theme.emptyTexture().usePixmap()) {
-        m_empty_pm = 0;
+        m_empty_pm.reset( 0 );
         m_icon_container.setBackgroundColor(m_theme.emptyTexture().color());
     } else {
-        m_empty_pm = m_screen.imageControl().renderImage(m_icon_container.width(),
-                                                         m_icon_container.height(),
-                                                         m_theme.emptyTexture());
+        m_empty_pm.reset( m_screen.imageControl().
+                          renderImage(m_icon_container.width(), m_icon_container.height(),
+                                      m_theme.emptyTexture(), orientation()) );
         m_icon_container.setBackgroundPixmap(m_empty_pm);
     }
 
-    if (tmp)
-        m_screen.imageControl().removeImage(tmp);
-
     // set to zero so its consistent and not ugly
     m_icon_container.setBorderWidth(m_theme.border().width());
     m_icon_container.setBorderColor(m_theme.border().color());
@@ -754,26 +688,40 @@ void IconbarTool::renderTheme() {
 
     // update buttons
     IconList::iterator icon_it = m_icon_list.begin();
-    const IconList::iterator icon_it_end = m_icon_list.end(); 
+    const IconList::iterator icon_it_end = m_icon_list.end();
     for (; icon_it != icon_it_end; ++icon_it) {
         renderButton(*(*icon_it));
     }
 }
 
-void IconbarTool::renderButton(IconButton &button, bool clear) {
+void IconbarTool::renderButton(IconButton &button, bool clear,
+                               int focusOption) {
 
     button.setPixmap(*m_rc_use_pixmap);
     button.setAlpha(m_alpha);
+    button.setTextPadding(*m_rc_client_padding);
 
     // The last button is always the regular width
     bool wider_button = false;
-    if (!m_icon_container.empty())
-        wider_button = (button.width() != m_icon_container.back()->width());
+    if (!m_icon_container.empty()) {
+        if (button.orientation() == FbTk::ROT0 || button.orientation() == FbTk::ROT180)
+            wider_button = button.width() != m_icon_container.back()->width();
+        else
+            wider_button = button.height() != m_icon_container.back()->height();
+//            wider_button = (button.width() != m_icon_container.maxWidthPerClient() || // height to cover both orients
 
-    if (button.win().isFocused()) { // focused texture
-        m_icon_container.setSelected(m_icon_container.find(&button));
+//                        button.height() != m_icon_container.back()->height());
+    }
 
-        button.setGC(m_theme.focusedText().textGC());     
+    if (focusOption == 1 ||
+        (focusOption == -1 &&
+         button.win().isFocused())) {
+
+        // focused texture
+        if (button.win().isFocused())
+            m_icon_container.setSelected(m_icon_container.find(&button));
+
+        button.setGC(m_theme.focusedText().textGC());
         button.setFont(m_theme.focusedText().font());
         button.setJustify(m_theme.focusedText().justify());
         button.setBorderWidth(m_theme.focusedBorder().width());
@@ -784,7 +732,7 @@ void IconbarTool::renderButton(IconButton &button, bool clear) {
         else if (wider_button && m_focused_err_pm != 0)
             button.setBackgroundPixmap(m_focused_err_pm);
         else
-            button.setBackgroundColor(m_theme.focusedTexture().color());        
+            button.setBackgroundColor(m_theme.focusedTexture().color());
 
     } else { // unfocused
         if (m_icon_container.selected() == &button)
@@ -830,7 +778,7 @@ void IconbarTool::removeWindow(FluxboxWindow &win) {
     }
 #ifdef DEBUG
     cerr<<"IconbarTool::"<<__FUNCTION__<<"( 0x"<<&win<<" title = "<<win.title()<<") found!"<<endl;
-#endif // DEBUG    
+#endif // DEBUG
     // detach from all signals
     win.focusSig().detach(this);
     win.dieSig().detach(this);
@@ -856,16 +804,14 @@ void IconbarTool::addWindow(FluxboxWindow &win) {
 #ifdef DEBUG
     cerr<<"IconbarTool::addWindow(0x"<<&win<<" title = "<<win.title()<<")"<<endl;
 #endif // DEBUG
-    IconButton *button = new IconButton(*this, 
-                                        m_icon_container, 
-                                        m_theme.focusedText().font(), 
+    IconButton *button = new IconButton(*this,
+                                        m_icon_container,
+                                        m_theme.focusedText().font(),
                                         win);
-  
 
-    button->setTextPadding(*m_rc_client_padding);
 
     renderButton(*button, false); // update the attributes, but don't clear it
-    m_icon_container.insertItem(button);    
+    m_icon_container.insertItem(button);
     m_icon_list.push_back(button);
 
     // dont forget to detach signal in removeWindow
@@ -873,91 +819,44 @@ void IconbarTool::addWindow(FluxboxWindow &win) {
     win.dieSig().attach(this);
     win.workspaceSig().attach(this);
     win.stateSig().attach(this);
-    win.titleSig().attach(this);
-
+    win.attentionSig().attach(this);
 }
 
-void IconbarTool::updateIcons() {
-    std::list<FluxboxWindow *> itemlist;
-    // add icons to the itemlist    
-    BScreen::Icons::iterator icon_it = m_screen.iconList().begin();
-    BScreen::Icons::iterator icon_it_end = m_screen.iconList().end();
-    for (; icon_it != icon_it_end; ++icon_it) {
-        if (mode() == ICONS)
-            itemlist.push_back(*icon_it);
-        else if (mode() == WORKSPACEICONS && 
-                   ((*icon_it)->workspaceNumber() == m_screen.currentWorkspaceID() ||
-                    (*icon_it)->isStuck()))
-            itemlist.push_back(*icon_it);
-    }
-    removeDuplicate(m_icon_list, itemlist);
-    addList(itemlist);    
-}
-
-void IconbarTool::updateWorkspace() {
-    std::list<FluxboxWindow *> itemlist;
-    // add current workspace windows
-    Workspace &space = *m_screen.currentWorkspace();
-    Workspace::Windows::iterator win_it = space.windowList().begin();
-    Workspace::Windows::iterator win_it_end = space.windowList().end();
-    for (; win_it != win_it_end; ++win_it) {
-        if (checkAddWindow(mode(), **win_it))
-            itemlist.push_back(*win_it);
+void IconbarTool::updateList() {
+    list<WinClient *> ordered_list =
+        m_screen.focusControl().creationOrderList();
+    list<WinClient *>::iterator it = ordered_list.begin();
+    list<WinClient *>::iterator it_end = ordered_list.end();
+    for (; it != it_end; ++it) {
+        if ((*it)->fbwindow() && checkAddWindow(mode(), *(*it)->fbwindow()) &&
+            !checkDuplicate(*(*it)->fbwindow()))
+            addWindow(*(*it)->fbwindow());
     }
 
-    // add icons from current workspace
-    if (mode() != WORKSPACENOICONS) {
-        BScreen::Icons::iterator icon_it =  m_screen.iconList().begin();
-        BScreen::Icons::iterator icon_it_end =  m_screen.iconList().end();
-        for (; icon_it != icon_it_end; ++icon_it) {
-            if ((*icon_it)->workspaceNumber() == m_screen.currentWorkspaceID() ||
-                (*icon_it)->isStuck())
-                itemlist.push_back(*icon_it);
-        }
-    }
-
-    removeDuplicate(m_icon_list, itemlist);
-    addList(itemlist);
-}
-
-
-void IconbarTool::updateAllWindows() {
-    std::list<FluxboxWindow *> full_list;
-    // for each workspace add clients to full list
-    BScreen::Workspaces::iterator workspace_it = m_screen.getWorkspacesList().begin();
-    BScreen::Workspaces::iterator workspace_it_end = m_screen.getWorkspacesList().end();
-    for (; workspace_it != workspace_it_end; ++workspace_it) {
-        full_list.insert(full_list.end(),
-                         (*workspace_it)->windowList().begin(),
-                         (*workspace_it)->windowList().end());
-    }
-    // add icons
-    if(mode() != NOICONS && mode() != WORKSPACENOICONS) {
-        full_list.insert(full_list.end(),
-                         m_screen.iconList().begin(),
-                         m_screen.iconList().end());
-    }
-
-    removeDuplicate(m_icon_list, full_list);
-    addList(full_list);
-}
-
-void IconbarTool::addList(std::list<FluxboxWindow *> &winlist) {
-    // ok, now we should have a list of icons that we need to add
-    std::list<FluxboxWindow *>::iterator it = winlist.begin();
-    std::list<FluxboxWindow *>::iterator it_end = winlist.end();
-    for (; it != it_end; ++it)
-        addWindow(**it);
     renderTheme();
 }
 
+bool IconbarTool::checkDuplicate(FluxboxWindow &win) {
+    IconList::iterator it = m_icon_list.begin();
+    IconList::iterator it_end = m_icon_list.end();
+    for (; it != it_end; ++it) {
+        if (&win == &(*it)->win())
+            return true;
+    }
+    return false;
+}
+
 void IconbarTool::timedRender() {
     WinClient *client = FocusControl::focusedWindow();
-    if (client == 0 || client->fbwindow() == 0)
+    IconButton *current_button = static_cast<IconButton *>(m_icon_container.selected());
+
+    if (client == 0 || client->fbwindow() == 0) {
+        if (current_button != 0)
+            renderButton(*current_button);
         return;
+    }
 
     IconButton *button = findButton(*client->fbwindow());
-    IconButton *current_button = static_cast<IconButton *>(m_icon_container.selected());
     // if old window is the same as the new focused window then ignore this render
     // else render old client and new client
     if (button == current_button)
@@ -968,3 +867,7 @@ void IconbarTool::timedRender() {
         renderButton(*current_button);
 }
 
+void IconbarTool::setOrientation(FbTk::Orientation orient) {
+    m_icon_container.setOrientation(orient);
+    ToolbarItem::setOrientation(orient);
+}
