@@ -1,5 +1,5 @@
 // RootCmdMenuItem.cc for Fluxbox Window Manager
-// Copyright (c) 2004 Mathias Gumz
+// Copyright (c) 2004 - 2006 Mathias Gumz (akira at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,23 +19,25 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: RootCmdMenuItem.cc 3700 2004-11-19 11:37:27Z mathias $
+// $Id: RootCmdMenuItem.cc 4980 2007-07-07 23:11:04Z markt $
 
 #include "RootCmdMenuItem.hh"
 
+#include "defaults.hh"
 #include "FbCommands.hh"
 #include "fluxbox.hh"
 
 #include "FbTk/StringUtil.hh"
 
-RootCmdMenuItem::RootCmdMenuItem(const std::string &label, 
+RootCmdMenuItem::RootCmdMenuItem(const FbTk::FbString &label, 
                              const std::string &filename,
                              const std::string &cmd):
-    FbTk::MenuItem(label.c_str()), 
+    FbTk::MenuItem(label), 
     m_filename(filename) {
-    
-      FbTk::RefCount<FbTk::Command> 
-        setwp_cmd(new FbCommands::ExecuteCmd(cmd + " \"" + m_filename + "\""));
+
+    std::string prog = cmd.empty() ? realProgramName("fbsetbg") : cmd;
+    FbTk::RefCount<FbTk::Command>
+        setwp_cmd(new FbCommands::ExecuteCmd(prog + " \"" + m_filename + "\""));
     setCommand(setwp_cmd);
     setToggleItem(true);
 }
