@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ScreenResources.cc 4227 2006-03-02 07:49:13Z mathias $
+// $Id: ScreenResources.cc 4945 2007-06-29 17:25:24Z markt $
 
 
 
@@ -27,7 +27,13 @@
 
 #include "Screen.hh"
 #include <string>
-using namespace std;
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+
+using std::string;
 
 namespace FbTk {
 
@@ -53,17 +59,17 @@ void FbTk::Resource<FbTk::MenuTheme::MenuMode>::setFromString(const char *str) {
 }
 
 template <>
-std::string FbTk::Resource<BScreen::ResizeModel>::getString() const {
+string FbTk::Resource<BScreen::ResizeModel>::getString() const {
     switch (m_value) {
     case BScreen::QUADRANTRESIZE:
-        return std::string("Quadrant");
+        return string("Quadrant");
     case BScreen::BOTTOMRESIZE:
-        return std::string("Bottom");
+        return string("Bottom");
     case BScreen::CENTERRESIZE:
-        return std::string("Center");
+        return string("Center");
     }
 
-    return std::string("Default");
+    return string("Default");
 }
 
 template<>
@@ -75,22 +81,28 @@ setFromString(char const *strval) {
         m_value = BScreen::QUADRANTRESIZE;
     } else if (strcasecmp(strval, "Center") == 0) {
         m_value = BScreen::CENTERRESIZE;
-    } else 
+    } else
         m_value = BScreen::DEFAULTRESIZE;
 }
 
 template<>
-std::string FbTk::Resource<BScreen::FollowModel>::getString() const {
+string FbTk::Resource<BScreen::FollowModel>::getString() const {
     switch (m_value) {
     case BScreen::FOLLOW_ACTIVE_WINDOW:
-        return std::string("Follow");
+    default:
+        return string("Follow");
         break;
     case BScreen::FETCH_ACTIVE_WINDOW:
-        return std::string("Current");
+        return string("Current");
+        break;
+    case BScreen::SEMIFOLLOW_ACTIVE_WINDOW:
+        return string("SemiFollow");
+        break;
+    case BScreen::IGNORE_OTHER_WORKSPACES:
+        return string("Ignore");
         break;
     }
 
-    return std::string("Ignore");
 }
 
 template<>
@@ -98,16 +110,20 @@ void FbTk::Resource<BScreen::FollowModel>::
 setFromString(char const *strval) {
     if (strcasecmp(strval, "Follow") == 0)
         m_value = BScreen::FOLLOW_ACTIVE_WINDOW;
-    else if (strcasecmp(strval, "Current") == 0 || 
+    else if (strcasecmp(strval, "Current") == 0 ||
              strcasecmp(strval, "CurrentWorkspace") == 0 ||
              strcasecmp(strval, "Fetch") == 0)
         m_value = BScreen::FETCH_ACTIVE_WINDOW;
+    else if (strcasecmp(strval, "SemiFollow") == 0)
+        m_value = BScreen::SEMIFOLLOW_ACTIVE_WINDOW;
+    else if (strcasecmp(strval, "Ignore") == 0)
+        m_value = BScreen::IGNORE_OTHER_WORKSPACES;
     else
         setDefaultValue();
 }
 
 template<>
-std::string FbTk::Resource<FbTk::GContext::LineStyle>::getString() const {
+string FbTk::Resource<FbTk::GContext::LineStyle>::getString() const {
     switch(m_value) {
     case FbTk::GContext::LINESOLID:
         return "LineSolid";
@@ -124,20 +140,20 @@ std::string FbTk::Resource<FbTk::GContext::LineStyle>::getString() const {
 
 template<>
 void FbTk::Resource<FbTk::GContext::LineStyle>
-::setFromString(char const *strval) { 
+::setFromString(char const *strval) {
 
     if (strcasecmp(strval, "LineSolid") == 0 )
         m_value = FbTk::GContext::LINESOLID;
     else if (strcasecmp(strval, "LineOnOffDash") == 0 )
         m_value = FbTk::GContext::LINEONOFFDASH;
-    else if (strcasecmp(strval, "LineDoubleDash") == 0) 
+    else if (strcasecmp(strval, "LineDoubleDash") == 0)
         m_value = FbTk::GContext::LINEDOUBLEDASH;
     else
         setDefaultValue();
 }
 
 template<>
-std::string FbTk::Resource<FbTk::GContext::JoinStyle>::getString() const {
+string FbTk::Resource<FbTk::GContext::JoinStyle>::getString() const {
     switch(m_value) {
     case FbTk::GContext::JOINMITER:
         return "JoinMiter";
@@ -154,20 +170,20 @@ std::string FbTk::Resource<FbTk::GContext::JoinStyle>::getString() const {
 
 template<>
 void FbTk::Resource<FbTk::GContext::JoinStyle>
-::setFromString(char const *strval) { 
+::setFromString(char const *strval) {
 
     if (strcasecmp(strval, "JoinRound") == 0 )
         m_value = FbTk::GContext::JOINROUND;
     else if (strcasecmp(strval, "JoinMiter") == 0 )
         m_value = FbTk::GContext::JOINMITER;
-    else if (strcasecmp(strval, "JoinBevel") == 0) 
+    else if (strcasecmp(strval, "JoinBevel") == 0)
         m_value = FbTk::GContext::JOINBEVEL;
     else
         setDefaultValue();
 }
 
 template<>
-std::string FbTk::Resource<FbTk::GContext::CapStyle>::getString() const {
+string FbTk::Resource<FbTk::GContext::CapStyle>::getString() const {
     switch(m_value) {
     case FbTk::GContext::CAPNOTLAST:
         return "CapNotLast";
@@ -187,13 +203,13 @@ std::string FbTk::Resource<FbTk::GContext::CapStyle>::getString() const {
 
 template<>
 void FbTk::Resource<FbTk::GContext::CapStyle>
-::setFromString(char const *strval) { 
+::setFromString(char const *strval) {
 
     if (strcasecmp(strval, "CapNotLast") == 0 )
         m_value = FbTk::GContext::CAPNOTLAST;
     else if (strcasecmp(strval, "CapProjecting") == 0 )
         m_value = FbTk::GContext::CAPPROJECTING;
-    else if (strcasecmp(strval, "CapRound") == 0) 
+    else if (strcasecmp(strval, "CapRound") == 0)
         m_value = FbTk::GContext::CAPROUND;
     else if (strcasecmp(strval, "CapButt" ) == 0)
         m_value = FbTk::GContext::CAPBUTT;
