@@ -1,5 +1,5 @@
 // Slit.cc for fluxbox
-// Copyright (c) 2002 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2002 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Slit.cc for Blackbox - an X11 Window manager
 // Copyright (c) 1997 - 2000 Brad Hughes (bhughes at tcac.net)
@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Slit.cc 3990 2005-05-03 13:53:25Z simonb $
+// $Id: Slit.cc 4967 2007-07-03 18:50:53Z markt $
 
 #include "Slit.hh"
 
@@ -43,31 +43,28 @@
 #include "SimpleCommand.hh"
 #include "MacroCommand.hh"
 #include "FbCommands.hh"
+#include "Layer.hh"
 #include "LayerMenu.hh"
-#include "fluxbox.hh"
 #include "XLayer.hh"
 #include "RootTheme.hh"
 #include "FbTk/Theme.hh"
 #include "FbMenu.hh"
 #include "Transparent.hh"
 #include "IntResMenuItem.hh"
+
+#ifdef XINERAMA
 #include "Xinerama.hh"
+#endif // XINERAMA
+
 #include "SlitTheme.hh"
 #include "SlitClient.hh"
 #include "Xutil.hh"
 #include "FbAtoms.hh"
+#include "FbTk/App.hh"
 #include "FbTk/MenuSeparator.hh"
 #include "FbTk/StringUtil.hh"
 #include "FbTk/I18n.hh"
 
-#include <algorithm>
-#include <iostream>
-#ifdef HAVE_CASSERT
-  #include <cassert>
-#else
-  #include <assert.h>
-#endif
-
 #ifdef HAVE_SYS_STAT_H
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -78,7 +75,26 @@
 
 #include <iostream>
 #include <algorithm>
-using namespace std;
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
+
+using std::string;
+using std::pair;
+using std::list;
+using std::ifstream;
+using std::ofstream;
+using std::endl;
+
+#ifdef DEBUG
+using std::cerr;
+using std::hex;
+using std::dec;
+#endif // DEBUG
+
+namespace FbTk {
 
 template<>
 void FbTk::Resource<Slit::Placement>::setFromString(const char *strval) {
@@ -106,16 +122,16 @@ void FbTk::Resource<Slit::Placement>::setFromString(const char *strval) {
 
 template<>
 void FbTk::Resource<Slit::Direction>::setFromString(const char *strval) {
-    if (strcasecmp(strval, "Vertical") == 0) 
+    if (strcasecmp(strval, "Vertical") == 0)
         m_value = Slit::VERTICAL;
-    else if (strcasecmp(strval, "Horizontal") == 0) 
+    else if (strcasecmp(strval, "Horizontal") == 0)
         m_value = Slit::HORIZONTAL;
     else
         setDefaultValue();
 }
 
 template<>
-string FbTk::Resource<Slit::Placement>::getString() {
+string FbTk::Resource<Slit::Placement>::getString() const {
     switch (m_value) {
     case Slit::TOPLEFT:
         return string("TopLeft");
@@ -128,7 +144,7 @@ string FbTk::Resource<Slit::Placement>::getString() {
         break;
     case Slit::TOPCENTER:
         return string("TopCenter");
-        break;			
+        break;
     case Slit::BOTTOMCENTER:
         return string("BottomCenter");
         break;
@@ -147,7 +163,7 @@ string FbTk::Resource<Slit::Placement>::getString() {
 }
 
 template<>
-string FbTk::Resource<Slit::Direction>::getString() {
+string FbTk::Resource<Slit::Direction>::getString() const {
     switch (m_value) {
     case Slit::VERTICAL:
         return string("Vertical");
@@ -159,8 +175,8 @@ string FbTk::Resource<Slit::Direction>::getString() {
     // default string
     return string("Vertical");
 }
-
-namespace { 
+} // end namespace FbTk
+namespace {
 
 class SlitClientMenuItem: public FbTk::MenuItem{
 public:
@@ -170,7 +186,7 @@ public:
         FbTk::MenuItem::setSelected(client.visible());
         setToggleItem(true);
     }
-    const std::string &label() const {
+    const string &label() const {
         return m_client.matchName();
     }
     bool isSelected() const {
@@ -194,11 +210,11 @@ private:
 
 class SlitDirMenuItem: public FbTk::MenuItem {
 public:
-    SlitDirMenuItem(const char *label, Slit &slit, FbTk::RefCount<FbTk::Command> &cmd)
-        :FbTk::MenuItem(label,cmd), 
-         m_slit(slit), 
-         m_label(label ? label : "") { 
-        setLabel(m_label.c_str()); // update label
+    SlitDirMenuItem(const FbTk::FbString &label, Slit &slit, FbTk::RefCount<FbTk::Command> &cmd)
+        :FbTk::MenuItem(label,cmd),
+         m_slit(slit),
+         m_label(label) {
+        setLabel(m_label); // update label
     }
 
     void click(int button, int time) {
@@ -207,30 +223,30 @@ public:
             m_slit.setDirection(Slit::VERTICAL);
         else
             m_slit.setDirection(Slit::HORIZONTAL);
-        setLabel(m_label.c_str());
+        setLabel(m_label);
         FbTk::MenuItem::click(button, time);
     }
 
-    void setLabel(const char *label) {
+    void setLabel(const FbTk::FbString &label) {
         _FB_USES_NLS;
-        m_label = (label ? label : "");
-        std::string reallabel = m_label + " " + 
-            ( m_slit.direction() == Slit::HORIZONTAL ? 
-              
-              _FBTEXT(Align, Horizontal, "Horizontal", "Horizontal"):
-              _FBTEXT(Align, Vertical,   "Vertical",   "Vertical"));
-        FbTk::MenuItem::setLabel(reallabel.c_str());
+        m_label = (label);
+        string reallabel = m_label + " " +
+            ( m_slit.direction() == Slit::HORIZONTAL ?
+
+              _FB_XTEXT(Align, Horizontal, "Horizontal", "Horizontal"):
+              _FB_XTEXT(Align, Vertical,   "Vertical",   "Vertical"));
+        FbTk::MenuItem::setLabel(reallabel);
     }
 private:
     Slit &m_slit;
-    std::string m_label;
+    string m_label;
 };
 
 class PlaceSlitMenuItem: public FbTk::MenuItem {
 public:
-    PlaceSlitMenuItem(const char *label, Slit &slit, Slit::Placement place, FbTk::RefCount<FbTk::Command> &cmd):
+    PlaceSlitMenuItem(const FbTk::FbString &label, Slit &slit, Slit::Placement place, FbTk::RefCount<FbTk::Command> &cmd):
         FbTk::MenuItem(label, cmd), m_slit(slit), m_place(place) {
-     
+
     }
     bool isEnabled() const { return m_slit.placement() != m_place; }
     void click(int button, int time) {
@@ -244,47 +260,45 @@ private:
 
 }; // End anonymous namespace
 
-unsigned int Slit::s_eventmask = SubstructureRedirectMask |  ButtonPressMask | 
+unsigned int Slit::s_eventmask = SubstructureRedirectMask |  ButtonPressMask |
                                  EnterWindowMask | LeaveWindowMask | ExposureMask;
 
 Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
-    : m_hidden(false),
+    : m_hidden(false), m_visible(false),
       m_screen(scr),
-      m_slitmenu(scr.menuTheme(), 
-                 scr.imageControl(),
-                 *scr.layerManager().getLayer(Fluxbox::instance()->getMenuLayer())),
-      m_placement_menu(scr.menuTheme(),
-                       scr.imageControl(),
-                       *scr.layerManager().getLayer(Fluxbox::instance()->getMenuLayer())),
       m_clientlist_menu(scr.menuTheme(),
                         scr.imageControl(),
-                        *scr.layerManager().getLayer(Fluxbox::instance()->getMenuLayer())),
+                        *scr.layerManager().getLayer(Layer::MENU)),
+      m_slitmenu(scr.menuTheme(),
+                 scr.imageControl(),
+                 *scr.layerManager().getLayer(Layer::MENU)),
       frame(scr.rootWindow()),
        //For KDE dock applets
-      m_kwm1_dockwindow(XInternAtom(FbTk::App::instance()->display(), 
+      m_kwm1_dockwindow(XInternAtom(FbTk::App::instance()->display(),
                                     "KWM_DOCKWINDOW", False)), //KDE v1.x
-      m_kwm2_dockwindow(XInternAtom(FbTk::App::instance()->display(), 
+      m_kwm2_dockwindow(XInternAtom(FbTk::App::instance()->display(),
                                     "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False)), //KDE v2.x
 
       m_layeritem(0),
+
       m_slit_theme(new SlitTheme(scr.rootWindow().screenNumber())),
       m_strut(0),
       // resources
       // lock in first resource
-      m_rc_auto_hide(scr.resourceManager().lock(), false, 
+      m_rc_auto_hide(scr.resourceManager().lock(), false,
                      scr.name() + ".slit.autoHide", scr.altName() + ".Slit.AutoHide"),
       // TODO: this resource name must change
       m_rc_maximize_over(scr.resourceManager(), false,
                          scr.name() + ".slit.maxOver", scr.altName() + ".Slit.MaxOver"),
       m_rc_placement(scr.resourceManager(), BOTTOMRIGHT,
                      scr.name() + ".slit.placement", scr.altName() + ".Slit.Placement"),
-      m_rc_direction(scr.resourceManager(), VERTICAL, 
+      m_rc_direction(scr.resourceManager(), VERTICAL,
                      scr.name() + ".slit.direction", scr.altName() + ".Slit.Direction"),
-      m_rc_alpha(scr.resourceManager(), 255, 
+      m_rc_alpha(scr.resourceManager(), 255,
                  scr.name() + ".slit.alpha", scr.altName() + ".Slit.Alpha"),
       m_rc_on_head(scr.resourceManager(), 0,
                    scr.name() + ".slit.onhead", scr.altName() + ".Slit.onHead"),
-      m_rc_layernum(scr.resourceManager(), Fluxbox::Layer(Fluxbox::instance()->getDockLayer()), 
+      m_rc_layernum(scr.resourceManager(), Layer(Layer::DOCK),
                     scr.name() + ".slit.layer", scr.altName() + ".Slit.Layer") {
 
     _FB_USES_NLS;
@@ -292,8 +306,11 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
     // attach to theme and root window change signal
     m_slit_theme->reconfigSig().attach(this);
     scr.resizeSig().attach(this);
+    scr.bgChangeSig().attach(this);
     scr.reconfigureSig().attach(this); // if alpha changed (we disablethis signal when we get theme change sig)
 
+    scr.addConfigMenu(_FB_XTEXT(Slit, Slit, "Slit", "The Slit"), m_slitmenu);
+
     frame.pixmap = None;
     // move the frame out of sight for a moment
     frame.window.move(-frame.window.width(), -frame.window.height());
@@ -305,7 +322,7 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
 
 
     FbTk::EventManager::instance()->add(*this, frame.window);
-    
+
     if (FbTk::Transparent::haveComposite()) {
         frame.window.setOpaque(*m_rc_alpha);
     } else {
@@ -313,16 +330,18 @@ Slit::Slit(BScreen &scr, FbTk::XLayer &layer, const char *filename)
     }
 
     m_layeritem.reset(new FbTk::XLayerItem(frame.window, layer));
-    m_layermenu.reset(new LayerMenu<Slit>(scr.menuTheme(),
-                                          scr.imageControl(),
-                                          *scr.layerManager().
-                                          getLayer(Fluxbox::instance()->getMenuLayer()), 
-                                          this,
-                                          true));
+
+    m_layermenu.reset(new LayerMenu(scr.menuTheme(),
+                                    scr.imageControl(),
+                                    *scr.layerManager().
+                                    getLayer(Layer::MENU),
+                                    this,
+                                    true));
+    m_layermenu->setLabel(_FB_XTEXT(Slit, Layer, "Slit Layer", "Title of Slit Layer Menu"));
+
     moveToLayer((*m_rc_layernum).getNum());
 
-    if (m_layermenu.get())
-        m_layermenu->setLabel(_FBTEXT(Slit, Layer, "Slit Layer", "Title of Slit Layer Menu"));
+
 
     // Get client list for sorting purposes
     loadClientList(filename);
@@ -338,6 +357,9 @@ Slit::~Slit() {
     if (frame.pixmap != 0)
         screen().imageControl().removeImage(frame.pixmap);
 
+    // otherwise it will try to access it on deletion
+    screen().removeConfigMenu(m_slitmenu);
+
     shutdown();
 }
 
@@ -352,7 +374,8 @@ void Slit::updateStrut() {
     bool had_strut = m_strut ? true : false;
     clearStrut();
     // no need for area if we're autohiding or set maximize over
-    if (doAutoHide() || *m_rc_maximize_over) {
+    // or if we dont have any clients
+    if (doAutoHide() || *m_rc_maximize_over || !m_visible) {
         // update screen area if we had a strut before
         if (had_strut)
             screen().updateAvailableWorkspaceArea();
@@ -420,7 +443,7 @@ void Slit::addClient(Window w) {
 
     // Look for slot in client list by name
     SlitClient *client = 0;
-    std::string match_name;
+    string match_name;
     match_name = Xutil::getWMClassName(w);
     SlitClients::iterator it = m_client_list.begin();
     SlitClients::iterator it_end = m_client_list.end();
@@ -436,7 +459,7 @@ void Slit::addClient(Window w) {
             }
             // Otherwise keep looking for an unused match or a non-match
             found_match = true;		// Possibly redundant
-			
+
         } else if (found_match) {
             // Insert before first non-match after a previously found match?
             client = new SlitClient(&screen(), w);
@@ -457,7 +480,7 @@ void Slit::addClient(Window w) {
         if ((wmhints->flags & IconWindowHint) &&
             (wmhints->icon_window != None)) {
             XMoveWindow(disp, client->clientWindow(), screen().width() + 10,
-                        screen().height() + 10);            
+                        screen().height() + 10);
             XMapWindow(disp, client->clientWindow());
             client->setIconWindow(wmhints->icon_window);
             client->setWindow(client->iconWindow());
@@ -477,12 +500,12 @@ void Slit::addClient(Window w) {
     FbAtoms *fbatoms = FbAtoms::instance();
 
     if (XGetWMProtocols(disp, w, &proto, &num_return)) {
-        
+
         for (int i = 0; i < num_return; ++i) {
             if (proto[i] == fbatoms->getFluxboxStructureMessagesAtom())
                 screen().addNetizen(w);
         }
-        
+
         XFree((void *) proto);
 #ifdef DEBUG
     } else {
@@ -491,13 +514,13 @@ void Slit::addClient(Window w) {
     }
 
     XWindowAttributes attrib;
-    
+
 #ifdef KDE
-    if (screen().isKdeDockapp(w))   
+    if (screen().isKdeDockapp(w))
         client->resize(24, 24);
     else
 #endif // KDE
-    
+
         {
             if (XGetWindowAttributes(disp, client->window(), &attrib)) {
                 client->resize(attrib.width, attrib.height);
@@ -512,7 +535,7 @@ void Slit::addClient(Window w) {
     // disable events to frame.window
     frame.window.setEventMask(NoEventMask);
     client->disableEvents();
-    
+
 
     XReparentWindow(disp, client->window(), frame.window.window(), 0, 0);
     XMapRaised(disp, client->window());
@@ -521,7 +544,7 @@ void Slit::addClient(Window w) {
     frame.window.setEventMask(s_eventmask);
     // setup event for slit client window
     client->enableEvents();
-    
+
     // flush events
     //    XFlush(disp);
 
@@ -640,11 +663,18 @@ void Slit::reconfigure() {
         // client created window?
         if ((*client_it)->window() != None && (*client_it)->visible()) {
             num_windows++;
+
+            // get the dockapps to update their backgrounds
+            if (screen().isKdeDockapp((*client_it)->window())) {
+                (*client_it)->hide();
+                (*client_it)->show();
+            }
+
             if (height_inc) {
                 // increase height of slit for each client (VERTICAL mode)
-                frame.height += (*client_it)->height() + bevel_width;		
+                frame.height += (*client_it)->height() + bevel_width;
                 // the slit should always have the width of the largest client
-                if (frame.width < (*client_it)->width()) 
+                if (frame.width < (*client_it)->width())
                     frame.width = (*client_it)->width();
             } else {
                 // increase width of slit for each client (HORIZONTAL mode)
@@ -666,16 +696,10 @@ void Slit::reconfigure() {
     else
         frame.height += bevel_width*2;
 
-    reposition();
     Display *disp = FbTk::App::instance()->display();
 
     frame.window.setBorderWidth(theme().borderWidth());
     frame.window.setBorderColor(theme().borderColor());
-    // did we actually use slit slots
-    if (num_windows == 0)
-        frame.window.hide();
-    else
-        frame.window.show();
 
     Pixmap tmp = frame.pixmap;
     FbTk::ImageControl &image_ctrl = screen().imageControl();
@@ -692,15 +716,25 @@ void Slit::reconfigure() {
             frame.window.setBackgroundPixmap(frame.pixmap);
     }
 
-    if (tmp) 
+    if (tmp)
         image_ctrl.removeImage(tmp);
 
+    // could have changed types, so we must set both
     if (FbTk::Transparent::haveComposite()) {
+        frame.window.setAlpha(255);
         frame.window.setOpaque(*m_rc_alpha);
     } else {
         frame.window.setAlpha(*m_rc_alpha);
+        frame.window.setOpaque(255);
     }
-    clearWindow();
+    // reposition clears the bg
+    reposition();
+
+    // did we actually use slit slots
+    if (num_windows == 0)
+        hide();
+    else
+        show();
 
     int x = 0, y = 0;
     height_inc = false;
@@ -723,7 +757,7 @@ void Slit::reconfigure() {
             continue;
 
         //client created window?
-        if ((*client_it)->visible()) 
+        if ((*client_it)->visible())
             (*client_it)->show();
         else {
             (*client_it)->disableEvents();
@@ -766,7 +800,7 @@ void Slit::reconfigure() {
             x += (*client_it)->width() + bevel_width;
     } // end for
 
-    if (doAutoHide() && !isHidden() && !m_timer.isTiming()) 
+    if (doAutoHide() && !isHidden() && !m_timer.isTiming())
         m_timer.start();
     else if (!doAutoHide() && isHidden())
         toggleHidden(); // restore visible
@@ -826,7 +860,7 @@ void Slit::reposition() {
 
     case BOTTOMLEFT:
         frame.x = head_x;
-        frame.y = head_h - frame.height - border_width*2;
+        frame.y = head_y + head_h - frame.height - border_width*2;
         if (direction() == VERTICAL) {
             frame.x_hidden = head_x + bevel_width -
                 border_width - frame.width;
@@ -881,12 +915,12 @@ void Slit::reposition() {
         frame.x = head_x + head_w - frame.width - border_width*2;
         frame.y = head_y + head_h - frame.height - border_width*2;
         if (direction() == VERTICAL) {
-            frame.x_hidden = head_x + head_w - 
+            frame.x_hidden = head_x + head_w -
                 bevel_width - border_width;
             frame.y_hidden = frame.y;
         } else {
             frame.x_hidden = frame.x;
-            frame.y_hidden = head_y + head_h - 
+            frame.y_hidden = head_y + head_h -
                 bevel_width - border_width;
         }
         break;
@@ -899,7 +933,9 @@ void Slit::reposition() {
         frame.window.moveResize(frame.x,  frame.y,
                                 frame.width, frame.height);
     }
-
+    frame.window.updateBackground(true);
+    if (*m_rc_alpha != 255)
+        clearWindow();
 }
 
 
@@ -912,12 +948,12 @@ void Slit::shutdown() {
 void Slit::clientUp(SlitClient* client) {
     if (!client || m_client_list.size() < 2)
         return;
-    
+
     if (client == m_client_list.front()) {
         cycleClientsUp();
         return;
     }
-    
+
     SlitClients::iterator it = m_client_list.begin();
     for(it++; it != m_client_list.end(); it++) {
         if ((*it) == client) {
@@ -933,12 +969,12 @@ void Slit::clientUp(SlitClient* client) {
 void Slit::clientDown(SlitClient* client) {
     if (!client || m_client_list.size() < 2)
         return;
-    
+
     if (client == m_client_list.back()) {
         cycleClientsDown();
         return;
     }
-    
+
     SlitClients::reverse_iterator it = m_client_list.rbegin();
     for(it++; it != m_client_list.rend(); it++) {
         if ((*it) == client) {
@@ -987,27 +1023,27 @@ void Slit::handleEvent(XEvent &event) {
     }
 }
 
-void Slit::buttonPressEvent(XButtonEvent &e) {
-    if (e.window != frame.window.window()) 
+void Slit::buttonPressEvent(XButtonEvent &be) {
+    if (be.window != frame.window.window())
         return;
 
-    if (e.button == Button3) {
+    if (be.button == Button3) {
         if (! m_slitmenu.isVisible()) {
-            int x = e.x_root - (m_slitmenu.width() / 2),
-                y = e.y_root - (m_slitmenu.height() / 2); 
+            int head = screen().getHead(be.x_root, be.y_root);
+            int borderw = m_slitmenu.fbwindow().borderWidth();
+            pair<int, int> m = screen().clampToHead(head,
+                    be.x_root - (m_slitmenu.width() / 2),
+                    be.y_root - (m_slitmenu.titleWindow().height() / 2),
+                    m_slitmenu.width() + 2*borderw,
+                    m_slitmenu.height() + 2*borderw);
 
-            if (x < 0)
-                x = 0;
-            else if (x + m_slitmenu.width() > screen().width())
-                x = screen().width() - m_slitmenu.width();
-
-            if (y < 0)
-                y = 0;
-            else if (y + m_slitmenu.height() > screen().height())
-                y = screen().height() - m_slitmenu.height();
-
-            m_slitmenu.move(x, y);
+            m_slitmenu.setScreen(screen().getHeadX(head),
+                                 screen().getHeadY(head),
+                                 screen().getHeadWidth(head),
+                                 screen().getHeadHeight(head));
+            m_slitmenu.move(m.first, m.second);
             m_slitmenu.show();
+            m_slitmenu.grabInputFocus();
         } else
             m_slitmenu.hide();
     }
@@ -1019,10 +1055,10 @@ void Slit::enterNotifyEvent(XCrossingEvent &) {
         return;
 
     if (isHidden()) {
-        if (! m_timer.isTiming()) 
+        if (! m_timer.isTiming())
             m_timer.start();
     } else {
-        if (m_timer.isTiming()) 
+        if (m_timer.isTiming())
             m_timer.stop();
     }
 }
@@ -1033,12 +1069,12 @@ void Slit::leaveNotifyEvent(XCrossingEvent &ev) {
         return;
 
     if (isHidden()) {
-        if (m_timer.isTiming()) 
+        if (m_timer.isTiming())
             m_timer.stop();
     } else {
         if (! m_timer.isTiming()) {
             // the menu is open, keep it firing until it closes
-            if (m_slitmenu.isVisible()) 
+            if (m_slitmenu.isVisible())
                 m_timer.fireOnce(false);
             m_timer.start();
         }
@@ -1059,7 +1095,7 @@ void Slit::configureRequestEvent(XConfigureRequestEvent &event) {
     xwc.sibling = event.above;
     xwc.stack_mode = event.detail;
 
-    XConfigureWindow(FbTk::App::instance()->display(), 
+    XConfigureWindow(FbTk::App::instance()->display(),
                      event.window, event.value_mask, &xwc);
 
     SlitClients::iterator it = m_client_list.begin();
@@ -1077,12 +1113,12 @@ void Slit::configureRequestEvent(XConfigureRequestEvent &event) {
         }
     }
 
-    if (reconf) 
+    if (reconf)
         reconfigure();
 }
 
 void Slit::exposeEvent(XExposeEvent &ev) {
-    // we don't need to clear the entire window 
+    // we don't need to clear the entire window
     // just the are that gets exposed
     frame.window.clearArea(ev.x, ev.y, ev.width, ev.height);
 }
@@ -1099,11 +1135,11 @@ void Slit::toggleHidden() {
     if (doAutoHide()) {
         if (!m_slitmenu.isVisible()) {
             m_timer.fireOnce(true);
-        } else 
+        } else
             return;
-    } else 
+    } else
         if (!isHidden()) return;
-        
+
     m_hidden = ! m_hidden; // toggle hidden state
     if (isHidden())
         frame.window.move(frame.x_hidden, frame.y_hidden);
@@ -1117,18 +1153,18 @@ void Slit::loadClientList(const char *filename) {
 
     // save filename so we can save client list later
     m_filename = filename;
-    std::string real_filename= FbTk::StringUtil::expandFilename(filename); 
+    string real_filename= FbTk::StringUtil::expandFilename(filename);
 
     struct stat buf;
     if (stat(real_filename.c_str(), &buf) == 0) {
-        std::ifstream file(real_filename.c_str());
-        std::string name;
+        ifstream file(real_filename.c_str());
+        string name;
         while (! file.eof()) {
             name = "";
-            std::getline(file, name); // get the entire line
+            getline(file, name); // get the entire line
             if (name.empty())
                 continue;
-             
+
             // remove whitespaces from start and end
             FbTk::StringUtil::removeFirstWhitespace(name);
 
@@ -1146,18 +1182,18 @@ void Slit::loadClientList(const char *filename) {
 }
 
 void Slit::updateClientmenu() {
-    if (screen().isShuttingdown()) 
+    if (screen().isShuttingdown())
         return;
     _FB_USES_NLS;
 
     // clear old items
     m_clientlist_menu.removeAll();
-    m_clientlist_menu.setLabel(_FBTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"));
+    m_clientlist_menu.setLabel(_FB_XTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"));
 
     FbTk::RefCount<FbTk::Command> cycle_up(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsUp));
     FbTk::RefCount<FbTk::Command> cycle_down(new FbTk::SimpleCommand<Slit>(*this, &Slit::cycleClientsDown));
-    m_clientlist_menu.insert(_FBTEXT(Slit, CycleUp, "Cycle Up", "Cycle clients upwards"), cycle_up);
-    m_clientlist_menu.insert(_FBTEXT(Slit, CycleDown, "Cycle Down", "Cycle clients downwards"), cycle_down);
+    m_clientlist_menu.insert(_FB_XTEXT(Slit, CycleUp, "Cycle Up", "Cycle clients upwards"), cycle_up);
+    m_clientlist_menu.insert(_FB_XTEXT(Slit, CycleDown, "Cycle Down", "Cycle clients downwards"), cycle_down);
 
     m_clientlist_menu.insert(new FbTk::MenuSeparator());
 
@@ -1170,9 +1206,9 @@ void Slit::updateClientmenu() {
 
     m_clientlist_menu.insert(new FbTk::MenuSeparator());
     FbTk::RefCount<FbTk::Command> savecmd(new FbTk::SimpleCommand<Slit>(*this, &Slit::saveClientList));
-    m_clientlist_menu.insert(_FBTEXT(Slit, 
+    m_clientlist_menu.insert(_FB_XTEXT(Slit,
                                      SaveSlitList,
-                                     "Save SlitList", "Saves the current order in the slit"), 
+                                     "Save SlitList", "Saves the current order in the slit"),
                              savecmd);
 
     m_clientlist_menu.updateMenu();
@@ -1180,31 +1216,20 @@ void Slit::updateClientmenu() {
 
 void Slit::saveClientList() {
 
-    std::ofstream file(FbTk::StringUtil::expandFilename(m_filename).c_str());
+    ofstream file(FbTk::StringUtil::expandFilename(m_filename).c_str());
     SlitClients::iterator it = m_client_list.begin();
     SlitClients::iterator it_end = m_client_list.end();
-    std::string prevName;
-    std::string name;
+    string prevName;
+    string name;
     for (; it != it_end; ++it) {
         name = (*it)->matchName();
         if (name != prevName)
-            file << name.c_str() << std::endl;
+            file << name.c_str() << endl;
 
         prevName = name;
     }
 }
 
-void Slit::setAutoHide(bool val) {
-    *m_rc_auto_hide = val;
-    if (doAutoHide()) {
-        if (! m_timer.isTiming()) {
-            if (m_slitmenu.isVisible())
-                m_timer.fireOnce(false);
-            m_timer.start();
-        }
-    }
-}
-
 void Slit::setupMenu() {
     _FB_USES_NLS;
     using namespace FbTk;
@@ -1224,42 +1249,49 @@ void Slit::setupMenu() {
     FbTk::RefCount<FbTk::Command> save_and_reconfigure(s_a_reconf_macro);
     FbTk::RefCount<FbTk::Command> save_and_reconfigure_slit(s_a_reconf_slit_macro);
 
-    // setup base menu
-    m_slitmenu.setLabel(_FBTEXT(Slit, Slit, "Slit", "The Slit"));
-    m_slitmenu.insert(_FBTEXT(Menu, Placement, "Placement", "Title of Placement menu"),
-                      &m_placement_menu);
 
-    m_slitmenu.insert(_FBTEXT(Menu, Layer, "Layer...", "Title of Layer menu"), m_layermenu.get());
+    // it'll be freed by the slitmenu (since not marked internal)
+    FbMenu *placement_menu = new FbMenu(m_screen.menuTheme(),
+                                        m_screen.imageControl(),
+                                        *m_screen.layerManager().getLayer(::Layer::MENU));
+
+
+    // setup base menu
+    m_slitmenu.setLabel(_FB_XTEXT(Slit, Slit, "Slit", "The Slit"));
+    m_slitmenu.insert(_FB_XTEXT(Menu, Placement, "Placement", "Title of Placement menu"),
+                      placement_menu);
+
+    m_slitmenu.insert(_FB_XTEXT(Menu, Layer, "Layer...", "Title of Layer menu"), m_layermenu.get());
 
 #ifdef XINERAMA
     if (screen().hasXinerama()) {
-        m_slitmenu.insert(_FBTEXT(Menu, OnHead, "On Head...", "Title of On Head menu"),
+        m_slitmenu.insert(_FB_XTEXT(Menu, OnHead, "On Head...", "Title of On Head menu"),
                           new XineramaHeadMenu<Slit>(
                               screen().menuTheme(),
                               screen(),
                               screen().imageControl(),
-                              *screen().layerManager().getLayer(Fluxbox::instance()->getMenuLayer()),
+                              *screen().layerManager().getLayer(::Layer::MENU),
                               *this,
-                              _FBTEXT(Slit, OnHead, "Slit on Head", "Title of Slits On Head menu")
+                              _FB_XTEXT(Slit, OnHead, "Slit on Head", "Title of Slits On Head menu")
                               ));
     }
-                    
 #endif //XINERAMA
-    m_slitmenu.insert(new BoolMenuItem(_FBTEXT(Common, AutoHide, "Auto hide", "This thing automatically hides when not close by"),
+
+    m_slitmenu.insert(new BoolMenuItem(_FB_XTEXT(Common, AutoHide, "Auto hide", "This thing automatically hides when not close by"),
                                        *m_rc_auto_hide,
                                        save_and_reconfigure_slit));
 
-    m_slitmenu.insert(new BoolMenuItem(_FBTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"), 
-                                       *m_rc_maximize_over, 
+    m_slitmenu.insert(new BoolMenuItem(_FB_XTEXT(Common, MaximizeOver,"Maximize Over", "Maximize over this thing when maximizing"),
+                                       *m_rc_maximize_over,
                                        save_and_reconfigure_slit));
 
     // this saves resources and clears the slit window to update alpha value
-    FbTk::MenuItem *alpha_menuitem = 
-        new IntResMenuItem(_FBTEXT(Common, Alpha, "Alpha", "Transparency level"),
+    FbTk::MenuItem *alpha_menuitem =
+        new IntResMenuItem< FbTk::Resource<int> >(_FB_XTEXT(Common, Alpha, "Alpha", "Transparency level"),
                            m_rc_alpha,
                            0, 255, m_slitmenu);
     // setup command for alpha value
-    MacroCommand *alpha_macrocmd = new MacroCommand(); 
+    MacroCommand *alpha_macrocmd = new MacroCommand();
     RefCount<Command> alpha_cmd(new SimpleCommand<Slit>(*this, &Slit::updateAlpha));
     alpha_macrocmd->add(saverc_cmd);
     alpha_macrocmd->add(alpha_cmd);
@@ -1268,53 +1300,54 @@ void Slit::setupMenu() {
 
     m_slitmenu.insert(alpha_menuitem);
 
-    m_slitmenu.insert(new SlitDirMenuItem(_FBTEXT(Slit, Direction, "Slit Direction", "Orientation of slit"), 
+    m_slitmenu.insert(new SlitDirMenuItem(_FB_XTEXT(Slit, Direction, "Slit Direction", "Orientation of slit"),
                                           *this,
                                           save_and_reconfigure));
-    m_slitmenu.insert(_FBTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"), &m_clientlist_menu);
+    m_slitmenu.insert(_FB_XTEXT(Slit, ClientsMenu, "Clients", "Slit client menu"), &m_clientlist_menu);
     m_slitmenu.updateMenu();
 
     // setup sub menu
-    m_placement_menu.setLabel(_FBTEXT(Slit, Placement, "Slit Placement", "Slit Placement"));
-    m_placement_menu.setMinimumSublevels(3);
+    placement_menu->setLabel(_FB_XTEXT(Slit, Placement, "Slit Placement", "Slit Placement"));
+    placement_menu->setMinimumSublevels(3);
     m_layermenu->setInternalMenu();
     m_clientlist_menu.setInternalMenu();
-   
-    typedef pair<const char *, Slit::Placement> PlacementP;
+    m_slitmenu.setInternalMenu();
+
+    typedef pair<FbTk::FbString, Slit::Placement> PlacementP;
     typedef list<PlacementP> Placements;
     Placements place_menu;
 
     // menu is 3 wide, 5 down
-    place_menu.push_back(PlacementP(_FBTEXT(Align, TopLeft, "Top Left", "Top Left"), Slit::TOPLEFT));
-    place_menu.push_back(PlacementP(_FBTEXT(Align, LeftCenter, "Left Center", "Left Center"), Slit::LEFTCENTER));
-    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomLeft, "Bottom Left", "Bottom Left"), Slit::BOTTOMLEFT));
-    place_menu.push_back(PlacementP(_FBTEXT(Align, TopCenter, "Top Center", "Top Center"), Slit::TOPCENTER));
-    place_menu.push_back(PlacementP((const char *)0, Slit::TOPLEFT));
-    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomCenter, "Bottom Center", "Bottom Center"), Slit::BOTTOMCENTER));
-    place_menu.push_back(PlacementP(_FBTEXT(Align, TopRight, "Top Right", "Top Right"), Slit::TOPRIGHT));
-    place_menu.push_back(PlacementP(_FBTEXT(Align, RightCenter, "Right Center", "Right Center"), Slit::RIGHTCENTER));
-    place_menu.push_back(PlacementP(_FBTEXT(Align, BottomRight, "Bottom Right", "Bottom Right"), Slit::BOTTOMRIGHT));
-    
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, TopLeft, "Top Left", "Top Left"), Slit::TOPLEFT));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, LeftCenter, "Left Center", "Left Center"), Slit::LEFTCENTER));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, BottomLeft, "Bottom Left", "Bottom Left"), Slit::BOTTOMLEFT));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, TopCenter, "Top Center", "Top Center"), Slit::TOPCENTER));
+    place_menu.push_back(PlacementP("", Slit::TOPLEFT));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, BottomCenter, "Bottom Center", "Bottom Center"), Slit::BOTTOMCENTER));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, TopRight, "Top Right", "Top Right"), Slit::TOPRIGHT));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, RightCenter, "Right Center", "Right Center"), Slit::RIGHTCENTER));
+    place_menu.push_back(PlacementP(_FB_XTEXT(Align, BottomRight, "Bottom Right", "Bottom Right"), Slit::BOTTOMRIGHT));
+
 
     // create items in sub menu
     for (size_t i=0; i<9; ++i) {
-        const char *str = place_menu.front().first;
+        const FbTk::FbString &str = place_menu.front().first;
         Slit::Placement placement = place_menu.front().second;
 
-        if (str == 0) {
-            m_placement_menu.insert("");
-            m_placement_menu.setItemEnabled(i, false);
+        if (str == "") {
+            placement_menu->insert("");
+            placement_menu->setItemEnabled(i, false);
         } else {
-            m_placement_menu.insert(new PlaceSlitMenuItem(str, *this,
-                                                          placement, 
+            placement_menu->insert(new PlaceSlitMenuItem(str, *this,
+                                                          placement,
                                                           save_and_reconfigure));
-                                                              
+
         }
         place_menu.pop_front();
     }
 
     // finaly update sub menu
-    m_placement_menu.updateMenu();
+    placement_menu->updateMenu();
 }
 
 void Slit::moveToLayer(int layernum) {
@@ -1334,6 +1367,7 @@ void Slit::updateAlpha() {
         frame.window.setOpaque(*m_rc_alpha);
     } else {
         frame.window.setAlpha(*m_rc_alpha);
+        frame.window.updateBackground(true);
         clearWindow();
     }
 }
