@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: SystemTray.cc 4199 2006-02-16 06:53:05Z mathias $
+// $Id: SystemTray.cc 5022 2007-08-06 04:19:48Z simonb $
 
 #include "SystemTray.hh"
 
@@ -36,12 +36,65 @@
 
 #include <string>
 
-using namespace std;
+using std::string;
+
+#ifdef DEBUG
+#include <iostream>
+using std::cerr;
+using std::endl;
+using std::hex;
+using std::dec;
+#endif // DEBUG
 
 /// helper class for tray windows, so we dont call XDestroyWindow
 class TrayWindow: public FbTk::FbWindow {
 public:
-    TrayWindow(Window win):FbTk::FbWindow(win) { }
+    TrayWindow(Window win, bool using_xembed):FbTk::FbWindow(win), m_visible(false), m_xembedded(using_xembed) {
+        setEventMask(PropertyChangeMask);
+    }
+
+    bool isVisible() { return m_visible; }
+    bool isXEmbedded() { return m_xembedded; }
+    void show() {
+        if (!m_visible) {
+            m_visible = true;
+            FbTk::FbWindow::show();
+        }
+    }
+    void hide() {
+        if (m_visible) {
+            m_visible = false;
+            FbTk::FbWindow::hide();
+        }
+    }
+
+/* Flags for _XEMBED_INFO */
+#define XEMBED_MAPPED                   (1 << 0)
+
+    bool getMappedDefault() const {
+        Atom actual_type;
+        int actual_format;
+        unsigned long nitems, bytes_after;
+        unsigned long *prop;
+        bool mapped = false;
+        Atom embed_info = SystemTray::getXEmbedInfoAtom();
+        if (property(embed_info, 0l, 2l, false, embed_info,
+                     &actual_type, &actual_format, &nitems, &bytes_after,
+                     (unsigned char **) &prop) && prop != 0) {
+            mapped = (bool)(static_cast<unsigned long>(prop[1]) & XEMBED_MAPPED);
+            XFree(static_cast<void *>(prop));
+
+#ifdef DEBUG
+            cerr<<__FILE__<<"(SystemTray::TrayWindow::getMappedDefault(): XEMBED_MAPPED = "<<mapped<<endl;
+#endif // DEBUG
+
+        }
+        return true;
+    }
+
+private:
+    bool m_visible;
+    bool m_xembedded; // using xembed protocol? (i.e. unmap when done)
 };
 
 /// handles clientmessage event and notifies systemtray
@@ -77,7 +130,7 @@ public:
             return;
         winclient.setEventMask(StructureNotifyMask |
                                SubstructureNotifyMask | EnterWindowMask);
-        m_tray.addClient(winclient.window());
+        m_tray.addClient(winclient.window(), false);
 
     };
 
@@ -107,10 +160,13 @@ SystemTray::SystemTray(const FbTk::FbWindow& parent, ButtonTheme& theme, BScreen
              SubstructureNotifyMask | SubstructureRedirectMask),
     m_theme(theme),
     m_screen(screen),
-    m_pixmap(0) {
-
+    m_pixmap(0), m_num_visible_clients(0),
+    m_selection_owner(m_window, 0, 0, 1, 1, SubstructureNotifyMask, false, false, CopyFromParent, InputOnly) {
+    
     FbTk::EventManager::instance()->add(*this, m_window);
+    FbTk::EventManager::instance()->add(*this, m_selection_owner);
     m_theme.reconfigSig().attach(this);
+    screen.bgChangeSig().attach(this);
 
     Fluxbox* fluxbox = Fluxbox::instance();
     Display *disp = fluxbox->display();
@@ -118,7 +174,7 @@ SystemTray::SystemTray(const FbTk::FbWindow& parent, ButtonTheme& theme, BScreen
     // setup atom name to _NET_SYSTEM_TRAY_S<screen number>
     char intbuff[16];
     sprintf(intbuff, "%d", m_window.screenNumber());
-    std::string atom_name("_NET_SYSTEM_TRAY_S");
+    string atom_name("_NET_SYSTEM_TRAY_S");
     atom_name += intbuff; // append number
 
     // get selection owner and see if it's free
@@ -136,7 +192,7 @@ SystemTray::SystemTray(const FbTk::FbWindow& parent, ButtonTheme& theme, BScreen
     cerr<<__FILE__<<"(SystemTray(const FbTk::FbWindow)): SETTING OWNER!"<<endl;
 #endif // DEBUG
     // set owner
-    XSetSelectionOwner(disp, tray_atom, m_window.window(), CurrentTime);
+    XSetSelectionOwner(disp, tray_atom, m_selection_owner.window(), CurrentTime);
 
     m_handler.reset(new SystemTrayHandler(*this));
 
@@ -153,7 +209,7 @@ SystemTray::SystemTray(const FbTk::FbWindow& parent, ButtonTheme& theme, BScreen
     ce.xclient.format = 32;
     ce.xclient.data.l[0] = CurrentTime; // timestamp
     ce.xclient.data.l[1] = tray_atom; // manager selection atom
-    ce.xclient.data.l[2] = m_window.window(); // the window owning the selection
+    ce.xclient.data.l[2] = m_selection_owner.window(); // the window owning the selection
     ce.xclient.data.l[3] = 0l; // selection specific data
     ce.xclient.data.l[4] = 0l; // selection specific data
 
@@ -164,7 +220,20 @@ SystemTray::SystemTray(const FbTk::FbWindow& parent, ButtonTheme& theme, BScreen
 
 SystemTray::~SystemTray() {
     // remove us, else fluxbox might delete the memory too
-    Fluxbox::instance()->removeAtomHandler(m_handler.get());
+    Fluxbox* fluxbox = Fluxbox::instance();
+    fluxbox->removeAtomHandler(m_handler.get());
+    Display *disp = fluxbox->display();
+    // setup atom name to _NET_SYSTEM_TRAY_S<screen number>
+    char intbuff[16];
+    sprintf(intbuff, "%d", m_window.screenNumber());
+    string atom_name("_NET_SYSTEM_TRAY_S");
+    atom_name += intbuff; // append number
+
+    // get selection owner and see if it's free
+    Atom tray_atom = XInternAtom(disp, atom_name.c_str(), False);
+
+    // Properly give up selection.
+    XSetSelectionOwner(disp, tray_atom, None, CurrentTime);
     removeAllClients();
 
     if (m_pixmap)
@@ -181,10 +250,9 @@ void SystemTray::resize(unsigned int width, unsigned int height) {
     if (width != m_window.width() ||
         height != m_window.height()) {
         m_window.resize(width, height);
-        if (!m_clients.empty()) {
+        if (m_num_visible_clients)
             rearrangeClients();
-            resizeSig().notify();
-        }
+        resizeSig().notify();
     }
 }
 
@@ -193,10 +261,9 @@ void SystemTray::moveResize(int x, int y,
     if (width != m_window.width() ||
         height != m_window.height()) {
         m_window.moveResize(x, y, width, height);
-        if (!m_clients.empty()) {
+        if (m_num_visible_clients)
             rearrangeClients();
-            resizeSig().notify();
-        }
+        resizeSig().notify();
     } else {
         move(x, y);
     }
@@ -213,11 +280,17 @@ void SystemTray::show() {
 }
 
 unsigned int SystemTray::width() const {
-    return m_clients.size()* (height() - 2 * m_theme.border().width());
+    if (orientation() == FbTk::ROT90 || orientation() == FbTk::ROT270)
+        return m_window.width();
+
+    return m_num_visible_clients * (height() - 2 * m_theme.border().width());
 }
 
 unsigned int SystemTray::height() const {
-    return m_window.height();
+    if (orientation() == FbTk::ROT0 || orientation() == FbTk::ROT180)
+        return m_window.height();
+
+    return m_num_visible_clients * (width() - 2 * m_theme.border().width());
 }
 
 unsigned int SystemTray::borderWidth() const {
@@ -228,17 +301,17 @@ bool SystemTray::clientMessage(const XClientMessageEvent &event) {
     static const int SYSTEM_TRAY_REQUEST_DOCK  =  0;
     //    static const int SYSTEM_TRAY_BEGIN_MESSAGE =  1;
     //    static const int SYSTEM_TRAY_CANCEL_MESSAGE = 2;
+    static Atom systray_opcode_atom = XInternAtom(FbTk::App::instance()->display(), "_NET_SYSTEM_TRAY_OPCODE", False);
 
-    if (event.message_type ==
-        XInternAtom(FbTk::App::instance()->display(), "_NET_SYSTEM_TRAY_OPCODE", False)) {
+    if (event.message_type == systray_opcode_atom) {
 
         int type = event.data.l[1];
         if (type == SYSTEM_TRAY_REQUEST_DOCK) {
-#ifndef DEBUG
+#ifdef DEBUG
             cerr<<"SystemTray::clientMessage(const XClientMessageEvent): SYSTEM_TRAY_REQUEST_DOCK"<<endl;
             cerr<<"window = event.data.l[2] = "<<event.data.l[2]<<endl;
 #endif // DEBUG
-            addClient(event.data.l[2]);
+            addClient(event.data.l[2], true);
         }
         /*
         else if (type == SYSTEM_TRAY_BEGIN_MESSAGE)
@@ -265,7 +338,7 @@ SystemTray::ClientList::iterator SystemTray::findClient(Window win) {
     return it;
 }
 
-void SystemTray::addClient(Window win) {
+void SystemTray::addClient(Window win, bool using_xembed) {
     if (win == 0)
         return;
 
@@ -273,39 +346,52 @@ void SystemTray::addClient(Window win) {
     if (it != m_clients.end())
         return;
 
+    Display *disp = Fluxbox::instance()->display();
     // make sure we have the same screen number
     XWindowAttributes attr;
     attr.screen = 0;
-    if (XGetWindowAttributes(FbTk::App::instance()->display(),
-                             win, &attr) != 0 &&
+    if (XGetWindowAttributes(disp, win, &attr) != 0 &&
         attr.screen != 0 &&
         XScreenNumberOfScreen(attr.screen) != window().screenNumber()) {
         return;
     }
 
-    FbTk::FbWindow *traywin = new TrayWindow(win);
+    TrayWindow *traywin = new TrayWindow(win, using_xembed);
 
 #ifdef DEBUG
     cerr<<"SystemTray::addClient(Window): 0x"<<hex<<win<<dec<<endl;
 #endif // DEBUG
-    if (m_clients.empty())
-        show();
 
     m_clients.push_back(traywin);
     FbTk::EventManager::instance()->add(*this, win);
-    FbTk::EventManager::instance()->addParent(*this, window());
-    XChangeSaveSet(FbTk::App::instance()->display(), win, SetModeInsert);
     traywin->reparent(m_window, 0, 0);
-    traywin->show();
+    traywin->addToSaveSet();
 
+    if (using_xembed) {
+        static Atom xembed_atom = XInternAtom(disp, "_XEMBED", False);
 
-#ifdef DEBUG
-    cerr<<"SystemTray: number of clients = "<<m_clients.size()<<endl;
-#endif // DEBUG
-    rearrangeClients();
+#define XEMBED_EMBEDDED_NOTIFY		0
+        // send embedded message
+        XEvent ce;
+        ce.xclient.type = ClientMessage;
+        ce.xclient.message_type = xembed_atom;
+        ce.xclient.display = disp;
+        ce.xclient.window = win;
+        ce.xclient.format = 32;
+        ce.xclient.data.l[0] = CurrentTime; // timestamp
+        ce.xclient.data.l[1] = XEMBED_EMBEDDED_NOTIFY;
+        ce.xclient.data.l[2] = 0l; // The protocol version we support
+        ce.xclient.data.l[3] = m_window.window(); // the window owning the selection
+        ce.xclient.data.l[4] = 0l; // unused
+
+        XSendEvent(disp, win, false, NoEventMask, &ce);
+    }
+
+    if (traywin->getMappedDefault())
+        showClient(traywin);
 }
 
-void SystemTray::removeClient(Window win) {
+void SystemTray::removeClient(Window win, bool destroyed) {
     ClientList::iterator tray_it = findClient(win);
     if (tray_it == m_clients.end())
         return;
@@ -313,15 +399,14 @@ void SystemTray::removeClient(Window win) {
 #ifdef DEBUG
     cerr<<__FILE__<<"(SystemTray::removeClient(Window)): 0x"<<hex<<win<<dec<<endl;
 #endif // DEBUG
-    FbTk::FbWindow *traywin = *tray_it;
+    TrayWindow *traywin = *tray_it;
     m_clients.erase(tray_it);
-    delete traywin;
-    resize(width(), height());
-    rearrangeClients();
-    if (m_clients.empty()) {
-        // so we send notify signal to parent
-        resizeSig().notify();
+    if (!destroyed) {
+        traywin->setEventMask(NoEventMask);
+        traywin->removeFromSaveSet();
     }
+    hideClient(traywin, destroyed);
+    delete traywin;
 }
 
 void SystemTray::exposeEvent(XExposeEvent &event) {
@@ -330,12 +415,16 @@ void SystemTray::exposeEvent(XExposeEvent &event) {
 
 void SystemTray::handleEvent(XEvent &event) {
     if (event.type == DestroyNotify) {
-        removeClient(event.xdestroywindow.window);
+        removeClient(event.xdestroywindow.window, true);
+    } else if (event.type == ReparentNotify && event.xreparent.parent != m_window.window()) {
+        removeClient(event.xreparent.window, false);
     } else if (event.type == UnmapNotify && event.xany.send_event) {
         // we ignore server-generated events, which can occur
         // on restart. The ICCCM says that a client must send
         // a synthetic event for the withdrawn state
-        removeClient(event.xunmap.window);
+        ClientList::iterator it = findClient(event.xunmap.window);
+        if (it != m_clients.end())
+            hideClient(*it);
     } else if (event.type == ConfigureNotify) {
         // we got configurenotify from an client
         // check and see if we need to update it's size
@@ -347,49 +436,96 @@ void SystemTray::handleEvent(XEvent &event) {
                 static_cast<unsigned int>(event.xconfigure.height) != (*it)->height()) {
                 // the position might differ so we update from our local
                 // copy of position
-                XMoveResizeWindow(FbTk::App::instance()->display(), (*it)->window(), 
+                XMoveResizeWindow(FbTk::App::instance()->display(), (*it)->window(),
                                   (*it)->x(), (*it)->y(),
                                   (*it)->width(), (*it)->height());
 
                 // this was why gaim wasn't centring the icon
                 (*it)->sendConfigureNotify(0, 0, (*it)->width(), (*it)->height());
+                // so toolbar know that we changed size
+                // done inside this loop, because otherwise we can get into nasty looping
+                resizeSig().notify();
             }
-            // so toolbar know that we changed size
-            resizeSig().notify();
         }
 
-    }
+    } else if (event.type == PropertyNotify) {
+        ClientList::iterator it = findClient(event.xproperty.window);
+        if (it != m_clients.end()) {
+            if (event.xproperty.atom == getXEmbedInfoAtom()) {
+                if ((*it)->getMappedDefault())
+                    showClient(*it);
+                else
+                    hideClient(*it);
+            }
+        }
+    } 
+
 }
 
 void SystemTray::rearrangeClients() {
-    const unsigned int h = height();
+    unsigned int w_rot0 = width(), h_rot0 = height();
     const unsigned int bw = m_theme.border().width();
-    int final_size = m_clients.size()*h + bw;
-    resize(final_size, h);
+    FbTk::translateSize(orientation(), w_rot0, h_rot0);
+    unsigned int trayw = m_num_visible_clients*h_rot0 + bw, trayh = h_rot0;
+    FbTk::translateSize(orientation(), trayw, trayh);
+    resize(trayw, trayh);
     update(0);
 
     // move and resize clients
     ClientList::iterator client_it = m_clients.begin();
     ClientList::iterator client_it_end = m_clients.end();
     int next_x = bw;
-    for (; client_it != client_it_end;
-         ++client_it, next_x += h+bw) {
-        (*client_it)->moveResize(next_x, bw, h, h);
-        (*client_it)->sendConfigureNotify(next_x, bw, h, h);
-    }
+    for (; client_it != client_it_end; ++client_it) {
+        if (!(*client_it)->isVisible())
+            continue;
+        int x = next_x, y = bw;
+        next_x += h_rot0+bw;
+        translateCoords(orientation(), x, y, w_rot0, h_rot0);
+        translatePosition(orientation(), x, y, h_rot0, h_rot0, 0);
 
-    client_it = m_clients.begin();
+        (*client_it)->moveResize(x, y, h_rot0, h_rot0);
+        (*client_it)->sendConfigureNotify(x, y, h_rot0, h_rot0);
+    }
 }
 
 void SystemTray::removeAllClients() {
     BScreen *screen = Fluxbox::instance()->findScreen(window().screenNumber());
     while (!m_clients.empty()) {
+        TrayWindow * traywin = m_clients.back();
+        traywin->setEventMask(NoEventMask);
+
+        if (traywin->isXEmbedded())
+            traywin->hide();
+
         if (screen)
-            m_clients.back()->reparent(screen->rootWindow(), 0, 0);
-        m_clients.back()->hide();
-        delete m_clients.back();
+            traywin->reparent(screen->rootWindow(), 0, 0, false);
+        traywin->removeFromSaveSet();
+        delete traywin;
         m_clients.pop_back();
     }
+    m_num_visible_clients = 0;
+}
+
+void SystemTray::hideClient(TrayWindow *traywin, bool destroyed) {
+    if (!traywin || !traywin->isVisible())
+        return;
+
+    if (!destroyed)
+        traywin->hide();
+    m_num_visible_clients--;
+    rearrangeClients();
+}
+
+void SystemTray::showClient(TrayWindow *traywin) {
+    if (!traywin || traywin->isVisible())
+        return;
+
+    if (!m_num_visible_clients)
+        show();
+
+    traywin->show();
+    m_num_visible_clients++;
+    rearrangeClients();
 }
 
 void SystemTray::update(FbTk::Subject* subject) {
@@ -401,24 +537,25 @@ void SystemTray::update(FbTk::Subject* subject) {
         if(m_pixmap)
             m_screen.imageControl().removeImage(m_pixmap);
         m_pixmap = m_screen.imageControl().renderImage(width(), height(),
-                                                       m_theme.texture());
+                                                       m_theme.texture(), orientation());
         m_window.setBackgroundPixmap(m_pixmap);
     }
 
-    // "themereconfigure"
-    if (subject) {
-        ClientList::iterator client_it = m_clients.begin();
-        ClientList::iterator client_it_end = m_clients.end();
-        int next_x = 0;
-        const unsigned int h = height();
-        const unsigned int b = m_theme.border().width();
-        for (; client_it != client_it_end;
-             ++client_it, next_x += h - 2 * b) {
+    ClientList::iterator client_it = m_clients.begin();
+    ClientList::iterator client_it_end = m_clients.end();
+    for (; client_it != client_it_end; ++client_it) {
 
-            // maybe not the best solution (yet), force a refresh of the
-            // background of the client
-            (*client_it)->hide();
-            (*client_it)->show();
-        }
+        // maybe not the best solution (yet), force a refresh of the
+        // background of the client
+        if (!(*client_it)->isVisible())
+            continue;
+        (*client_it)->hide();
+        (*client_it)->show();
     }
+
+}
+
+Atom SystemTray::getXEmbedInfoAtom() {
+static Atom theatom =  XInternAtom(Fluxbox::instance()->display(), "_XEMBED_INFO", False);
+return theatom;
 }
