@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WinButtonTheme.cc 4199 2006-02-16 06:53:05Z mathias $
+// $Id: WinButtonTheme.cc 4954 2007-07-01 17:59:57Z markt $
 
 #include "WinButtonTheme.hh"
 
@@ -29,7 +29,7 @@
 
 #include "FbWinFrameTheme.hh"
 
-WinButtonTheme::WinButtonTheme(int screen_num, const FbWinFrameTheme &frame_theme):
+WinButtonTheme::WinButtonTheme(int screen_num, FbWinFrameTheme &frame_theme):
     FbTk::Theme(screen_num),
     m_close_pm(*this, "window.close.pixmap", "Window.Close.Pixmap"),
     m_close_unfocus_pm(*this, "window.close.unfocus.pixmap", "Window.Close.Unfocus.Pixmap"),
@@ -44,7 +44,7 @@ WinButtonTheme::WinButtonTheme(int screen_num, const FbWinFrameTheme &frame_them
     m_shade_unfocus_pm(*this, "window.shade.unfocus.pixmap", "Window.Shade.Unfocus.Pixmap"),
     m_shade_pressed_pm(*this, "window.shade.pressed.pixmap", "Window.Shade.Pressed.Pixmap"),
     m_unshade_pm(*this, "window.unshade.pixmap", "Window.Unshade.Pixmap"),
-    m_unshade_unfocus_pm(*this, "window.unshade.unfocus.pixmap", "Window.Unhade.Unfocus.Pixmap"),
+    m_unshade_unfocus_pm(*this, "window.unshade.unfocus.pixmap", "Window.Unshade.Unfocus.Pixmap"),
     m_unshade_pressed_pm(*this, "window.unshade.pressed.pixmap", "Window.Unshade.Pressed.Pixmap"),
     m_menuicon_pm(*this, "window.menuicon.pixmap", "Window.MenuIcon.Pixmap"),
     m_menuicon_unfocus_pm(*this, "window.menuicon.unfocus.pixmap", "Window.MenuIcon.Unfocus.Pixmap"),
@@ -67,7 +67,8 @@ WinButtonTheme::~WinButtonTheme() {
 void WinButtonTheme::reconfigTheme() {
     // rescale the pixmaps to match frame theme height
 
-    unsigned int size = m_frame_theme.titleHeight();
+    unsigned int size = m_frame_theme.titleHeight()
+                        - 2 * m_frame_theme.bevelWidth();
     if (m_frame_theme.titleHeight() == 0) {
         // calculate height from font and border width to scale pixmaps
         size = m_frame_theme.font().height() + 2;
