@@ -1,5 +1,5 @@
 // WinClient.cc for Fluxbox - an X11 Window manager
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,7 +19,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WinClient.cc 3967 2005-04-29 02:52:36Z fluxgen $
+// $Id: WinClient.cc 4970 2007-07-03 21:08:07Z markt $
 
 #include "WinClient.hh"
 
@@ -31,7 +31,6 @@
 #include "Xutil.hh"
 
 #include "EventManager.hh"
-#include "FbTk/I18n.hh"
 #include "FbTk/MultLayers.hh"
 
 #include <iostream>
@@ -44,9 +43,22 @@
 #else
   #include <assert.h>
 #endif
+#ifdef HAVE_CSTRING
+  #include <cstring>
+#else
+  #include <string.h>
+#endif
 
-using namespace std;
+using std::string;
+using std::list;
+using std::mem_fun;
 
+#ifdef DEBUG
+using std::cerr;
+using std::endl;
+using std::hex;
+using std::dec;
+#endif // DEBUG
 
 WinClient::TransientWaitMap WinClient::s_transient_wait;
 
@@ -64,16 +76,20 @@ WinClient::WinClient(Window win, BScreen &screen, FluxboxWindow *fbwin):FbTk::Fb
                      normal_hint_flags(0),
                      wm_hint_flags(0),
                      m_win(fbwin),
-                     m_modal(0),
+                     m_modal_count(0),
+                     m_modal(false),
                      send_focus_message(false),
                      send_close_message(false),
                      m_win_gravity(0),
                      m_title(""), m_icon_title(""),
                      m_class_name(""), m_instance_name(""),
+                     m_title_override(false),
+                     m_icon_title_override(false),
                      m_blackbox_hint(0),
                      m_mwm_hint(0),
                      m_focus_mode(F_PASSIVE),
-                     m_diesig(*this), m_screen(screen), 
+                     m_diesig(*this), m_focussig(*this),
+                     m_screen(screen),
                      m_strut(0) {
     updateWMProtocols();
     updateBlackboxHints();
@@ -84,7 +100,7 @@ WinClient::WinClient(Window win, BScreen &screen, FluxboxWindow *fbwin):FbTk::Fb
     updateTitle();
     updateIconTitle();
     Fluxbox::instance()->saveWindowSearch(win, this);
-    if (window_group != None) 
+    if (window_group != None)
         Fluxbox::instance()->saveGroupSearch(window_group, this);
 
     // search for this in transient waiting list
@@ -97,6 +113,11 @@ WinClient::WinClient(Window win, BScreen &screen, FluxboxWindow *fbwin):FbTk::Fb
         // clear transient waiting list for this window
         s_transient_wait.erase(win);
     }
+
+    // also check if this window is a transient
+    // this needs to be done before creating an fbwindow, so this doesn't get
+    // tabbed using the apps file
+    updateTransientInfo();
 }
 
 WinClient::~WinClient() {
@@ -108,6 +129,22 @@ WinClient::~WinClient() {
 
     clearStrut();
 
+    //
+    // clear transients and transient_for
+    //
+    if (transient_for != 0) {
+        assert(transient_for != this);
+        transient_for->transientList().remove(this);
+        if (m_modal)
+            transient_for->removeModal();
+        transient_for = 0;
+    }
+
+    while (!transients.empty()) {
+        transients.back()->transient_for = 0;
+        transients.pop_back();
+    }
+
     if (m_win != 0)
         m_win->removeClient(*this);
 
@@ -116,20 +153,6 @@ WinClient::~WinClient() {
 
     Fluxbox *fluxbox = Fluxbox::instance();
 
-
-    //
-    // clear transients and transient_for
-    //
-    if (transient_for != 0) {
-        assert(transient_for != this);
-        transient_for->transientList().remove(this);
-        transient_for = 0;
-    }
-    
-    while (!transients.empty()) {
-        transients.back()->transient_for = 0;
-        transients.pop_back();
-    }
     // This fixes issue 1 (see WinClient.hh):
     // If transients die before the transient_for is created
     removeTransientFromWaitingList();
@@ -156,8 +179,8 @@ WinClient::~WinClient() {
 }
 
 bool WinClient::acceptsFocus() const {
-    return (m_focus_mode == F_LOCALLYACTIVE || 
-            m_focus_mode == F_PASSIVE || 
+    return (m_focus_mode == F_LOCALLYACTIVE ||
+            m_focus_mode == F_PASSIVE ||
             m_focus_mode == F_GLOBALLYACTIVE && send_focus_message);
 }
 
@@ -217,14 +240,14 @@ bool WinClient::getWMName(XTextProperty &textprop) const {
 }
 
 bool WinClient::getWMIconName(XTextProperty &textprop) const {
-    return XGetWMName(display(), window(), &textprop);
+    return XGetWMIconName(display(), window(), &textprop);
 }
 
-const std::string &WinClient::getWMClassName() const {
+const string &WinClient::getWMClassName() const {
     return m_instance_name;
 }
 
-const std::string &WinClient::getWMClassClass() const {
+const string &WinClient::getWMClassClass() const {
     return m_class_name;
 }
 
@@ -234,15 +257,15 @@ void WinClient::updateWMClassHint() {
 #ifdef DEBUG
         cerr<<"WinClient: Failed to read class hint!"<<endl;
 #endif //DEBUG
-    } else {        
+    } else {
 
         if (ch.res_name != 0) {
             m_instance_name = const_cast<char *>(ch.res_name);
             XFree(ch.res_name);
             ch.res_name = 0;
-        } else 
+        } else
             m_instance_name = "";
-        
+
         if (ch.res_class != 0) {
             m_class_name = const_cast<char *>(ch.res_class);
             XFree(ch.res_class);
@@ -253,18 +276,13 @@ void WinClient::updateWMClassHint() {
 }
 
 void WinClient::updateTransientInfo() {
-#ifdef DEBUG
-    cerr<<__FUNCTION__<<": m_win = "<<m_win<<endl;
-#endif // DEBUG
-    if (m_win == 0)
-        return;
-
-
     // remove this from parent
     if (transientFor() != 0) {
         transientFor()->transientList().remove(this);
+        if (m_modal)
+            transientFor()->removeModal();
     }
-    
+
     transient_for = 0;
     // determine if this is a transient window
     Window win = 0;
@@ -282,24 +300,24 @@ void WinClient::updateTransientInfo() {
 #endif // DEBUG
         return;
     }
-	
-    if (win != None && m_win->screen().rootWindow() == win) {
+
+    if (win != None && screen().rootWindow() == win) {
         // transient for root window... =  transient for group
         // I don't think we are group-aware yet
-        return; 
+        return;
     }
 
 
     transient_for = Fluxbox::instance()->searchWindow(win);
     // if we did not find a transient WinClient but still
-    // have a transient X window, then we have to put the 
+    // have a transient X window, then we have to put the
     // X transient_for window in a waiting list and update this clients transient
     // list later when the transient_for has a Winclient
     if (!transient_for) {
         // We might also already waiting for an old transient_for;
-        // 
+        //
         // this call fixes issue 2:
-        // If transients changes to new transient_for before the old transient_for is created. 
+        // If transients changes to new transient_for before the old transient_for is created.
         // (see comment in WinClient.hh)
         //
         removeTransientFromWaitingList();
@@ -324,29 +342,46 @@ void WinClient::updateTransientInfo() {
         // we need to add ourself to the right client in
         // the transientFor() window so we search client
         transient_for->transientList().push_back(this);
-
-        if (transientFor()->fbwindow() && transientFor()->fbwindow()->isStuck())
-            m_win->stick();
+        if (m_modal)
+            transient_for->addModal();
     }
 
 }
 
 
 void WinClient::updateTitle() {
-    // why 512? very very long wmnames seem to either 
+    // why 512? very very long wmnames seem to either
     // crash fluxbox or to make it have high cpuload
-    // see also: 
+    // see also:
     //    http://www.securityfocus.com/archive/1/382398/2004-11-24/2004-11-30/2
-    // 
+    //
     // TODO: - find out why this mostly happens when using xft-fonts
     //       - why other windowmanagers (pekwm/pwm3/openbox etc) are
     //         also influenced
     //
     // the limitation to 512 chars only avoids running in that trap
+    if (m_title_override)
+        return;
+
     m_title = string(Xutil::getWMName(window()), 0, 512);
 }
 
+void WinClient::setTitle(FbTk::FbString &title) {
+    m_title = title;
+    m_title_override = true;
+    if (m_win)
+        m_win->updateTitleFromClient(*this);
+}
+
+void WinClient::setIconTitle(FbTk::FbString &icon_title) {
+    m_icon_title = icon_title;
+    m_icon_title_override = true;
+}
+
 void WinClient::updateIconTitle() {
+    if (m_icon_title_override)
+        return;
+
     XTextProperty text_prop;
     char **list = 0;
     int num = 0;
@@ -472,7 +507,7 @@ void WinClient::updateWMHints() {
                     m_focus_mode = F_NOINPUT;
             }
         } else // InputHint not present: ignoring send_focus_message and assuming F_PASSIVE
-            m_focus_mode = F_PASSIVE; 
+            m_focus_mode = F_PASSIVE;
 
         if (wmhint->flags & StateHint)
             initial_state = wmhint->initial_state;
@@ -485,6 +520,25 @@ void WinClient::updateWMHints() {
         } else
             window_group = None;
 
+        if ((bool)(wmhint->flags & IconPixmapHint) && wmhint->icon_pixmap != 0)
+            m_icon_pixmap.copy(wmhint->icon_pixmap, 0, 0);
+        else
+            m_icon_pixmap = 0;
+
+        if ((bool)(wmhint->flags & IconMaskHint) && wmhint->icon_mask != 0)
+            m_icon_mask.copy(wmhint->icon_mask, 0, 0);
+        else
+            m_icon_mask = 0;
+
+        if (m_win) {
+            if (wmhint->flags & XUrgencyHint) {
+                Fluxbox::instance()->attentionHandler().addAttention(*this);
+            } else {
+                Fluxbox::instance()->attentionHandler().
+                    update(&m_focussig);
+            }
+        }
+
         XFree(wmhint);
     }
 }
@@ -584,8 +638,10 @@ Window WinClient::getGroupLeftWindow() const {
 
 
 void WinClient::setGroupLeftWindow(Window win) {
+    if (m_screen.isShuttingdown())
+        return;
     Atom group_left_hint = XInternAtom(display(), "_FLUXBOX_GROUP_LEFT", False);
-    changeProperty(group_left_hint, XA_WINDOW, 32, 
+    changeProperty(group_left_hint, XA_WINDOW, 32,
                    PropModeReplace, (unsigned char *) &win, 1);
 }
 
@@ -614,16 +670,21 @@ bool WinClient::hasGroupLeftWindow() const {
     return false;
 }
 
-void WinClient::addModal() {
-    ++m_modal;
-    if (transient_for)
-        transient_for->addModal();
-}
+void WinClient::setStateModal(bool state) {
+    if (state == m_modal)
+        return;
 
-void WinClient::removeModal() {
-    --m_modal;
-    if (transient_for)
-        transient_for->removeModal();
+    m_modal = state;
+    if (transient_for) {
+        if (state)
+            transient_for->addModal();
+        else
+            transient_for->removeModal();
+    }
+
+    // TODO: we're not implementing the following part of EWMH spec:
+    // "if WM_TRANSIENT_FOR is not set or set to the root window the dialog is
+    //  modal for its window group."
 }
 
 bool WinClient::validateClient() const {
@@ -631,7 +692,7 @@ bool WinClient::validateClient() const {
 
     XEvent e;
     if (( XCheckTypedWindowEvent(display(), window(), DestroyNotify, &e) ||
-          XCheckTypedWindowEvent(display(), window(), UnmapNotify, &e)) 
+          XCheckTypedWindowEvent(display(), window(), UnmapNotify, &e))
         && XPutBackEvent(display(), &e)) {
         Fluxbox::instance()->ungrab();
         return false;
@@ -640,7 +701,7 @@ bool WinClient::validateClient() const {
     return true;
 }
 
-void WinClient::setStrut(Strut *strut) {    
+void WinClient::setStrut(Strut *strut) {
     clearStrut();
     m_strut = strut;
 }
@@ -695,14 +756,14 @@ void WinClient::updateWMProtocols() {
    Note that its slightly simplified in that only the
    line gradient is given - this is because for aspect
    ratios, we always have the line going through the origin
-   
+
    * Based on this formula:
    http://astronomy.swin.edu.au/~pbourke/geometry/pointline/
 
    Note that a gradient from origin goes through ( grad , 1 )
  */
 
-void closestPointToLine(double &ret_x, double &ret_y, 
+void closestPointToLine(double &ret_x, double &ret_y,
                         double point_x, double point_y,
                         double gradient) {
     double u = (point_x * gradient + point_y) /
@@ -722,13 +783,14 @@ void closestPointToLine(double &ret_x, double &ret_y,
  *
  * See ICCCM section 4.1.2.3
  */
-void WinClient::applySizeHints(int &width, int &height, 
-                               int *display_width, int *display_height) {
+void WinClient::applySizeHints(int &width, int &height,
+                               int *display_width, int *display_height,
+                               bool maximizing) {
 
     int i = width, j = height;
 
     // Check minimum size
-    if (width < 0 || width < static_cast<signed>(min_width)) 
+    if (width < 0 || width < static_cast<signed>(min_width))
         width = min_width;
 
     if (height < 0 || height < static_cast<signed>(min_height))
@@ -742,8 +804,8 @@ void WinClient::applySizeHints(int &width, int &height,
         height = max_height;
 
     // we apply aspect ratios before incrementals
-    // Too difficult to exactly satisfy both incremental+aspect 
-    // in most situations 
+    // Too difficult to exactly satisfy both incremental+aspect
+    // in most situations
     // (they really shouldn't happen at the same time anyway).
 
     /* aspect ratios are applied exclusive to the base_width
@@ -761,6 +823,10 @@ void WinClient::applySizeHints(int &width, int &height,
      *  Consider that the aspect ratio is a line, and the current
      *  w/h is a point, so we're just using the formula for
      *  shortest distance from a point to a line!
+     *
+     * When maximizing, we must not increase any of the sizes, because we
+     * would end up with the window partly off a screen, so a simpler formula
+     * is used in that case.
      */
 
     if (min_aspect_y > 0 && max_aspect_y > 0 &&
@@ -768,11 +834,11 @@ void WinClient::applySizeHints(int &width, int &height,
         double widthd = static_cast<double>(width - base_width);
         double heightd = static_cast<double>(height - base_height);
 
-        double min = static_cast<double>(min_aspect_x) / 
-            static_cast<double>(min_aspect_y); 
+        double min = static_cast<double>(min_aspect_x) /
+            static_cast<double>(min_aspect_y);
 
-        double max = static_cast<double>(max_aspect_x) / 
-            static_cast<double>(max_aspect_y); 
+        double max = static_cast<double>(max_aspect_x) /
+            static_cast<double>(max_aspect_y);
 
         double actual = widthd / heightd;
 
@@ -780,10 +846,16 @@ void WinClient::applySizeHints(int &width, int &height,
             bool changed = false;
             if (actual < min) {
                 changed = true;
-                closestPointToLine(widthd, heightd, widthd, heightd, min);
+                if (maximizing)
+                    heightd = widthd / min;
+                else
+                    closestPointToLine(widthd, heightd, widthd, heightd, min);
             } else if (actual > max) {
                 changed = true;
-                closestPointToLine(widthd, heightd, widthd, heightd, max);
+                if (maximizing)
+                    widthd = heightd * max;
+                else
+                    closestPointToLine(widthd, heightd, widthd, heightd, max);
             }
 
             if (changed) {
@@ -795,11 +867,15 @@ void WinClient::applySizeHints(int &width, int &height,
 
     // enforce incremental size limits, wrt base size
     // only calculate this if we really need to
-    i = (width - base_width) / width_inc;
-    width = i*width_inc + base_width;
+    i = (width - static_cast<signed>(base_width)) /
+        static_cast<signed>(width_inc);
+    width = i*static_cast<signed>(width_inc) +
+        static_cast<signed>(base_width);
 
-    j = (height - base_height) / height_inc;
-    height = j*height_inc + base_height;
+    j = (height - static_cast<signed>(base_height)) /
+        static_cast<signed>(height_inc);
+    height = j*static_cast<signed>(height_inc) +
+        static_cast<signed>(base_height);
 
     if (display_width)
         *display_width = i;
@@ -808,13 +884,38 @@ void WinClient::applySizeHints(int &width, int &height,
         *display_height = j;
 }
 
+// check if the given width and height satisfy the size hints
+bool WinClient::checkSizeHints(unsigned int width, unsigned int height) {
+    if (width < min_width || height < min_height)
+        return false;
+
+    if (width > max_width || height > max_height)
+        return false;
+
+    if ((width - base_width) % width_inc != 0)
+        return false;
+
+    if ((height - base_height) % height_inc != 0)
+        return false;
+
+    double ratio = (double)width / (double)height;
+
+    if (min_aspect_y > 0 && (double)min_aspect_x / (double)min_aspect_y > ratio)
+        return false;
+
+    if (max_aspect_y > 0 && (double)max_aspect_x / (double)max_aspect_y < ratio)
+        return false;
+
+    return true;
+}
+
 void WinClient::removeTransientFromWaitingList() {
 
     // holds the windows that dont have empty
     // transient waiting list
-    std::list<Window> remove_list;
+    list<Window> remove_list;
 
-    // The worst case complexity is huge, but since we usually do not (virtualy never) 
+    // The worst case complexity is huge, but since we usually do not (virtualy never)
     // have a large transient waiting list the time spent here is neglectable
     TransientWaitMap::iterator t_it = s_transient_wait.begin();
     TransientWaitMap::iterator t_it_end = s_transient_wait.end();
@@ -827,8 +928,8 @@ void WinClient::removeTransientFromWaitingList() {
     }
 
     // erase empty waiting lists
-    std::list<Window>::iterator it = remove_list.begin();
-    std::list<Window>::iterator it_end = remove_list.end();
+    list<Window>::iterator it = remove_list.begin();
+    list<Window>::iterator it_end = remove_list.end();
     for (; it != it_end; ++it)
         s_transient_wait.erase(*it);
 }
