@@ -1,5 +1,5 @@
 // WorkspaceCmd.cc for Fluxbox - an X11 Window manager
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //                and Simon Bowden (rathnor at users.sourceforge.net)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
@@ -20,7 +20,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: WorkspaceCmd.cc 3991 2005-05-06 09:22:53Z mathias $
+// $Id: WorkspaceCmd.cc 4978 2007-07-07 04:54:34Z markt $
 
 #include "WorkspaceCmd.hh"
 
@@ -29,6 +29,7 @@
 #include "Screen.hh"
 #include "fluxbox.hh"
 #include "WinClient.hh"
+#include "FocusControl.hh"
 
 #include "FbTk/KeyUtil.hh"
 
@@ -39,52 +40,17 @@
 #endif
 #include <algorithm>
 #include <functional>
-#include <iostream>
 
 void NextWindowCmd::execute() {
     BScreen *screen = Fluxbox::instance()->keyScreen();
-    if (screen != 0) {
-        Fluxbox *fb = Fluxbox::instance();
-        // special case for commands from key events
-        if (fb->lastEvent().type == KeyPress) {
-            unsigned int mods = FbTk::KeyUtil::instance().cleanMods(fb->lastEvent().xkey.state);
-            mods = FbTk::KeyUtil::instance().isolateModifierMask(mods);
-            if (mods == 0) // can't stacked cycle unless there is a mod to grab
-                screen->nextFocus(m_option | BScreen::CYCLELINEAR);
-            else {
-                // if stacked cycling, then set a watch for 
-                // the release of exactly these modifiers
-                if (!fb->watchingScreen() && 
-                    !(m_option & BScreen::CYCLELINEAR))
-                    Fluxbox::instance()->watchKeyRelease(*screen, mods);
-                screen->nextFocus(m_option);
-            }
-        } else
-            screen->nextFocus(m_option);
-    }
+    if (screen != 0)
+        screen->cycleFocus(m_option, false);
 }
 
 void PrevWindowCmd::execute() {
     BScreen *screen = Fluxbox::instance()->keyScreen();
-    if (screen != 0) {
-        Fluxbox *fb = Fluxbox::instance();
-        // special case for commands from key events
-        if (fb->lastEvent().type == KeyPress) {
-            unsigned int mods = FbTk::KeyUtil::instance().cleanMods(fb->lastEvent().xkey.state);
-            mods = FbTk::KeyUtil::instance().isolateModifierMask(mods);
-            if (mods == 0) // can't stacked cycle unless there is a mod to grab
-                screen->prevFocus(m_option | BScreen::CYCLELINEAR);
-            else {
-                // if stacked cycling, then set a watch for 
-                // the release of exactly these modifiers
-                if (!fb->watchingScreen() 
-                    && !(m_option & BScreen::CYCLELINEAR))
-                    Fluxbox::instance()->watchKeyRelease(*screen, mods);
-                screen->prevFocus(m_option);
-            }
-        } else
-            screen->nextFocus(m_option);
-    }
+    if (screen != 0)
+        screen->cycleFocus(m_option, true);
 }
 
 void DirFocusCmd::execute() {
@@ -92,23 +58,33 @@ void DirFocusCmd::execute() {
     if (screen == 0)
         return;
 
-    WinClient *client = Fluxbox::instance()->getFocusedWindow();
-    if (client == 0 || client->fbwindow() == 0)
-        return;
+    FluxboxWindow *win = FocusControl::focusedFbWindow();
+    if (win)
+        screen->focusControl().dirFocus(*win, m_dir);
+}
 
-    screen->dirFocus(*client->fbwindow(), m_dir);
+void AddWorkspaceCmd::execute() {
+    BScreen *screen = Fluxbox::instance()->mouseScreen();
+    if (screen != 0)
+        screen->addWorkspace();
+}
+
+void RemoveLastWorkspaceCmd::execute() {
+    BScreen *screen = Fluxbox::instance()->mouseScreen();
+    if (screen != 0)
+        screen->removeLastWorkspace();
 }
 
 void NextWorkspaceCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
     if (screen != 0)
-        screen->nextWorkspace();
+        screen->nextWorkspace(m_option == 0 ? 1 : m_option);
 }
 
 void PrevWorkspaceCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
     if (screen != 0)
-        screen->prevWorkspace();
+        screen->prevWorkspace(m_option == 0 ? 1 : m_option);
 }
 
 void LeftWorkspaceCmd::execute() {
@@ -127,8 +103,15 @@ JumpToWorkspaceCmd::JumpToWorkspaceCmd(int workspace_num):m_workspace_num(worksp
 
 void JumpToWorkspaceCmd::execute() {
     BScreen *screen = Fluxbox::instance()->mouseScreen();
-    if (screen != 0 && m_workspace_num >= 0 && m_workspace_num < screen->getNumberOfWorkspaces())
-        screen->changeWorkspaceID(m_workspace_num);
+    if (screen != 0) {
+        int num = screen->numberOfWorkspaces();
+        int actual = m_workspace_num;
+        // we need an extra +1, since it's subtracted in FbCommandFactory
+        if (actual < 0) actual += num+1;
+        if (actual < 0) actual = 0;
+        if (actual >= num) actual = num - 1;
+        screen->changeWorkspaceID(actual);
+    }
 }
 
 
@@ -143,18 +126,18 @@ void ArrangeWindowsCmd::execute() {
         return;
 
     Workspace *space = screen->currentWorkspace();
-    unsigned int win_count = space->windowList().size();
-  
-    if (win_count == 0) 
+    size_t win_count = space->windowList().size();
+
+    if (win_count == 0)
         return;
 
-    // TODO: choice between 
+    // TODO: choice between
     //        -  arrange using all windows on all heads
     //        -  arrange for each head
     //        -  only on current head
     const int head = screen->getCurrHead();
     Workspace::Windows::iterator win;
-    
+
     Workspace::Windows normal_windows;
     Workspace::Windows shaded_windows;
     for(win = space->windowList().begin(); win != space->windowList().end(); win++) {
@@ -172,20 +155,20 @@ void ArrangeWindowsCmd::execute() {
         return;
 
     win_count = normal_windows.size();
-    
+
     const unsigned int max_width = screen->maxRight(head) - screen->maxLeft(head);
     unsigned int max_height = screen->maxBottom(head) - screen->maxTop(head);
 
-	// try to get the same number of rows as columns.
-	unsigned int rows = int(sqrt((float)win_count));  // truncate to lower
-	unsigned int cols = int(0.99 + float(win_count) / float(rows));
-	if (max_width<max_height) {	// rotate
+    // try to get the same number of rows as columns.
+    unsigned int cols = int(sqrt((float)win_count));  // truncate to lower
+    unsigned int rows = int(0.99 + float(win_count) / float(cols));
+    if (max_width<max_height) {    // rotate
         std::swap(cols, rows);
-	}
+    }
 
     unsigned int x_offs = screen->maxLeft(head); // window position offset in x
     unsigned int y_offs = screen->maxTop(head); // window position offset in y
-    unsigned int window = 0; // current window 
+   // unsigned int window = 0; // current window
     const unsigned int cal_width = max_width/cols; // calculated width ratio (width of every window)
     unsigned int i;
     unsigned int j;
@@ -199,7 +182,7 @@ void ArrangeWindowsCmd::execute() {
             (*win)->move(x_offs, y_offs);
         else
             (*win)->move(screen->maxRight(head) - (*win)->frame().width(), y_offs);
-            
+
         y_offs += (*win)->frame().height();
     }
 
@@ -207,18 +190,47 @@ void ArrangeWindowsCmd::execute() {
     // with really little space left for the normal windows? how to handle
     // this?
     if (!shaded_windows.empty())
-        max_height -= i * (*shaded_windows.begin())->frame().height(); 
+        max_height -= i * (*shaded_windows.begin())->frame().height();
 
     const unsigned int cal_height = max_height/rows; // height ratio (height of every window)
     // Resizes and sets windows positions in columns and rows.
     for (i = 0; i < rows; ++i) {
         x_offs = screen->maxLeft(head);
-        for (j = 0; j < cols && window < win_count; ++j, ++window) {
-			if (window!=(win_count-1)) {
-	            normal_windows[window]->moveResize(x_offs, y_offs, cal_width, cal_height);
-			} else { // the last window gets everything that is left.
-	            normal_windows[window]->moveResize(x_offs, y_offs, screen->maxRight(head)-x_offs, cal_height);
-			}
+        for (j = 0; j < cols && normal_windows.size() > 0; ++j) {
+
+
+            int cell_center_x = x_offs + (x_offs + cal_width) / 2;
+            int cell_center_y = y_offs + (y_offs + cal_height) / 2;
+            unsigned int closest_dist = ~0;
+
+            Workspace::Windows::iterator closest = normal_windows.end();
+            for (win = normal_windows.begin(); win != normal_windows.end(); win++) {
+
+                int win_center_x = (*win)->frame().x() + ((*win)->frame().x() + (*win)->frame().width() / 2);
+                int win_center_y = (*win)->frame().y() + ((*win)->frame().y() + (*win)->frame().height() / 2);
+                unsigned int dist = (win_center_x - cell_center_x) * (win_center_x - cell_center_x) + 
+                                    (win_center_y - cell_center_y) * (win_center_y - cell_center_y);
+
+                if (dist < closest_dist) {
+                    closest = win;
+                    closest_dist = dist;
+                }
+            }
+
+            if (normal_windows.size() > 1) {
+                (*closest)->moveResize(x_offs + (*closest)->xOffset(),
+			y_offs + (*closest)->yOffset(),
+			cal_width - (*closest)->widthOffset(),
+			cal_height - (*closest)->heightOffset());
+            } else { // the last window gets everything that is left.
+                (*closest)->moveResize(x_offs + (*closest)->xOffset(),
+			y_offs + (*closest)->yOffset(),
+			screen->maxRight(head) - x_offs - (*closest)->widthOffset(),
+			cal_height - (*closest)->heightOffset());
+            }
+
+            normal_windows.erase(closest);
+
             // next x offset
             x_offs += cal_width;
         }
@@ -237,3 +249,37 @@ void ShowDesktopCmd::execute() {
                   windows.end(),
                   std::mem_fun(&FluxboxWindow::iconify));
 }
+
+void MinimizeLayerCmd::execute() {
+    FluxboxWindow *win = FocusControl::focusedFbWindow();
+    if (!win)
+        return;
+
+    Workspace::Windows windows(win->screen().currentWorkspace()->windowList());
+    Workspace::Windows::iterator it = windows.begin(), it_end = windows.end();
+    for (; it != it_end; ++it) {
+        if (win->layerNum() == (*it)->layerNum())
+            (*it)->iconify();
+    }
+}
+
+void CloseAllWindowsCmd::execute() {
+    BScreen *screen = Fluxbox::instance()->mouseScreen();
+    if (screen == 0)
+        return;
+  
+    BScreen::Workspaces::iterator workspace_it = screen->getWorkspacesList().begin();
+    BScreen::Workspaces::iterator workspace_it_end = screen->getWorkspacesList().end();
+       for (; workspace_it != workspace_it_end; ++workspace_it) {
+            Workspace::Windows windows((*workspace_it)->windowList());
+            std::for_each(windows.begin(),
+            windows.end(),
+            std::mem_fun(&FluxboxWindow::close));
+           }
+    BScreen::Icons::iterator icon_it = screen->iconList().begin();
+    BScreen::Icons::iterator icon_it_end = screen->iconList().end();
+	    for (; icon_it != icon_it_end; ++icon_it ) {
+	         (*icon_it)->close();
+	      }
+				   
+}
